(function() {
    'use strict';

    angular.module('wemGoalsApp', ['wemApp', 'wemMostVisitedPagesApp']);
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').run(wemGoalsAppRun);

    wemGoalsAppRun.$inject = ['$rootScope', '$location', 'jcrService', 'modalService', 'i18nService'];

    function wemGoalsAppRun($rootScope, $location, jcrService, modalService, i18nService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.getGoalViewTemplate = function (goal) {
            if (goal && goal.metadata && goal.metadata.tags && goal.metadata.tags.length > 0) {
                return ManagersContext.wemAppPath + "/wem-goals/goal-" + goal.metadata.tags[0] + "-view.html";
            }
        };

        $rootScope.getGoalLinksView = function () {
            return ManagersContext.wemAppPath + "/wem-goals/goal-widget-links.directive.html";
        };

        var history = [];

        $rootScope.$on('$routeChangeSuccess', function () {
            history.push($location.$$path);
        });

        $rootScope.back = function () {
            var prevUrl = history.length > 1 ? history.splice(-2)[0] : "/";
            $location.path(prevUrl);
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalCreateController', GoalCreateController);

    GoalCreateController.$inject = ['$scope', '$location', '$route', '$timeout', '_', 'i18nService', 'unomiGoalService', 'unomiQueryService', 'notificationService', 'unomiListService', 'unomiRulesService', 'loadingSpinnerService', 'goal', 'modalService'];

    function GoalCreateController($scope, $location, $route, $timeout, _, i18nService, unomiGoalService, unomiQueryService, notificationService, unomiListService, unomiRulesService, loadingSpinnerService, goal, modalService) {
        _init();

        $scope.getEditTemplate = function () {
            if ($scope.goal.type) {
                $scope.template = ManagersContext.wemAppPath + "/wem-goals/goal-" + $scope.goal.type.id + "-edit.html";
            }

            return $scope.template
        };

        $scope.saveGoal = function (createForm) {
            // validate form
            if (createForm.$invalid) {
                var parsedErrors = ManagersUtils.parseErrorsInForm(createForm);
                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var goalMetadata = $scope.goal.metadata;
            var type = $scope.goal.type;
            if ($scope.newGoal) {
                // check that we don't already have a goal of this type and name
                unomiQueryService.createWithCallbackIfDoesNotExist('goal', ManagersContext.dmfScope, goalMetadata.name, type.id,
                    "serverSettings.manageGoals.modal.newGoal.alreadyExisting", type.name + "|" + goalMetadata.name, function () {
                        // perform save from specific callback (as defined in specific goal type controllers) to add what is needed for that particular goal type
                        $scope.callback.save(function () {
                            loadingSpinnerService.show();
                            goalMetadata.tags = [type.id];
                            delete $scope.goal.type;
                            var lists = $scope.goal.lists;
                            delete $scope.goal.lists;

                            unomiGoalService.setGoal($scope.goal).then(function () {
                                unomiRulesService.setRule(unomiGoalService.localToServerGoalRule($scope.goal, lists));
                                // Add timeout before reloading to make sure Unomi has time to create the new goal
                                $timeout(function () {
                                    loadingSpinnerService.hide();
                                    $location.path("/goals");
                                }, 1000);
                            });
                        });
                    });
            } else {

                var saveGoalIfConfirm = function () {
                    $scope.callback.save(function () {
                        loadingSpinnerService.show();
                        goalMetadata.tags = [type.id];
                        delete $scope.goal.type;
                        delete $scope.goal.lists;

                        unomiGoalService.setGoal($scope.goal).then(function () {
                            $timeout(function () {
                                loadingSpinnerService.hide();
                                $location.path("/goal-report/" + $scope.goal.itemId);
                            }, 1000);
                        });
                    })
                };

                modalService.newAlertModal({
                    title: i18nService.message('serverSettings.manageGoals.title.editConfirm'),
                    bodyMessage: i18nService.message('serverSettings.manageGoals.message.editConfirm')
                }, saveGoalIfConfirm);
            }
        };

        $scope.resetGoal = function () {
            if($route) {
                $route.reload();
            }
        };

        // Private functions under this line
        function _init() {
            unomiListService.query().then(function (response) {
                $scope.userLists = response.data.list;
            });

            if (!$scope.goalTags) {
                var tags = _.map(ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService), function (tag, key) {
                    return tag;
                });
                $scope.goalTags = angular.copy(tags);
            }
            // this callback object will be inherited in the specific edit goal ctrl and complete by this one with a save() method
            // this will allow specific edit ctrl to do specific logic to build the goal
            $scope.callback = {};

            if (goal) {
                $scope.goal = goal;
                $scope.newGoal = false;
                angular.forEach($scope.goalTags, function(goalTag) {
                    if (goalTag.id == goal.metadata.tags[0]) {
                        $scope.goal.type = goalTag;
                    }
                });
            } else {
                $scope.newGoal = true;
                $scope.goal = {
                    metadata: {
                        id: ManagersUtils.generateId(),
                        name: '',
                        description: '',
                        enabled: false,
                        scope: ManagersContext.dmfScope
                    },
                    type: $scope.goalTags[0]
                };
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalDownloadGoalController', GoalDownloadGoalController);

    GoalDownloadGoalController.$inject = ['$scope', 'jcrService'];

    function GoalDownloadGoalController($scope, jcrService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetFile = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.filePath");


        var doSave = function(callback) {
            // construct download events:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                },
                "type": "sessionCreatedEventCondition"
            };
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "filePath": $scope._goal.targetFile
                },
                "type": "downloadEventCondition"
            };

            jcrService.addMixin("live", null, $scope._goal.targetFile, "wemmix:trackDownloadedFile", {}, true).then(function (response) {
                // mixin added, save goal on Apache Unomi
            });

            if (callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalFormGoalController', GoalFormGoalController);

    GoalFormGoalController.$inject = ['$scope', 'validationUtilsService', 'modalService', 'i18nService'];

    function GoalFormGoalController($scope, validationUtilsService, modalService, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        var subConditions = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.subConditions");
        if (subConditions) {
            for (var i = 0; i < subConditions.length; i++) {
                var subCondition = subConditions[i];
                if (subCondition.type == "formEventCondition") {
                    $scope._goal.targetForm = ManagersUtils.getProperty(subCondition, "parameterValues.formId");
                }
                if (subCondition.type == "sourceEventPropertyCondition") {
                    $scope._goal.startPath = ManagersUtils.getProperty(subCondition, "parameterValues.path");
                }
            }
        }

        var doSave = function(callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.startPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.targetEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "formId": $scope._goal.targetForm
                            },
                            "type": "formEventCondition"
                        },
                        {
                            "parameterValues": {
                                "path": $scope._goal.startPath
                            },
                            "type": "sourceEventPropertyCondition"
                        }
                    ]
                }
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalFunnelGoalController', GoalFunnelGoalController);

    GoalFunnelGoalController.$inject = ['$scope'];

    function GoalFunnelGoalController($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetPath = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.pagePath");
        $scope._goal.startPath = ManagersUtils.getProperty($scope.goal, "startEvent.parameterValues.pagePath");

        var doSave = function (callback) {
            // construct funnel events:
            $scope.goal.metadata.enabled = true;
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.targetPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.startEvent = angular.copy($scope.goal.targetEvent);
            $scope.goal.startEvent.parameterValues.pagePath = $scope._goal.startPath;

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalLandingPageGoalController', GoalLandingPageGoalController);

    GoalLandingPageGoalController.$inject = ['$scope', 'i18nService'];

    function GoalLandingPageGoalController($scope, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.numberOfPagesView = {};
        $scope.targetEvents = [
            { id: 'pageView', name: i18nService.message('serverSettings.manageGoals.selectLandingType.pageView') },
            { id: 'formSubmit', name: i18nService.message('serverSettings.manageGoals.selectLandingType.formSubmit') },
            { id: 'numberOfPagesView', name: i18nService.message('serverSettings.manageGoals.selectLandingType.numberOfPagesView') }
        ];

        var conditions = ManagersUtils.getProperty($scope.goal, "startEvent.parameterValues.subConditions");
        if (conditions && conditions.length > 0) {
            conditions.forEach(function (condition) {
                if (condition.type == "pageViewEventCondition") {
                    $scope._goal.startPath = ManagersUtils.getProperty(condition, "parameterValues.pagePath")
                }
            })
        }

        if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "pageViewEventCondition") {
            $scope._goal.landingType = "pageView";
            $scope._goal.targetPath = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.pagePath");
        } else if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "booleanCondition") {
            var subConditions = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.subConditions");
            if (subConditions) {
                for (var i = 0; i < subConditions.length; i++) {
                    var subCondition = subConditions[i];
                    if (subCondition.type == "formEventCondition") {
                        $scope._goal.targetForm = ManagersUtils.getProperty(subCondition, "parameterValues.formId");
                        $scope._goal.landingType = "formSubmit";
                    }
                    if (subCondition.type == "hasSeenNPagesCondition") {
                        $scope._goal.landingType = "numberOfPagesView";
                        $scope._goal.numberOfPagesView.value = subCondition.parameterValues.value + 1;
                    }
                }
            }
        }

        var doSave = function(callback) {
            // construct landing page event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "pagePath": $scope._goal.startPath
                            },
                            "type": "pageViewEventCondition"
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "duration",
                                "comparisonOperator": "equals",
                                "propertyValueInteger": 0
                            }
                        }
                    ]
                }
            };
            switch ($scope._goal.landingType) {
                case "pageView":
                    $scope.goal.targetEvent = {
                        "parameterValues": {
                            "pagePath": $scope._goal.targetPath
                        },
                        "type": "pageViewEventCondition"
                    };
                    break;
                case "formSubmit":
                    $scope.goal.targetEvent = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "parameterValues": {
                                        "formId": $scope._goal.targetForm
                                    },
                                    "type": "formEventCondition"
                                },
                                {
                                    "parameterValues": {
                                        "path": $scope._goal.startPath
                                    },
                                    "type": "sourceEventPropertyCondition"
                                }
                            ]
                        }
                    };
                    break;
                case "numberOfPagesView":
                    $scope.goal.targetEvent = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "pageViewEventCondition",
                                    "parameterValues": {}
                                },
                                {
                                    "type": "hasSeenNPagesCondition",
                                    "parameterValues": {
                                        "value": $scope._goal.numberOfPagesView.value - 1,
                                        "scope": ManagersContext.currentSiteKey,
                                        "comparisonOperator": 'greaterThanOrEqualTo'
                                    }
                                }
                            ]
                        }
                    };
                    break;
            }

            if (callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalPageVisitGoalController', GoalPageVisitGoalController);

    GoalPageVisitGoalController.$inject = ['$scope'];

    function GoalPageVisitGoalController($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetPath = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.pagePath");

        var doSave = function(callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                },
                "type": "sessionCreatedEventCondition"
            };
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.targetPath
                },
                "type": "pageViewEventCondition"
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').directive('goalReportTable', goalReportTableDirective);

    function goalReportTableDirective() {
        return {
            restrict: 'E',
            scope: {
                goal: '=',
                properties: '=',
                data: '=ngModel',
                type: '@',
                queryRowHook: '@',
                queryHook: '@',
                tableColumnsHook: '@'
            },
            templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-report-table.directive.html',
            controller: GoalReportTableController
        };
    }

    GoalReportTableController.$inject = ['$scope', 'unomiGoalService', 'i18nService', 'unomiSegmentService', 'DTOptionsBuilder', 'DTColumnBuilder', '$uibModal', 'unomiGeonamesService'];

    function GoalReportTableController($scope, unomiGoalService, i18nService, unomiSegmentService, DTOptionsBuilder, DTColumnBuilder, $uibModal, unomiGeonamesService) {
        $scope.facets = {};
        $scope.split = [];
        $scope.ranges = {};

        $scope.actionsHtml = function (data, type, full, meta) {
            // data.selectedProperty.
            return '<button title="' + i18nService.message('serverSettings.manageSegments.manageMembers') + '" type="button" class="btn btn-icon command-profiles" ng-click="openManagement(' + data.key + ')" ><i class="material-icons">person</i></button>';
        };

        $scope.openManagement = function () {
            var dialogModal = {
                backdrop: true,
                keyboard: true,
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/sessions-list-modal.html',
                controller: 'SessionsListModalController',
                resolve: {
                    scopeParent: function () {
                        return $scope; // We pass a ref of the parent scope to the modal
                    },
                    currentProperty: function () {
                        return $scope.data.selectedProperty.itemId;
                    }
                }
            };
            $uibModal.open(dialogModal);
        };

        $scope.setDtInstance = function (val) {
            $scope.dtInstance = val;
        };

        $scope.isNotFacetedProp = function (value, index) {
            return !$scope.facets[value.id];
        };

        $scope.showSegment = function (value, index) {
            return value.itemType != 'segments' || (value.itemType == 'segments' && $scope.isQualitativeReport());
        };

        $scope.showRangeCheckbox = function () {
            return $scope.data && $scope.data.selectedProperty && $scope._isPropertyRanged($scope.data.selectedProperty);
        };

        $scope.showDateInterval = function () {
            return $scope.data && $scope.data.selectedProperty && $scope.data.selectedProperty.type == 'date';
        };

        $scope.getFacetDisplay = function (facet) {
            return facet.property.metadata.name + ' : ' + (facet.value.displayableKey ? ((facet.value.displayableKey.length > 50)?facet.value.displayableKey.substring(0, 50) + '...':facet.value.displayableKey) : facet.value.key);
        };

        $scope.addFacet = function (selectedProperty, rowValue) {
            var facet = {
                property: angular.copy(selectedProperty),
                value: angular.copy(rowValue),
                conditions: []
            };

            // resolve associated conditions to selectedProperty and rowValue
            var isSession = facet.property.itemType == 'sessions';

            var propertyName = facet.property.metadata.id;
            if (facet.property.itemType == 'sessions') {
                propertyName = 'properties.' + propertyName;
            } else if (facet.property.itemType == 'profiles') {
                propertyName = 'profile.properties.' + propertyName;
            } else if (propertyName == 'segments') {
                propertyName = 'profile.segments'
            }
            if ($scope._isPropertyRanged(facet.property) && $scope.data.useRanges) {
                var range = $scope.ranges[facet.property.metadata.id] ? $scope.ranges[facet.property.metadata.id][facet.value.key] : undefined;
                if (range) {
                    if (range.from) {
                        facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'greaterThan', facet.property.type, false, range.from));
                    }
                    if (range.to) {
                        facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'lessThan', facet.property.type, false, range.to));
                    }
                }
            } else if (facet.property.type == 'date') {
                // handle date range
                for (var i = 0; i < $scope.split.length; i++) {
                    var split = $scope.split[i];
                    if (split.key == facet.value.key) {
                        if (i == $scope.split.length - 1) {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'greaterThan', facet.property.type, false,
                                $scope._getSelectedIntervalAsDate(split.key)));
                        } else {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'between', facet.property.type, true,
                                [$scope._getSelectedIntervalAsDate(split.key), $scope._getSelectedIntervalAsDate($scope.split[i + 1].key)]));
                        }
                        break;
                    }
                }
            } else {
                if (facet.value.missing) {
                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'missing', facet.property.type, false, null));
                } else {
                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'equals', facet.property.type, false, facet.value.key));
                }
            }


            $scope.facets[selectedProperty.metadata.id] = facet;
            var filteredProperties = $scope._getFilteredProperties();
            $scope.data.selectedProperty = filteredProperties && filteredProperties.length > 0 ? filteredProperties[0] : undefined;
        };

        $scope.removeFacet = function (facet) {
            if (facet && $scope.facets[facet.property.metadata.id]) {
                delete $scope.facets[facet.property.metadata.id];
                $scope.data.selectedProperty = $scope._getProperty(facet.property.metadata.id);
            }
        };

        $scope.isQualitativeReport = function () {
            return $scope.type == 'qualitative';
        };

        $scope.isQuantitativeReport = function () {
            return $scope.type == 'quantitative';
        };

        $scope.getTableLabel = function () {
            return i18nService.message('serverSettings.manageGoals.report.title.' + $scope.type);
        };

        $scope._getProperty = function (propertyId) {
            for (var propIndex in $scope.properties) {
                if (propertyId == $scope.properties[propIndex].metadata.id) {
                    return $scope.properties[propIndex];
                }
            }
            return null;
        };

        $scope._getFilteredProperties = function () {
            return $scope.properties.filter(function (property) {
                return !$scope.facets[property.metadata.id];
            });
        };

        $scope._isPropertyRanged = function (property) {
            return (property.numericRanges && property.numericRanges.length > 0) ||
                (property.dateRanges && property.dateRanges.length > 0) ||
                (property.ipRanges && property.ipRanges.length > 0);
        };

        $scope._getPropertyCondition = function (isSession, propertyName, operator, type, multiple, propertyValue) {
            var condition = {
                'type': isSession ? 'sessionPropertyCondition' : 'profilePropertyCondition',
                'parameterValues': {
                    'propertyName': propertyName,
                    'comparisonOperator': operator
                }
            };

            switch (type) {
                case 'integer':
                    if (multiple) {
                        condition.parameterValues.propertyValuesInteger = propertyValue;
                    } else {
                        condition.parameterValues.propertyValueInteger = propertyValue;
                    }
                    break;
                case 'date':
                    if (multiple) {
                        condition.parameterValues.propertyValuesDate = propertyValue;
                    } else {
                        condition.parameterValues.propertyValueDate = propertyValue;
                    }
                    break;
                default :
                    condition.parameterValues.propertyValue = propertyValue;
            }

            return condition;
        };

        $scope._getI18nForRange = function (type, to, from) {
            var t,f;
            if (to != 'now') {
                t = to;
            }
            if (from != 'now') {
                f = from;
            }
            if (type == 'date') {
                if (t && _.isString(t)) {
                    t = ManagersUtils.parseDateMath(t);
                }
                if (f && _.isString(f)) {
                    f = ManagersUtils.parseDateMath(f);
                }
            }
            if (t && f) {
                return i18nService.format('ranges.' + type, f.toString() + '|' + t.toString());
            }
            if (t) {
                return i18nService.format('ranges.' + type + '.leftLimit', t.toString());
            }
            if (f) {
                return i18nService.format('ranges.' + type + '.rightLimit', f.toString());
            }
        };

        $scope._getSelectedIntervalAsDate = function (value) {
            var splitted = value.split('-');
            if ($scope.data.selectedInterval.key == '1M') {
                return new Date(splitted[0], parseInt(splitted[1] - 1), 1, null, null, null, null)
            } else {
                return new Date(splitted[0], parseInt(splitted[1] - 1), splitted[2], null, null, null, null)
            }
        };

        $scope._addConditionToQuery = function (query, condition) {
            if (query.condition) {
                query.condition.parameterValues.subConditions.push(condition);
            } else {
                query.condition = {
                    'type': 'booleanCondition',
                    'parameterValues': {
                        'operator': 'and',
                        'subConditions': []
                    }
                };
                $scope._addConditionToQuery(query, condition);
            }
        };

        function doQuery(callback) {
            if ($scope.data) {
                var aggregate;
                switch ($scope.data.selectedProperty.itemType) {
                    case 'profiles':
                        aggregate = 'profile.properties.' + $scope.data.selectedProperty.metadata.id;
                        break;
                    case 'segments':
                        aggregate = 'profile.segments';
                        break;
                    default :
                        aggregate = 'properties.' + $scope.data.selectedProperty.metadata.id;
                }
                //
                var query = {
                    aggregate: {
                        property: aggregate
                    }
                };

                // handle ranged properties
                if ($scope._isPropertyRanged($scope.data.selectedProperty) && $scope.data.useRanges) {
                    var ranges;
                    if ($scope.data.selectedProperty.ipRanges.length > 0) {
                        query.aggregate.type = 'ipRange';
                        query.aggregate.ipRanges = angular.copy($scope.data.selectedProperty.ipRanges);
                        ranges = query.aggregate.ipRanges;
                    } else if ($scope.data.selectedProperty.dateRanges.length > 0) {
                        query.aggregate.type = 'dateRange';
                        query.aggregate.dateRanges = angular.copy($scope.data.selectedProperty.dateRanges);
                        ranges = query.aggregate.dateRanges;
                    } else if ($scope.data.selectedProperty.numericRanges.length > 0) {
                        query.aggregate.type = 'numericRange';
                        query.aggregate.numericRanges = angular.copy($scope.data.selectedProperty.numericRanges);
                        ranges = query.aggregate.numericRanges;
                    }
                    $scope.ranges[$scope.data.selectedProperty.metadata.id] = {};
                    angular.forEach(ranges, function (v, k) {
                        $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                        v.key = $scope._getI18nForRange($scope.data.selectedProperty.type, v.to, v.from);
                        $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                    });
                } else if ($scope.data.selectedProperty.type == 'date') {
                    query.aggregate.type = 'date';
                    query.aggregate.parameters = {
                        'interval': $scope.data.selectedInterval.key,
                        'format': 'yyyy-MM-dd'
                    }
                }

                // handle fromDate and toDate
                if ($scope.data.fromDate && $scope.data.toDate) {
                    var queryFromDate = moment($scope.data.fromDate).toISOString();
                    var queryToDate = moment($scope.data.toDate).endOf($scope.data.timelineInterval.unit).toISOString();
                    $scope._addConditionToQuery(query, $scope._getPropertyCondition(true, 'timeStamp', 'between', 'date', true, [queryFromDate, queryToDate]));
                }

                // handle facets
                if ($scope.facets) {
                    for (var key in $scope.facets) {
                        if ($scope.facets.hasOwnProperty(key)) {
                            var facet = $scope.facets[key];
                            if (facet.conditions && facet.conditions.length > 0) {
                                for (var i = 0; i < facet.conditions.length; i++) {
                                    $scope._addConditionToQuery(query, facet.conditions[i]);
                                }
                            }
                        }
                    }
                }


                if ($scope.goal) {
                    unomiGoalService.getGoalReportAggregated($scope.goal.metadata.id, query).then(function (response) {
                        var data = response.data;
                        if ($scope.queryHook) {
                            $scope.$parent[$scope.queryHook](data, $scope.data);
                        }

                        angular.forEach(data.split, function (v, k) {
                            if (v.key == '_missing') {
                                v.missing = true;
                                v.key = i18nService.message('serverSettings.manageGoals.report.table.missing');
                            } else if ($scope.data.selectedProperty.itemType == 'segments') {
                                if ($scope.segmentsIndexedById.hasOwnProperty(v.key)) {
                                    v.displayableKey = $scope.segmentsIndexedById[v.key].name;
                                } else {
                                    v.displayableKey = i18nService.message('serverSettings.manageGoals.report.table.unknownSegment')
                                }
                            } else if (query.aggregate.type == 'date') {
                                v.displayableKey = moment(v.key).format($scope.data.selectedInterval.aggregateFormat);
                            }
                            if ($scope.queryRowHook) {
                                $scope.$parent[$scope.queryRowHook](v, $scope.data);
                            }
                        });
                        if (callback) {
                            callback(data.split);
                        }
                    });
                } else {
                    var aggregateSessionQuery = {
                        'aggregate': {
                            'type': 'string',
                            'property': aggregate
                        }
                    };

                    var tabSplit = [];
                    var queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext;
                    unomiQueryService.query('session', aggregate, aggregateSessionQuery).then(function (response) {
                        var aggregateSessionsData = response.data;
                        if (callback) {
                            var tabLine = {};
                            if (aggregateSessionsData._all) {
                                angular.forEach(aggregateSessionsData, function (v, k) {
                                    if (k != '_all') {
                                        tabLine = {};
                                        tabLine.key = k;
                                        tabLine.targetCount = v;
                                        tabLine.percentage = v / aggregateSessionsData._all;
                                        tabSplit.push(tabLine);
                                    }
                                });
                            }
                            callback(tabSplit);
                        }
                    });


                }

            }
        }

        function loadTableColumns() {
            var table = [];
            if ($scope.isQualitativeReport()) {
                table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                    return full.displayableKey ? full.displayableKey : data;
                }));
                table.push(DTColumnBuilder.newColumn('startCount').withTitle(i18nService.message('serverSettings.manageGoals.views')));
                table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.success')));
                table.push(DTColumnBuilder.newColumn('conversionRate').withTitle(i18nService.message('serverSettings.manageGoals.conversion')).renderWith(function (data, type, full) {
                    return ManagersUtils.getPercentage(data) + ' %';
                }));
            } else {
                table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                    return full.displayableKey ? full.displayableKey : data;
                }));
                if ($scope.goal) {
                    table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.success')));
                    table.push(DTColumnBuilder.newColumn('percentage').withTitle(i18nService.message('serverSettings.manageGoals.percentage')).renderWith(function (data, type, full) {
                        return ManagersUtils.getPercentage(data) + ' %';
                    }));
                } else {
                    table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.count')));
                    table.push(
                        DTColumnBuilder
                            .newColumn(null)
                            .withTitle(i18nService.message('serverSettings.manageWem.listAction'))
                            .notSortable()
                            .renderWith($scope.actionsHtml)
                            .withOption('width', '25%')
                            .withOption('contentPadding', '0'));

                }
            }
            if ($scope.tableColumnsHook) {
                $scope.$parent[$scope.tableColumnsHook](table, DTColumnBuilder, $scope.data);
            }
            $scope.dtColumns = angular.copy(table);
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('aaSorting', [])
            .withOption('sDom', 'lt')
            .withDisplayLength(10)
            .withOption('lengthMenu', [10, 25, 50, 100])
            .withOption('rowCallback', function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $('td', nRow).unbind();
                $('td', nRow).bind('click', function () {
                    $scope.$apply(function () {
                        $scope.addFacet($scope.data.selectedProperty, aData);
                    });
                });
                $('td .btn.btn-icon.command-profiles', nRow).unbind();
                $('td .btn.btn-icon.command-profiles', nRow).bind('click', function (event) {
                    event.stopPropagation();
                    $scope.openManagement();

                });
                return nRow;
            });

        // watch data to refresh table
        $scope.$watch('data', function(newVal, oldVal) {
            if ($scope.data) {
                doQuery(function(split) {
                    //Handle geonames properties in goal report
                    if ($scope.data.selectedProperty.itemId && $scope.data.selectedProperty.itemId.indexOf('sessionAdminSubDiv') != -1) {
                        //Browsing on each table line
                        _.forEach(split, function(object, index) {
                            //If not localhost
                            if (object.key != 'GE' && object.key != 2500 && object.key!= 'Not provided' ) {
                                var geoId = object.key;
                                //Request the geonames entity for the property value
                                unomiGeonamesService.getCapital(geoId).then(function(response) {
                                    var data = response.data;
                                    var filtered = _.filter(data, function(o) {
                                        return o.itemId == geoId;
                                    });
                                    if (filtered.length > 0 ) {
                                        //If result in geonames display the geonames entity name
                                        split[index].displayableKey = filtered[0].name
                                    }
                                    if (index == (split.length - 1)) {
                                        //Refresh datatable after geonames call for the last line
                                        $scope.split = split;
                                        $scope.dtOptions.data = split;
                                        loadTableColumns();
                                    }
                                });
                            }
                        });
                    }

                    $scope.split = split;
                    $scope.dtOptions.data = split;
                    loadTableColumns();
                });
            }
        }, true);


        if ($scope.isQualitativeReport()) {
            unomiSegmentService.getAll().then(function (response) {
                $scope.segmentsIndexedById = _.indexBy(response.data, 'id');
            });
        }

    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalReportController', GoalReportController);

    GoalReportController.$inject = ['$scope', '$route', '$routeParams', '$location', '$timeout', 'unomiGoalService', 'goal', 'i18nService', 'unomiProfileService', 'modalService', 'loadingSpinnerService'];

    function GoalReportController($scope, $route, $routeParams, $location, $timeout, unomiGoalService, goal, i18nService, unomiProfileService, modalService, loadingSpinnerService) {
        $scope.goalId = $routeParams.goalId;
        $scope.goal = goal;
        $scope.goalReportIntervales = ManagersUtils.supportedTimelineIntervales;

        // timeline data
        $scope.timelineData = {
            toDate: moment(moment().format("YYYY-MM"), "YYYY-MM").toDate(),
            fromDate: moment(moment().subtract(12, "months").format("YYYY-MM"), "YYYY-MM").toDate(),
            interval: $scope.goalReportIntervales[0],
            title: "serverSettings.manageGoals.report.timeline",
            goalId: $scope.goalId,
            goal: $scope.goal,
            series: [i18nService.message('serverSettings.manageGoals.report.startCountTimeline'), i18nService.message('serverSettings.manageGoals.report.targetCountTimeline')],
            loadData: function (success) {
                var query = {
                    aggregate: {
                        type: "date",
                        property: "timeStamp",
                        parameters: {
                            interval: $scope.timelineData.interval.key,
                            format: "yyyy-MM-dd"
                        }
                    }
                };
                unomiGoalService.getGoalReportAggregated($scope.timelineData.goalId, query).then(function (response) {
                    var data = response.data;
                    $scope.report = data;
                    $scope.report.globalStats.conversion = ManagersUtils.getPercentage($scope.report.globalStats.conversionRate);

                    var datasets = [{}, {}];

                    if (data.split && data.split.length > 0) {
                        angular.forEach(data.split, function (v, k) {
                            if (v.key) {
                                datasets[0][v.key] = v.startCount;
                                datasets[1][v.key] = v.targetCount;
                            }
                        });
                    }
                    success(datasets);
                });
            }
        };

        // get properties filter
        $scope.quantitativeProperties = [];
        $scope.qualitativeProperties = [];
        $scope.qualitativeProperties.push(
            {
                "type": 'string',
                "itemType": 'segments',
                "metadata": {
                    "id": 'segments',
                    "name": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
                },
                "itemTypeLabel": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
            });
        unomiProfileService.getPropertyTypes().then(function (response) {
            var data = response.data;
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {
                                $scope.qualitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };
                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.qualitativeReportData.fromDate = newVal.fromDate;
                                    $scope.qualitativeReportData.toDate = newVal.toDate;
                                    $scope.qualitativeReportData.timelineInterval = newVal.interval;
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                            $scope.qualitativeProperties.push(p);
                        });
                    }
                });
            }
        });

        $scope.refresh = function () {
            $route.reload();
        };

        $scope.deleteGoal = function (id) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.message("serverSettings.manageGoals.delete.confirm")
            }, function () {
                loadingSpinnerService.show();
                unomiGoalService.deleteGoal(id).then(function (response) {
                    // added timeout before reloading to make sure Unomi has time to remove the goal
                    $timeout(function() {
                        loadingSpinnerService.hide();
                        $location.path('/goals');
                    }, 1000);
                });
            });
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalVideoGoalController', GoalVideoGoalController);

    GoalVideoGoalController.$inject = ['$scope'];

    function GoalVideoGoalController($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.startPath = ManagersUtils.getProperty($scope.goal, "startEvent.parameterValues.pagePath");
        var subConditions = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.subConditions");
        if (subConditions) {
            for (var i = 0; i < subConditions.length; i++) {
                var subCondition = subConditions[i];
                if (subCondition.type == "videoViewEventCondition") {
                    $scope._goal.targetVideo = ManagersUtils.getProperty(subCondition, "parameterValues.videoId");
                }
            }
        }

        var doSave = function (callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.startPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.targetEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "videoId": $scope._goal.targetVideo
                            },
                            "type": "videoViewEventCondition"
                        },
                        {
                            "parameterValues": {
                                "path": $scope._goal.startPath
                            },
                            "type": "sourceEventPropertyCondition"
                        }
                    ]
                }
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').directive('goalWidget', goalWidgetDirective);

    function goalWidgetDirective() {
        return {
            restrict: 'E',
            scope: {
                goal: '=',
                tagId: '=',
                campaignId: '@'
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: GoalWidgetController
        };
    }

    GoalWidgetController.$inject = ['$rootScope', '$scope', 'unomiGoalService'];

    function GoalWidgetController($rootScope, $scope, unomiGoalService) {
        // get report
        var query = {
            aggregate: {
                type: "date",
                property: "timeStamp",
                parameters: {
                    interval: "1d"
                }
            }
        };
        unomiGoalService.getGoalReportAggregated($scope.goal.id, query).then(function (response) {
            var reportData = response.data;
            $scope.report = reportData.globalStats;
            $scope.report.conversion = ManagersUtils.getPercentage($scope.report.conversionRate);
            $scope.doughnutLabels = ['Success', 'Views'];
            $scope.doughnutData = [$scope.report.conversion, 100 - $scope.report.conversion];
            var doughnutColor = _getDoughnutColor($scope.report.conversion);
            $scope.doughnutColor = [doughnutColor, '#eeeeee'];
            $scope.doughnutOpts = {
                cutoutPercentage: 60,
                title: {
                    display: true,
                    text: $scope.report.conversion + '%',
                    fontColor: doughnutColor,
                    fontSize: 22,
                    fullWidth: false,
                    fontStyle: 'normal',
                    position: 'bottom',
                    fontFamily: "'Nunito Sans','Helvetica Neue',Helvetica,Arial,sans-serif"
                },
                animation: {
                    easing: 'easeOutBounce'
                }
            };

            // compute trend
            if (reportData.split && reportData.split.length > 0) {
                var currentWeek = moment().startOf('week');
                var weekOneBeforeCurrent = moment().startOf('week').subtract(1, 'weeks');
                var weekTwoBeforeCurrent = moment().startOf('week').subtract(2, 'weeks');
                var result = {
                    startCountBefore: 0,
                    startCountBeforeTwo: 0,
                    targetCountBefore: 0,
                    targetCountBeforeTwo: 0
                };
                angular.forEach(reportData.split, function (v, k) {
                    if (v && v.key) {
                        var thisMoment = moment(v.key);
                        if (thisMoment.isBetween(weekTwoBeforeCurrent, currentWeek)) {
                            if (thisMoment.isAfter(weekOneBeforeCurrent)) {
                                result.startCountBefore = result.startCountBefore + v.startCount;
                                result.targetCountBefore = result.targetCountBefore + v.targetCount;
                            } else {
                                result.startCountBeforeTwo = result.startCountBeforeTwo + v.startCount;
                                result.targetCountBeforeTwo = result.targetCountBeforeTwo + v.targetCount;
                            }
                        }
                    }
                });
                result.conversionBefore = result.startCountBefore > 0 ? result.targetCountBefore / result.startCountBefore : 0;
                result.conversionBeforeTwo = result.startCountBeforeTwo > 0 ? result.targetCountBeforeTwo / result.startCountBeforeTwo : 0;
                if (result.conversionBefore > result.conversionBeforeTwo) {
                    $scope.progressionArrow = 'trending_up';
                } else if (result.conversionBefore == result.conversionBeforeTwo) {
                    $scope.progressionArrow = 'trending_flat';
                } else {
                    $scope.progressionArrow = 'trending_down';
                }
            }
        });

        // resolve template
        $scope.templateUrl = ManagersContext.wemAppPath + '/wem-goals/goal-widget-' + $scope.tagId + '.directive.html';

        $scope.goalLinksTmpl = $rootScope.getGoalLinksView();

        function _getDoughnutColor(percentage) {
            if (percentage > 90) {
                return '#8454c6';
            }
            if (percentage > 20) {
                return '#4BE45F';
            }
            if (percentage > 5) {
                return '#ffc500';
            }
            return '#E44759';
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalsListController', GoalsListController);

    GoalsListController.$inject = ['$scope', '$route', 'unomiGoalService', 'unomiQueryService', 'i18nService', 'currentCampaign', 'filteringService', 'unomiProfileService', '_', 'loadingSpinnerService'];

    function GoalsListController($scope, $route, unomiGoalService, unomiQueryService, i18nService, currentCampaign, filteringService, unomiProfileService, _, loadingSpinnerService) {
        $scope.aggregateSessions = null;
        $scope.aggregateNewSessions = {};
        $scope.lineChartData = {
            labels: [],
            datasets: [],
            series: []
        };
        $scope.goalsByTag = {};
        $scope.currentCampaign = currentCampaign;
        $scope.primaryGoal = {};

        // timeline data
        $scope.timelineData = {
            toDate: _.isUndefined(currentCampaign) ? moment().endOf('month').toDate() : moment(currentCampaign.endDate).toDate(),
            fromDate: _.isUndefined(currentCampaign) ? moment().subtract(1, "years").startOf('month').toDate() : moment(currentCampaign.startDate).toDate(),
            interval: ManagersUtils.supportedTimelineIntervalesObj[_.isUndefined(currentCampaign) ? "1M" : "1d"],
            title: _.isUndefined(currentCampaign) ? "serverSettings.manageGoals.sessionsTimeline" : currentCampaign.metadata.name,
            currentCampaign: currentCampaign,
            series: [i18nService.message('serverSettings.manageGoals.allSessions'), i18nService.message('serverSettings.manageGoals.newSessions')],
            loadData: function (success) {
                var aggregateSessionQuery = {
                    "aggregate": {
                        "type": "date",
                        "parameters": {
                            "interval": this.interval.key,
                            "format": "yyyy-MM-dd"
                        }
                    },
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "scope",
                                        "comparisonOperator": "equals",
                                        "propertyValue": ManagersContext.dmfScope
                                    }
                                }
                            ]
                        }
                    }
                };
                // get all sessions
                unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                    var aggregateSessionsData = response.data;
                    var datasets = [];
                    datasets[0] = aggregateSessionsData;

                    // get new sessions:
                    aggregateSessionQuery.condition.parameterValues.subConditions.push({
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "profile.properties.nbOfVisits",
                            "comparisonOperator": "equals",
                            "propertyValueInteger": 1
                        }
                    });
                    unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                        datasets[1] = response.data;
                        success(datasets);
                    });
                });
            }
        };

        var processGoals = function (goals, currentCampaign) {
            goals = filteringService.filterItems(goals, unomiGoalService);

            if (goals && goals.length > 0) {
                goals.forEach(function (goal) {
                    if (currentCampaign && goal.id == currentCampaign.primaryGoal) {
                        $scope.primaryGoal = goal;
                        goal.isPrimary = true;
                    } else {
                        if (goal.tags && goal.tags.length > 0 && !goal.hidden) {
                            var tag = $scope.goalsByTag[goal.tags[0]];
                            if (tag) {
                                if (tag.goals) {
                                    tag.goals.push(goal);
                                } else {
                                    tag.goals = [goal];
                                }
                            }
                        }
                    }
                });
            }
        };

        $scope.goalsByTag = {};
        // load goals
        // process tags and create a dictionary by tag id
        if (_.isUndefined(currentCampaign)) {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService);
            unomiGoalService.getAllForSite().then(function (response) {
                processGoals(response.data);
                loadingSpinnerService.hide();
            });
        } else {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.campaingTags, i18nService);
            unomiGoalService.getAllForCampaign(currentCampaign.metadata.id).then(function (response) {
                processGoals(response.data, currentCampaign);
                $scope.subGoalsAvailable = _.reduce($scope.goalsByTag, function (memo, tag) {
                    return memo + tag.goals.length;
                }, 0) > 0;
                loadingSpinnerService.hide();
            });
        }

        $scope.refresh = function () {
            $route.reload();
        };

        $scope.subGoalsAvailable = false;

        // ABO(+) :
        $scope.quantitativeProperties = [];
        unomiProfileService.getPropertyTypes().then(function (response) {
            var data = response.data;
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {

                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": '',
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": ''
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                        });
                    }
                });
            }
        });
    }
})();

(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('SessionsListModalController', SessionsListModalController);

    SessionsListModalController.$inject = ['$scope', 'scopeParent'];

    function SessionsListModalController($scope, scopeParent) {
        // get back the query without the aggregate
        var queryWithoutAggregate = {};
        $scope._addConditionToQuery = function (query, condition) {
            if (query.condition) {
                query.condition.parameterValues.subConditions.push(condition);
            } else {
                query.condition = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": []
                    }
                };
                $scope._addConditionToQuery(query, condition);
            }
        };

        var parentFacets = angular.copy(scopeParent.facets);
        var conditions = [];
        var clickedCondition = {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": scope.getCurrentProperty(),
                "comparisonOperator": "equals",
                "propertyValue": "male"
            }
        };
        conditions.push(clickedCondition);
        angular.forEach(parentFacets, function (value, key) {
            var facet = value;
            if (facet) {
                if (facet.conditions && facet.conditions.length > 0) {
                    for (var i = 0; i < facet.conditions.length; i++) {
                        conditions.push(angular.copy(facet.conditions[i]));
                    }
                }
            }
        });

        //
        scope.getCurrentProperty();

        // Ajout de la condition sur la ligne selectionnée
        // Requete au service Unomi.
        // $query('sessions');


        // recuperation de toutes les conditions.
        $scope.dtColumnsSession = angular.copy(scopeParent.dtColumns);
        $scope.dtOptionsSessions = angular.copy(scopeParent.dtOptions);


    }
})();
