(function() {
    'use strict';

    angular.module('wemInternalSearchesApp', ['wemApp', 'wemProfilesApp']);
})();

(function() {
    'use strict';

    angular.module('wemInternalSearchesApp').run(wemInternalSearchesAppRun);

    wemInternalSearchesAppRun.$inject = ['$rootScope'];

    function wemInternalSearchesAppRun($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemInternalSearchesApp').controller('InternalSearchDetailsController', InternalSearchDetailsController);

    InternalSearchDetailsController.$inject = ['$scope', 'internalSearchService', 'i18nService', 'notificationService'];

    function InternalSearchDetailsController($scope, internalSearchService, i18nService, notificationService) {
        $scope.$on("$routeChangeError", function (evt, current, previous, rejection) {
            if (rejection == "profile_not_found") {
                notificationService.errorToast(i18nService.message("route.reject.profile_not_found"))
            }
        });

        $scope.searchDetail = internalSearchService.getSearchDetails();

        $scope.getTabTitle = function (key) {
            return i18nService.message(key);
        };
    }
})();
(function() {
    'use strict';

    angular.module('wemInternalSearchesApp').controller('InternalSearchesListController', InternalSearchesListController);

    InternalSearchesListController.$inject = ['$scope', 'uibDatepickerConfig', '$compile', 'dataTableService', 'i18nService', '$location', 'internalSearchService', 'dtPagerService'];

    function InternalSearchesListController($scope, uibDatepickerConfig, $compile, dataTableService, i18nService, $location, internalSearchService, dtPagerService) {
        $scope.internalSearchFrom = Date.now() + -30*24*3600*1000;
        $scope.internalSearchTo = Date.now();
        $scope.selectedLanguage = {id: "_all", name: "all"};
        internalSearchService.getSearchLanguages().then(function (result) {
            $scope.availableLanguages = result;
        });

        var actionsHtml = function (data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.internalSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><i class=\"material-icons\">search</i></button> ";
        };

        var handleMissing = function (data, type, full, meta) {
            return ManagersUtils.humanReadableMissingIfNeeded(data, i18nService);
        };

        //No global controller where to put this function, it must be redefined for each app
        var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
            $compile(nRow)($scope);
        };

        var init = function (from, to, language) {
            var promise = internalSearchService.getSearchOccurrencesByTermAndDate(from, to, language);
            dataTableService.setPromise(promise);
            $scope.dtOptions = dataTableService.generateDataTableOptions(
                {defaultOrder: [[2, 'desc']], rowCallback: rowCallback}
            );

            var columns = [
                {name: 'searchString', bundleKey: 'serverSettings.internalSearches.searchString', actionsHtml: handleMissing},
                {name: 'nbOcc', bundleKey: 'serverSettings.internalSearches.occurrences'},
                {name: 'importance', bundleKey: 'serverSettings.internalSearches.importance'},
                {name: null, bundleKey: 'serverSettings.manageWem.listAction',
                    actionsHtml: actionsHtml,
                    notSortable: true,
                    options: [
                        ["width", "180px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];
            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
            promise.then(function(){
                dtPagerService.refreshPager($scope);
            });
        };

        init($scope.internalSearchFrom, $scope.internalSearchTo, $scope.selectedLanguage.id);

        // date config
        uibDatepickerConfig.startingDay = 1;
        uibDatepickerConfig.minMode = "day";
        uibDatepickerConfig.datepickerMode = "day";
        uibDatepickerConfig.showWeeks = false;
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.selectedLanguageLabel = i18nService.message("serverSettings.manageWem.selectedLanguage");
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            angular.forEach($scope.datePickers, function(value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToDetails = function (index) {
            internalSearchService.setSelectedSearch($scope.dtInstance.DataTable.row(index).data().searchString);
            // the redirect to search details view using a fake route parameter so that we can properly find the previous page in profiles controllers
            $location.path('/search/selected');
        };

        $scope.updateSearchesList = function () {
            init(new Date($scope.internalSearchFrom).getTime(), new Date($scope.internalSearchTo).getTime(), $scope.selectedLanguage.id);
            $scope.dtInstance.DataTable.draw();
        };

        $scope.getMessage = function (key) {
            return i18nService.message(key);
        };
    }
})();