(function() {
    'use strict';

    angular.module('wemManagePersonalizationConditionsApp', ['wemApp', 'ui.sortable']);
})();

(function() {
    'use strict';

    angular.module('wemManagePersonalizationConditionsApp')
        .controller('ManagePersonalizationConditionsController', ManagePersonalizationConditionsController);

    ManagePersonalizationConditionsController.$inject = ['$scope', '$resource', 'unomiDefinitionService', 'i18nService', 'notificationService', 'unomiSegmentService'];

    function ManagePersonalizationConditionsController($scope, $resource, unomiDefinitionService, i18nService, notificationService, unomiSegmentService) {
        var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition,sessionCondition"}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

        var modes = {
            segments: {
                "parameterValues": {
                    "segments": [],
                    "matchType": "all"
                },
                "type": "profileSegmentCondition"

            },
            location: {
                "parameterValues": {},
                "type": "geoLocationSessionCondition"

            },
            custom: {}
        };

        $scope.variants = variants;
        $scope.tabs = ["no-condition", "segments", "location", "custom"];

        $scope.$watch("toBeSaveVariant.conditions[toBeSaveVariant.tab]", function (newValue, oldValue) {
            if (newValue != oldValue && _.isEmpty(newValue) && !_.isEmpty(oldValue) && $scope.toBeSaveVariant.tab === 'custom') {
                $scope.toBeSaveVariant.conditions['custom'] = {};
                modes ['custom'] = {};
            }
        });

        $scope.toBeSaveVariants = [];
        angular.forEach($scope.variants, function (variant) {
            $scope.toBeSaveVariants.push({
                "id": variant.id,
                "name": variant.name,
                "systemName": variant.systemName,
                "parent": variant.parent,
                "position": variant.position,
                "tab": variant.tab ? variant.tab : "no-condition",
                "init": variant.tab && variant.jsonFilter,
                "conditions": {}
            });

            var toBeSaveVariantFound = _.find($scope.toBeSaveVariants, function (toBeSaveVariant) {
                return toBeSaveVariant.id === variant.id;
            });

            angular.forEach($scope.tabs, function (tab) {
                if (variant.tab == tab && variant.jsonFilter) {
                    toBeSaveVariantFound.conditions[tab] = angular.copy(variant.jsonFilter);
                } else {
                    toBeSaveVariantFound.conditions[tab] = angular.copy(modes[tab]);
                }
            });
        });

        $scope.sortableOptions = {
            cursor: 'move',
            stop: function(e, ui) {
                for(var i =0 ; i < $scope.toBeSaveVariants.length ; i++) {
                    $scope.toBeSaveVariants[i].position = '' + i;
                }
            }
        };

        $scope.conditionTypes = [];
        $scope.tags = {};

        $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition,sessionCondition"});
        $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"});
        $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
            console.log("manage-personalization-conditions.js: tags loaded from Apache Unomi.");
            angular.forEach(value.subTags, function (v) {
                $scope.tags[v.id] = v
            });
            unomiDefinitionService.fillTags("conditionTypes", $scope);
            unomiDefinitionService.fillTags("eventConditionTypes", $scope);
        });

        $scope.validate = function (gwtValidation) {
            console.log("manage-personalization-conditions.js: validating...");
            if ($scope.personalized && $scope.personalized.$invalid) {
                $scope.personalized.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm($scope.personalized);
                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    var message = "";
                    angular.forEach(parsedErrors, function (value, key) {
                        message += ("- " + i18nService.message(key) + ". </br></br>")
                    });
                    gwtValidation.addValidationError(message.substring(0, message.length - "</br></br>".length));
                }
            }
        };

        $scope.save = function (gwtNode) {
            angular.forEach($scope.toBeSaveVariants, function (variantToBeSave) {
                if (!variantToBeSave.parent) {
                    var child = gwtNode.child(variantToBeSave.id, variantToBeSave.systemName);
                    if (variantToBeSave.init) {
                        unomiSegmentService.addDownloadMixin(variantToBeSave.conditions[variantToBeSave.tab]);
                        child.setProperty("wem:tab", variantToBeSave.tab, 1);
                        child.setProperty("wem:jsonFilter", angular.toJson(variantToBeSave.conditions[variantToBeSave.tab]), 1);
                    } else {
                        child.setProperty("wem:tab", "", 1);
                        child.setProperty("wem:jsonFilter", "", 1);
                    }
                    if(persoType && persoType === 'page') {
                        child.setProperty("wem:position", variantToBeSave.position, 1);
                    }
                } else {
                    if (variantToBeSave.init) {
                        unomiSegmentService.addDownloadMixin(variantToBeSave.conditions[variantToBeSave.tab]);
                        gwtNode.setProperty("wem:tab", variantToBeSave.tab, 1);
                        gwtNode.setProperty("wem:jsonFilter", angular.toJson(variantToBeSave.conditions[variantToBeSave.tab]), 1);
                    } else {
                        gwtNode.setProperty("wem:tab", "", 1);
                        gwtNode.setProperty("wem:jsonFilter", "", 1);
                    }
                    if(persoType){
                        gwtNode.setProperty("wem:position", variantToBeSave.position, 1);
                    }
                }
            });
        };

        $scope.selectVariant = function (selectedVariant) {
            $scope.toBeSaveVariant = selectedVariant;
            //validate current condition if there is one
            if ($scope.toBeSaveVariant.init && $scope.personalized && $scope.personalized.$invalid) {
                $scope.personalized.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm($scope.personalized);

                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                }
            } else {
                $scope.$broadcast('eventVariantSelected', null);
            }
        };

        $scope.hasError = function (propertyName) {
            return ManagersUtils.hasErrors($scope.personalized, $scope.personalized, propertyName);
        };

        $scope.selectTab = function (selectedTab) {
            $scope.toBeSaveVariant.tab = selectedTab;
            $scope.toBeSaveVariant.init = (selectedTab === 'no-condition') ? false : true;
            $scope.toBeSaveVariant.disableEdition = (selectedTab != "custom");
        };

        $scope.trimVariantName = function (variantName) {
            if (variantName.length > 32) {
                return variantName.substring(0, 32) + '...';
            } else {
                return variantName;
            }
        };
    }
})();
