(function() {
    'use strict';

    angular.module('wemScoringApp', ['wemApp', 'wemProfilesApp']);
})();

(function() {
    'use strict';

    angular.module('wemScoringApp').run(wemScoringAppRun);

    wemScoringAppRun.$inject = ['$rootScope', '$timeout', 'i18nService', 'modalService', 'unomiProfileService', 'unomiRulesService', 'unomiScoringService', 'loadingSpinnerService'];

    function wemScoringAppRun($rootScope, $timeout, i18nService, modalService, unomiProfileService, unomiRulesService, unomiScoringService, loadingSpinnerService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.tryDeleteItem = function (scoring, callback) {
            // if the scoring we're trying to delete is shared, display a modal to ask confirmation
            if (scoring.scope != ManagersContext.dmfScope) {
                modalService.newAlertModal({
                    title: i18nService.format("serverSettings.manageScoring.delete.sharedWarning.title", scoring.name),
                    bodyMessage: i18nService.message("serverSettings.manageScoring.delete.sharedWarning.message")
                }, function () {
                    deleteScoring(scoring, callback);
                });
            } else {
                modalService.newAlertModal({
                    title: i18nService.format("serverSettings.manageScoring.delete.title", scoring.name),
                    bodyMessage: i18nService.format("serverSettings.manageScoring.delete.message", scoring.name)
                }, function () {
                    deleteScoring(scoring, callback);
                });
            }
        };

        var deleteScoring = function(scoring, callback) {
            loadingSpinnerService.show();
            var update = unomiScoringService.buildUpdateBatchRemoveScoring(scoring.id);
            unomiProfileService.batchProfilesUpdate(update).then(function () {
                unomiScoringService.deleteScoring(scoring.id, true).then(function (response) {
                    var data = response.data;
                    loadingSpinnerService.hide();
                    unomiRulesService.deleteRule("sendEmailRulePrefix_" + scoring.id);
                    // if data isn't empty, it contains the list of the scoring that use the scoring we're trying to delete as a condition
                    if (data.segments.length > 0 || data.scorings.length > 0) {
                        // display the list of dependent scoring and ask what to do
                        modalService.newAlertModal({
                            title: i18nService.format("serverSettings.manageScoring.delete.dependentsModal.title", scoring.name),
                            bodyTemplateUrl: ManagersContext.wemAppPath + "/wem-scoring/scoring-delete-dependents-modal-body.html",
                            scoringName: scoring.name,
                            dependents: data
                        }, function () {
                            loadingSpinnerService.show();
                            unomiScoringService.deleteScoring(scoring.id, false).then(function (response) {
                                $timeout(function () {
                                    loadingSpinnerService.hide();
                                    if (callback) {
                                        callback();
                                    }
                                }, 1000);
                            });
                        });
                    } else {
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            if (callback) {
                                callback();
                            }
                        }, 1000);
                    }
                });
            },
            function () {
                console.error("not updated. Failure!");
                $timeout(function () {
                    loadingSpinnerService.hide();
                    if (callback) {
                        callback();
                    }
                }, 1000);
            });
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemScoringApp').controller('ScoringEditController', ScoringEditController);

    ScoringEditController.$inject = ['$scope', '$rootScope', '$resource', '$location', '$route', '$q','unomiProfileService', 'unomiRulesService', 'unomiScoringService','currentScoring', 'newScoring', 'unomiDefinitionService', 'unomiQueryService', 'i18nService', 'notificationService', 'loadingSpinnerService'];

    function ScoringEditController($scope, $rootScope, $resource, $location, $route, $q, unomiProfileService, unomiRulesService, unomiScoringService, currentScoring, newScoring, unomiDefinitionService, unomiQueryService, i18nService, notificationService, loadingSpinnerService) {
        $scope.currentScoring = currentScoring;
        $scope.conditionTypes = [];
        $scope.sharedScoring = currentScoring.metadata.scope != ManagersContext.dmfScope;
        $scope.newScoring = newScoring || false;
        $scope.cannotShareTooltip = i18nService.message('serverSettings.manageScoring.sharedCannotEdit');

        var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

        $scope.tags = {};

        $rootScope.allowedTags = ['aggregated', 'demographic', 'event', 'commerceFactory'];

        $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value) {
            console.log("scoring-edit.controller.js : tags loaded from Apache Unomi.");

            angular.forEach(value.subTags, function (v) {
                $scope.tags[v.id] = v;
            });
            unomiDefinitionService.fillTags("conditionTypes", $scope);
            unomiDefinitionService.fillTags("eventConditionTypes", $scope);
            $scope.allowedConditionTypes = getAllowedConditions();
        });

        var getAllowedConditions = function () {
            $scope.$watch("conditionTypes.length", function (newVal, oldVal, scope) {
                if(!$scope.allowedConditionTypes){
                    $scope.allowedConditionTypes = [];
                }
                angular.forEach($scope.conditionTypes, function (ct) {
                    if ($rootScope.allowedTags.indexOf(ct.profileTag) !== -1) {
                        ct.name_tr = i18nService.message('condition.' + ct.id + '.name');
                        ct.profileTag_tr = i18nService.message('tag.' + ct.profileTag + '.name');
                        $scope.allowedConditionTypes.push(ct);
                    }
                });
            });
        };

        $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
            console.log("scoring-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
        });

        $scope.scoringType = _.find($scope.currentScoring.metadata.tags, function (tag) {
                                return tag.indexOf('st:') !== -1;
                            });
        /*if(!$scope.scoringType){
            $scope.scoringType = "st:all";
        }*/

        $scope.scoringTypeChange = function (){
            $scope.currentScoring.metadata.tags = _.filter($scope.currentScoring.metadata.tags, function (tag) {
                return tag && tag.indexOf(':') === -1;
            });
            $scope.currentScoring.metadata.tags.push($scope.scoringType);
        };
        $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
            console.log("scoring-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
        });

        unomiScoringService.getImpacted(currentScoring.metadata.id).then(function(response){
            $scope.dependents = response.data;
        });

        $scope.resetScoring = function ($event) {
            unomiScoringService.getScoring(currentScoring.metadata.id).then(function (response) {
                currentScoring = response.data;
                $route.reload();
            });
        };

        $scope.copyScoring = function ($event) {
            unomiScoringService.scoringToCopy = angular.copy(currentScoring);
            $location.path("/newScoring");
        };

        $scope.hasError = function(propertyName) {
            return ManagersUtils.hasErrors($scope.scoringForm, $scope.scoringForm, propertyName);
        };

        $scope.saveScoring = function (scoringForm) {
            if(scoringForm.$invalid){
                scoringForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(scoringForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var save = function () {
                if ($scope.newScoring) {
                    unomiScoringService.setScoring($scope.currentScoring, $scope.sharedScoring).then(function () {
                        $location.path("/scoring");
                    });
                } else {
                    unomiScoringService.setScoring($scope.currentScoring, $scope.sharedScoring).then(function () {
                        if (!$scope.newScoring) {
                            var update = unomiScoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                            unomiProfileService.batchProfilesUpdate(update);
                        }
                        $location.path("/scoring");
                    });
                }
            };

            loadingSpinnerService.show();
            if ($scope.newScoring) {
                unomiQueryService.createWithCallbackIfDoesNotExist('scoring', ManagersContext.dmfScope, $scope.currentScoring.metadata.name, null,
                    "serverSettings.manageScoring.create.alreadyExisting", $scope.currentScoring.metadata.name, save, function() {
                        loadingSpinnerService.hide();
                    });
            } else {
                save();
            }
        };

        $scope.deleteCurrent = function () {
            var id = $scope.currentScoring.metadata.id;
            var rulePromise = unomiRulesService.deleteRule("sendEmailRulePrefix_" + $scope.currentScoring.metadata.id).then(function(response){
                var update = unomiScoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                unomiProfileService.batchProfilesUpdate(update);
            });
            $q.all([rulePromise]).then(function () {
                $scope.tryDeleteItem($scope.currentScoring.metadata, function() {
                    unomiScoringService.deletedItem = id;
                    $location.path('/scoring');
                });
            });
        };

        $scope.deleteScoringElement = function (index) {
            $scope.currentScoring.elements.splice(index, 1);
        };

        $scope.duplicateScoringElement = function (index) {
            $scope.currentScoring.elements.splice(index + 1, 0, angular.copy($scope.currentScoring.elements[index]));
        };

        $scope.addScoringElement = function () {
            if (!$scope.currentScoring.elements) {
                $scope.currentScoring.elements = [];
            }
            $scope.currentScoring.elements.push({condition: {}, value: 1});
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemScoringApp').controller('ScoringListController', ScoringListController);

    ScoringListController.$inject = ['$scope', 'unomiScoringService', 'unomiQueryService', '$location', 'DTOptionsBuilder', 'DTColumnBuilder', 'i18nService', '$compile', 'dtPagerService', 'loadingSpinnerService'];

    function ScoringListController($scope, unomiScoringService, unomiQueryService, $location, DTOptionsBuilder, DTColumnBuilder, i18nService, $compile, dtPagerService, loadingSpinnerService) {
        $scope.items = {};
        $scope.stopPolling = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                if (prop == "scoringType") {
                    prop = "metadata.tags";
                } else {
                    prop = "metadata." + prop;
                }
                return prop + ":" + o.dir
            }).join(",");

            unomiScoringService.query(unomiScoringService.constructQuery(length, start, order)).then(function (response) {
                fnCallback(getResult(draw, response.data, unomiScoringService));
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            },
            function () {
                fnCallback(getErrorResult());
            });
        }

        var getResult = function(draw, data, service) {
            angular.forEach(data.list, processItem);

            $scope.stopPolling = false;

            return {
                "sEcho": draw,
                "iTotalRecords": data.totalSize,
                "iTotalDisplayRecords": data.totalSize,
                "aaData": data.list
            };
        };

        var getErrorResult = function (draw) {
            $scope.searchError = true;
            return {
                "sEcho": draw,
                "iTotalRecords": 0,
                "iTotalDisplayRecords": 0,
                "aaData": []
            };
        };

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('lengthMenu', [ 25, 50, 100 ])
            .withOption('pageLength', 25)
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 'lt')
            .withOption('aaSorting', [])
            .withOption('createdRow', createdRow)
            .withFnServerData(serverData)
        ;

        $scope.dtColumns = [
            DTColumnBuilder
                .newColumn("name")
                .withTitle(i18nService.message("serverSettings.manageScoring.scoringName"))
                .renderWith(renderNameCol)
                .withOption("width","35%"),
            DTColumnBuilder
                .newColumn("scoringType")
                .withTitle(i18nService.message("serverSettings.manageScoring.type"))
                .withOption("width","10%"),
            DTColumnBuilder
                .newColumn("description")
                .withTitle(i18nService.message("serverSettings.manageWem.description"))
                .withOption("width","40%"),
            DTColumnBuilder
                .newColumn(null)
                .withTitle(i18nService.message("serverSettings.manageWem.listAction"))
                .notSortable()
                .renderWith(actionsHtml)
                .withOption("width","15%")
                .withOption("contentPadding","0")
        ];

        $scope.setDtInstance = function(instance) {
            $scope.dtInstance = instance;
        };

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: "+row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        function actionsHtml(data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.manageScoring.editScoring") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                "<button title=\"" + i18nService.message("serverSettings.manageScoring.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><i class=\"material-icons\">person</i></button>" +
                "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><i class=\"material-icons\">delete</i></button>";
        }

        $scope.goToEdit = function(id) {
            $scope.stopPolling = true;
            $location.path( "/scoring/" +id );
        };

        $scope.goToProfiles = function(id) {
            $scope.stopPolling = true;
            $location.path( "/scoring/" +id + "/profiles" );
        };

        function renderNameCol(data, type, full, meta) {
            var result = data + ' <span class="badge deep-purple-bg">{{items["' + full.id + '"].count}}</span>';
            if(full.isShared) {
                result += ' <span class="badge  mf-pink-bg">' + i18nService.message("serverSettings.manageScoring.shared") + '</span>'
            }
            if(!full.enabled) {
                result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
            }
            return result;
        }

        function getCount (scoringId) {
            var scoringCondition = {
                "parameterValues": {
                    "subConditions": [
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "mergedWith",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "systemProperties.isAnonymousProfile",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            parameterValues: {
                                propertyName:"scores." + scoringId,
                                comparisonOperator: "exists"
                            },
                            type: "profilePropertyCondition"
                        },
                        {
                            parameterValues: {
                                propertyName:"scores." + scoringId,
                                comparisonOperator: "greaterThan",
                                propertyValueInteger: 0
                            },
                            type: "profilePropertyCondition"
                        }
                    ],
                    "operator": "and"
                },
                "type": "booleanCondition"
            };

            var observable =Rx.Observable.create( function(o){
                unomiQueryService.getMatchingItemsCountForCondition('profile', scoringCondition)
                .then(function(response){
                    o.onNext(response.data)
                    o.onCompleted();
                })
            });

            return observable.merge(
                    Rx.Observable
                        .interval(5000)
                        .flatMapLatest(observable)
                        .takeWhile(function (x) { return !$scope.stopPolling; })
                );
        }

        var processItem = function (itemMetadata) {
            $scope.items[itemMetadata.id] = itemMetadata;
            itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
            itemMetadata.count = 0;
            if(_.isEmpty(itemMetadata.tags)){
                itemMetadata.scoringType = i18nService.message("serverSettings.manageScoring.type.all");
            }else{
                //tag.demographicProfileProperties.name
                var type = _.find(itemMetadata.tags, function (tag) {
                    if(tag.indexOf('st:')>=0){
                        return true;
                    }
                }).split(':')[1];
                if(type){
                    itemMetadata.scoringType = i18nService.message("tag."+type+".name");
                }else{
                    itemMetadata.scoringType = "";
                }
            }
            $scope.items[itemMetadata.id].observable = getCount(itemMetadata.id).subscribe(function(count){
                $scope.items[itemMetadata.id].count = count;
            });
        };

        $scope.deleteItem = function (id) {
            $scope.tryDeleteItem($scope.items[id], function() {
                unomiScoringService.deletedItem = id;
                $scope.dtInstance.DataTable.draw();
            });
        };
    }
})();
