(function() {
    'use strict';

    angular.module('wemSegmentsApp', ['wemApp', 'wemProfilesApp'])
})();

(function() {
    'use strict';

    angular.module('wemSegmentsApp').run(wemSegmentsAppRun);

    wemSegmentsAppRun.$inject = ['$rootScope', '$timeout', 'i18nService', 'modalService', 'unomiListService', 'unomiSegmentService', 'loadingSpinnerService'];

    function wemSegmentsAppRun($rootScope, $timeout, i18nService, modalService, unomiListService, unomiSegmentService, loadingSpinnerService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.tryDeleteItem = function (segmentOrList, isStaticList, staticListCB, segmentCB) {
            // if the segment we're trying to delete is shared, display a modal to ask confirmation
            if (segmentOrList.scope != ManagersContext.dmfScope) {
                modalService.newAlertModal({
                    title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.title" : "serverSettings.manageSegments.delete.sharedWarning.title", segmentOrList.name),
                    bodyMessage: i18nService.message(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.message" : "serverSettings.manageSegments.delete.sharedWarning.message")
                }, function () {
                    if(isStaticList) {
                        deleteList(segmentOrList, staticListCB);
                    } else {
                        deleteSegment(segmentOrList, segmentCB);
                    }
                });
            } else {
                modalService.newAlertModal({
                    title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.title" : "serverSettings.manageSegments.delete.title", segmentOrList.name),
                    bodyMessage: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.message" : "serverSettings.manageSegments.delete.message", segmentOrList.name)
                }, function () {
                    if(isStaticList) {
                        deleteList(segmentOrList, staticListCB);
                    } else {
                        deleteSegment(segmentOrList, segmentCB);
                    }
                });
            }
        };

        var deleteList = function (list, callback) {
            loadingSpinnerService.show();
            unomiListService.deleteList(list.id);
            $timeout(function () {
                loadingSpinnerService.hide();
                if (callback) {
                    callback();
                }
            }, 1000);
        };

        var deleteSegment = function(segment, callback) {
            loadingSpinnerService.show();
            unomiSegmentService.deleteSegment(segment.id, true).then(function (response) {
                var data = response.data;
                // if data isn't empty, it contains the list of the segments that use the segment we're trying to delete as a condition
                if (data.segments.length > 0 || data.scorings.length > 0) {
                    loadingSpinnerService.hide();
                    // display the list of dependent segments and ask what to do
                    modalService.newAlertModal({
                        title: i18nService.format("serverSettings.manageSegments.delete.dependentsModal.title", segment.name),
                        bodyTemplateUrl: ManagersContext.wemAppPath + "/wem-segments/segment-delete-dependents-modal-body.html",
                        segmentName: segment.name,
                        dependents: data
                    }, function () {
                        loadingSpinnerService.show();
                        unomiSegmentService.deleteSegment(segment.id, false);
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            if (callback) {
                                callback();
                            }
                        }, 1000);
                    });
                } else {
                    $timeout(function () {
                        loadingSpinnerService.hide();
                        if (callback) {
                            callback();
                        }
                    }, 1000);
                }
            }, function() {
                loadingSpinnerService.hide();
                if (callback) {
                    callback();
                }
            });
        }
    }
})();

(function() {
    'use strict';

    angular.module('wemSegmentsApp').controller('SegmentEditController', SegmentEditController);

    SegmentEditController.$inject = ['$scope', '$resource', '$location', '$route', 'unomiSegmentService', 'unomiListService', 'currentSegment', 'newSegment', 'unomiDefinitionService', 'unomiQueryService', 'i18nService', 'notificationService', 'loadingSpinnerService', '_'];

    function SegmentEditController($scope, $resource, $location, $route, unomiSegmentService, unomiListService, currentSegment, newSegment, unomiDefinitionService, unomiQueryService, i18nService, notificationService, loadingSpinnerService, _) {
        $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
        $scope.currentSegment = currentSegment;
        $scope.deleteAllowed = !_.contains(['contacts', 'leads'], currentSegment.metadata.id);
        $scope.conditionTypes = [];
        $scope.sharedSegment = currentSegment.metadata.scope != ManagersContext.dmfScope;
        $scope.newSegment = newSegment || false;
        $scope.cannotShareTooltip = i18nService.message('serverSettings.manageSegments.sharedCannotEdit');

        if(!$scope.isStaticList) {
            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

            $scope.tags = {};

            $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
                console.log("segment-edit.controller.js : tags loaded from Apache Unomi.");

                angular.forEach(value.subTags, function (v) {
                    $scope.tags[v.id] = v
                });
                unomiDefinitionService.fillTags("conditionTypes", $scope);
                unomiDefinitionService.fillTags("eventConditionTypes", $scope);
            });

            $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
                console.log("segment-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
            });
            $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
                console.log("segment-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
            });

            unomiSegmentService.getImpacted(currentSegment.metadata.id).then(function(data){
                $scope.dependents = data;
            });

            $scope.resetSegment = function ($event) {
                unomiSegmentService.getSegment(currentSegment.metadata.id).then(function (segmentData) {
                    currentSegment = segmentData;
                    $route.reload();
                });
            };

            $scope.copySegment = function ($event) {
                unomiSegmentService.segmentToCopy = angular.copy(currentSegment);
                $location.path("/newSegment");
            };

            $scope.hasError = function(propertyName) {
                return ManagersUtils.hasErrors($scope.segmentForm, $scope.segmentForm, propertyName);
            };
        }

        $scope.saveSegment = function (segmentForm) {
            if(segmentForm.$invalid){
                segmentForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(segmentForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            var type = $scope.isStaticList ? 'userList' : 'segment';
            var errorKey = $scope.isStaticList ? "serverSettings.manageLists.create.alreadyExisting" : "serverSettings.manageSegments.create.alreadyExisting";

            var save = function () {
                if ($scope.isStaticList) {
                    unomiListService.setList($scope.currentSegment, $scope.sharedSegment).then(function () {
                        $location.path("/lists")
                    });
                } else {
                    unomiSegmentService.setSegment($scope.currentSegment, $scope.sharedSegment).then(function () {
                        $location.path("/segments")
                    });
                }
            };

            loadingSpinnerService.show();
            if ($scope.newSegment) {
                unomiQueryService.createWithCallbackIfDoesNotExist(type, ManagersContext.dmfScope, $scope.currentSegment.metadata.name, null,
                    errorKey, $scope.currentSegment.metadata.name, save, function() {
                        loadingSpinnerService.hide();
                    });
            } else {
                save();
            }
        };

        $scope.deleteCurrent = function () {
            var id = $scope.currentSegment.metadata.id;
            $scope.tryDeleteItem($scope.currentSegment.metadata, $scope.isStaticList,
            function() {
                $location.path('/lists');
            }, function() {
                unomiSegmentService.deletedItem = id;
                $location.path('/segments');
            });
        };
    }
})();

(function() {
    'use strict';

    angular.module('wemSegmentsApp').controller('SegmentsListController', SegmentsListController);

    SegmentsListController.$inject = ['$scope', 'unomiSegmentService', 'unomiQueryService', 'unomiListService', '$location', 'DTColumnBuilder', 'i18nService', '$compile', 'dataTableService', 'dtPagerService'];

    function SegmentsListController($scope, unomiSegmentService, unomiQueryService, unomiListService, $location, DTColumnBuilder, i18nService, $compile, dataTableService, dtPagerService) {
        $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
        $scope.items = {};
        $scope.stopPolling = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return "metadata." + ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
            }).join(",");

            if($scope.isStaticList){
                unomiListService.query(unomiListService.constructQuery(length, start, order)).then(function (response) {
                    fnCallback(getResult(draw, response.data, unomiListService));
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    fnCallback(getErrorResult());
                });
            } else {
                unomiSegmentService.query(unomiSegmentService.constructQuery(length, start, order)).then(function (response) {
                    fnCallback(getResult(draw, response.data, unomiSegmentService));
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    fnCallback(getErrorResult());
                });
            }
        }

        var getResult = function(draw, data, service) {
            angular.forEach(data.list, processItem);

            $scope.stopPolling = false;

            return {
                "sEcho": draw,
                "iTotalRecords": data.totalSize,
                "iTotalDisplayRecords": data.totalSize,
                "aaData": data.list
            };
        };

        var getErrorResult = function (draw) {
            $scope.searchError = true;
            return {
                "sEcho": draw,
                "iTotalRecords": 0,
                "iTotalDisplayRecords": 0,
                "aaData": []
            };
        };

        $scope.dtOptions = dataTableService.generateDataTableOptions({
            pageLength: 25,
            lengthMenu: [ 25, 50, 100 ],
            processing: true,
            serverSide: true,
            sDom: 'lt',
            aaSorting: [],
            createdRow: function (row, data, dataIndex) {
                console.log("Row is created: "+row);
                // Recompiling so we can bind Angular directive to the DT
                $compile(angular.element(row).contents())($scope);
            }
        });

        $scope.dtOptions.withFnServerData(serverData);

        $scope.dtColumns = [
            DTColumnBuilder
                .newColumn("name")
                .withTitle(i18nService.message($scope.isStaticList ? "serverSettings.manageLists.list.name" : "serverSettings.manageSegments.segmentName"))
                .withOption("width","35%")
                .renderWith(renderNameCol),

            DTColumnBuilder.newColumn("description")
                .withTitle(i18nService.message("serverSettings.manageWem.description"))
                .withOption("width","50%"),

            DTColumnBuilder.newColumn(null)
                .withTitle(i18nService.message("serverSettings.manageWem.listAction"))
                .notSortable()
                .renderWith(actionsHtml)
                .withOption("width","15%")
                .withOption("contentPadding","0")
        ];

        $scope.setDtInstance = function(instance) {
            $scope.dtInstance = instance;
        };

        function actionsHtml(data, type, full, meta) {

            var actions = "<button title=\"" + i18nService.message($scope.isStaticList ? "serverSettings.manageSegments.editList" : "serverSettings.manageSegments.editSegment") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                "<button title=\"" + i18nService.message("serverSettings.manageSegments.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><i class=\"material-icons\">person</i></button>";
            if( !_.contains(['contacts', 'leads'], data.id)){
                actions +=  "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><i class=\"material-icons\">delete</i></button>";
            }
            return actions;
        }

        $scope.goToEdit = function(id) {
            $scope.stopPolling = true;
            $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id );
        };

        $scope.goToProfiles = function(id) {
            $scope.stopPolling = true;
            $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id + "/profiles" );
        };

        $scope.goToNewSegment = function() {
            $scope.stopPolling = true;
            $location.path( "/newSegment");
        };

        $scope.goToNewList = function() {
            $scope.stopPolling = true;
            $location.path( "/newList");
        };

        function renderNameCol(data, type, full, meta) {
            var result = data + ' <span class="badge deep-purple-bg">{{items["' + full.id + '"].count}}</span>';
            if(full.isShared) {
                result += ' <span class="badge  mf-pink-bg">' + i18nService.message("serverSettings.manageSegments.shared") + '</span>'
            }
            if(!full.enabled) {
                result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
            }
            return result;
        }

        function getCount (segmentId) {
            var segmentCondition = {
                type:"booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "profilePropertyCondition",

                            parameterValues: {
                                propertyName: $scope.isStaticList ? "systemProperties.lists" : "segments",
                                comparisonOperator: "equals",
                                propertyValue: segmentId
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "mergedWith",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "systemProperties.isAnonymousProfile",
                                comparisonOperator: "missing"
                            }
                        }],
                    operator: "and"
                }
            };

            var observable = Rx.Observable.create(function(o) {
                unomiQueryService.getMatchingItemsCountForCondition('profile', segmentCondition)
                .then(function(response) {
                    o.onNext(response.data);
                    o.onCompleted();
                })
            });
            return observable.merge(
                Rx.Observable
                    .interval(5000)
                    .flatMapLatest(observable)
                    .takeWhile(function (x) { return !$scope.stopPolling; })
            );
        }


        var processItem = function (itemMetadata) {
            $scope.items[itemMetadata.id] = itemMetadata;
            itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
            $scope.items[itemMetadata.id].observable = getCount(itemMetadata.id).distinctUntilChanged().subscribe(function(count){
                $scope.items[itemMetadata.id].count = count;
            });
        };

        $scope.deleteItem = function (id) {
            $scope.tryDeleteItem($scope.items[id], $scope.isStaticList, function() {
                $scope.dtInstance.DataTable.draw();
            }, function() {
                unomiSegmentService.deletedItem = id;
                $scope.dtInstance.DataTable.draw();
            });
        };
    }
})();
