(function() {
    'use strict';

    angular.module('wemApp.components').controller('SetPropertyActionController', SetPropertyActionController);

    SetPropertyActionController.$inject = ['i18nService', 'unomiProfileService', '_'];

    function SetPropertyActionController(i18nService, unomiProfileService, _) {

        var vm = this;

        vm.selectProperty = selectProperty;
        vm.changeProperties = changeProperties;
        vm.setParentRef = setParentRef;
        vm.changeDate = changeDate;
        vm.changeDateIsNow = changeDateIsNow;
        vm.setCustomPropertyName = setCustomPropertyName;

        _init();

        // public functions

        function setParentRef(parentRef) {
            vm.parentRef = parentRef;
            vm.propertyAction.storeInSession = '' + (vm.parentRef.parameterValues != null &&
                vm.parentRef.parameterValues.storeInSession != null &&
                vm.parentRef.parameterValues.storeInSession);

            _loadPropertyTypes();
        }

        function selectProperty() {
            vm.propertyAction.predefinedProperty = vm.propertyAction.selectedProperty.metadata.id != 'user-specified-property';
            vm.parentRef.parameterValues.setPropertyName = vm.propertyAction.predefinedProperty ? vm.propertyAction.selectedProperty.metadata.id : '';
            vm.parentRef.parameterValues.setPropertyValue = null;
            console.debug("Selected property", vm.propertyAction.selectedProperty);
        }

        function changeProperties() {
            vm.parentRef.parameterValues.storeInSession = vm.propertyAction.storeInSession == 'true';
            vm.propertyAction.selectedProperty = null;
            vm.propertyAction.selectedDate = null;
            vm.propertyAction.selectedDateIsNow = false;
            vm.parentRef.parameterValues.setPropertyName = null;
            vm.parentRef.parameterValues.setPropertyValue = null;
            console.info("Store in session=", vm.propertyAction.storeInSession);
            _loadPropertyTypes();
        }

        function changeDate() {
            vm.propertyAction.selectedDateIsNow = false;
            vm.parentRef.parameterValues.setPropertyValue = vm.propertyAction.selectedDate;
        }

        function changeDateIsNow() {
            if (vm.propertyAction.selectedDateIsNow) {
                vm.propertyAction.selectedDate = null;
                vm.parentRef.parameterValues.setPropertyValue = "now";
            }
        }

        function setCustomPropertyName() {
            if (vm.propertyAction.customPropertyName) {
                if (vm.propertyAction.customPropertyName.indexOf("properties.") == 0) {
                    vm.parentRef.parameterValues.setPropertyName = vm.propertyAction.customPropertyName;
                } else {
                    vm.parentRef.parameterValues.setPropertyName = "properties." + vm.propertyAction.customPropertyName;
                }
            }
        }

        // private functions

        function _loadPropertyTypes() {
            if (vm.propertyAction.storeInSession == 'true') {
                unomiProfileService.getPropertyTypesByTag("sessionProperties", true).then(_processLoadedProperties);
            } else {
                unomiProfileService.getPropertyTypesByTag("profileProperties", true).then(_processLoadedProperties);
            }
        }

        function _processLoadedProperties(response) {
            var data = response.data;
            // we filter multi-valued property types because they are not (yet) supported by the UI.
            // todo add support for multi-valued properties
            data = _.filter(data, function (property) {
                if (property.multivalued) {
                    console.info("Removing multivalued property type ", property, " because they are not supported by the UI (yet)");
                    return false;
                }
                return true;
            });
            data.forEach(function(prop) {
                prop.metadata.id = "properties." + prop.metadata.id;
            });
            data.push(vm.otherPropertyType);
            if (vm.propertyAction.storeInSession == 'true') {
                vm.sessionProperties = data;
                vm.propertyAction.propertyTypes = vm.sessionProperties;
            } else {
                vm.profileProperties = data;
                vm.propertyAction.propertyTypes = vm.profileProperties;
            }

            var propertyTypesIds = [];
            vm.propertyAction.propertyTypes.forEach(function(prop) {
                if (prop.metadata.id.indexOf("properties.") == 0) {
                    propertyTypesIds.push(prop.metadata.id);
                }
            });

            vm.propertyAction.predefinedProperty = vm.parentRef.parameterValues.setPropertyName == null ||
                vm.parentRef.parameterValues.setPropertyName.length == 0 ||
                propertyTypesIds.indexOf(vm.parentRef.parameterValues.setPropertyName) > -1;

            if (vm.propertyAction.predefinedProperty) {
                var selectedProperty = _findPropertyTypeByName(vm.propertyAction.propertyTypes, vm.parentRef.parameterValues.setPropertyName);
                if (selectedProperty) {
                    vm.propertyAction.selectedProperty = selectedProperty;
                } else {
                    // this can happen on initialization or when switching between session or profile properties
                    vm.propertyAction.selectedProperty = null;
                }
            } else {
                vm.propertyAction.selectedProperty = vm.otherPropertyType;
                if (vm.parentRef.parameterValues.setPropertyName) {
                    if (vm.parentRef.parameterValues.setPropertyName.indexOf("properties.") == 0) {
                        vm.propertyAction.customPropertyName = vm.parentRef.parameterValues.setPropertyName.substring("properties.".length);
                    } else {
                        vm.propertyAction.customPropertyName = vm.parentRef.parameterValues.setPropertyName;
                    }
                }
            }

            if (vm.propertyAction.selectedProperty && vm.propertyAction.selectedProperty.type == "date") {
                if (vm.parentRef.parameterValues.setPropertyValue == "now") {
                    vm.propertyAction.selectedDateIsNow = true;
                    vm.propertyAction.selectedDate = null;
                } else {
                    vm.propertyAction.selectedDateIsNow = false;
                    vm.propertyAction.selectedDate = vm.parentRef.parameterValues.setPropertyValue;
                }
            }
        }

        function _findPropertyTypeByName(propertyTypes, propertyTypeId) {
            for (var i = 0; i < propertyTypes.length; i++) {
                if (propertyTypes[i].metadata.id == propertyTypeId) {
                    return propertyTypes[i];
                }
            }
        }

        function _init() {

            vm.propertyAction = {};
            vm.otherPropertyType = {
                metadata: {
                    id:'user-specified-property',
                    name:i18nService.message('action.setPropertyAction.other')
                },
                type:'string',
                multivalued: false
            };

        }

    }
})();