(function() {
    'use strict';

    angular.module('wemApp').config(wemAppConfig);

    wemAppConfig.$inject = ['$routeProvider'];

    function wemAppConfig($routeProvider) {
        $routeProvider
            .when('/context-server-issue', {
                template: '<wem-context-server-issue></wem-context-server-issue>'
            });
    }

    angular.module('wemApp.components').config(wemAppComponentsConfig);

    wemAppComponentsConfig.$inject = ['$sceDelegateProvider', 'uiGmapGoogleMapApiProvider'];

    function wemAppComponentsConfig($sceDelegateProvider, uiGmapGoogleMapApiProvider) {
        // THIS IS FOR THE ACTIONS AND THE PROPERTIES
        // we need to normalize the URL as angular uses normalized URLs to check against the white list
        var urlParsingNode = document.createElement('a');
        urlParsingNode.setAttribute('href', ManagersContext.proxyServletUrl);
        var normalizedProxyServletUrl = urlParsingNode.href;

        $sceDelegateProvider.resourceUrlWhitelist([
            // Allow same origin resource loads.
            'self',
            // Allow loading from our assets domain.  Notice the difference between * and **.
            normalizedProxyServletUrl + '**'
        ]);

        // condition-mapCondition.controller.js and wemPersonaPanelApp
        uiGmapGoogleMapApiProvider.configure({
            key: ManagersContext.googleAPIKey,
            v: '3.17',
            libraries: 'weather,geometry,visualization'
        });
    }

    if (!wemAppIsNewUI) {
        angular.module('wemApp.ui').config(wemAppOldUIConfig);

        wemAppOldUIConfig.$inject = ['$httpProvider', 'NotificationProvider'];
    } else {
        angular.module('wemApp.ui').config(wemAppNewUIConfig);

        wemAppNewUIConfig.$inject = ['$mdThemingProvider'];
    }

    function wemAppOldUIConfig($httpProvider, NotificationProvider) {
        $httpProvider.defaults.headers.common['Accept-Language'] = ManagersContext.jahiaUILocale;

        $httpProvider.interceptors.push(function ($q, i18nService) {
            return {
                responseError: function (rejection) {
                    if (rejection.status != 404) {
                        if (rejection.data && rejection.data.error) {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.data.error);
                        } else if (rejection.data && rejection.data != null) {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.data);
                        } else {
                            ManagersUtils.notificationService.errorToast("Error : " + rejection.statusText);
                        }
                    }
                    if (rejection.config.url.match(/templates\/condition/)) {
                        rejection.data = i18nService.message("condition.unknown");
                        return rejection;
                    }

                    return $q.reject(rejection);
                }
            };
        });

        NotificationProvider.setOptions({
            startTop: 20,
            startRight: 20,
            verticalSpacing: 20,
            horizontalSpacing: 20,
            positionX: 'left',
            positionY: 'bottom'
        });
    }

    function wemAppNewUIConfig($mdThemingProvider) {
        $mdThemingProvider.theme('error-toast');

        // Add jahia blue as default 500 for blue palette
        var jahiaBlue = $mdThemingProvider.extendPalette('blue', {
            '500': '#00A0E3',
            'contrastDefaultColor': 'light'
        });

        var mfPink = $mdThemingProvider.extendPalette('pink', {
            '500': '#F21D8E',
            'contrastDefaultColor': 'light'
        });

        $mdThemingProvider.definePalette('jahia-blue', jahiaBlue);

        $mdThemingProvider.definePalette('mf-pink', mfPink);

        $mdThemingProvider.theme('wemApp')
            .primaryPalette('jahia-blue')
            .accentPalette('deep-purple')
            .warnPalette('mf-pink');

        $mdThemingProvider.setDefaultTheme('wemApp');
    }
})();