(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemFoldableInput', wemFoldableInput);

    wemFoldableInput.$inject = ['i18nService'];

    function wemFoldableInput(i18nService) {
        return {
            restrict: 'E',
            scope: {
                model: '=',
                name: '@',
                multivalued: '=',
                required: '=',
                labelKey: '@',
                label: '@',
                altClass: '@',
                type: '@',
                disabled: '=',
                pattern: '@',
                hasError: '='
            },
            transclude:true,
            controller: WemFoldableInputController,
            link: linkFunc,
            templateUrl:  ManagersContext.wemAppPath + "/components/foldable-input/base.directive.html"
        };

        function linkFunc(scope, element, attrs) {
            if (scope.labelKey && i18nService.contains(scope.labelKey)) {
                scope.label = i18nService.message(scope.labelKey);
            }
            if (scope.required) {
                scope.label = scope.label + " (" + i18nService.message('label.required') + ")";
            }
            if (attrs.type !== "boolean") {
                element.find("label").text(scope.label).click(function () {
                    // need for IE that doesn't support  pointer-events: none; in CSS
                    if (attrs.type == "textarea") {
                        element.find("textarea").focus();
                    } else {
                        element.find("input").focus();
                    }
                });
            }

            if (attrs.type == "textarea") {
                scope.templateUrl = ManagersContext.wemAppPath + "/components/foldable-input/textarea.directive.html";
            } else if(attrs.type == "date") {
                scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/date.directive.html";
            } else if(attrs.type == "boolean") {
                scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/boolean.directive.html";
            } else {
                if(scope.multivalued){
                    scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/multipleInput.directive.html";
                }else{
                    scope.templateUrl =  ManagersContext.wemAppPath + "/components/foldable-input/generic.directive.html";
                }
            }
        }
    }

    WemFoldableInputController.$inject = ['$scope', 'uibDatepickerConfig', 'i18nService'];

    function WemFoldableInputController($scope, uibDatepickerConfig, i18nService) {
        if ($scope.type == 'date' && $scope.model != null && typeof $scope.model == "string") {
            $scope.model = moment($scope.model).toDate();
        }

        $scope.inner = {
            model: $scope.model
        };

        $scope.tempValue = null;

        var foldedClass;
        if ($scope.altClass) {
            foldedClass = "foldedLabel " + $scope.altClass;
        } else {
            foldedClass = "foldedLabel";
        }

        function updateLabel(newValue) {
            if (((newValue != null && (typeof newValue == "number" || newValue != '')) && !$scope.multivalued) || $scope.type == "boolean") {
                $scope.labelClass = foldedClass;
            } else {
                $scope.labelClass = "";
            }
        }

        $scope.focusInput = function (event) {
            $scope.labelClass = foldedClass;
            $scope.focused = true;
            if ($scope.type == "date") {
                $scope.datepicker.datePickerOpen = true;
                event.target.blur()
            }
        };

        $scope.blurInput = function (event) {
            //Callback for blur event defined on the parent Scope
            if ($scope.type != "date" && $scope.$parent["blurCallback"]) {
                $scope.$parent["blurCallback"](event);
            }
            updateLabel(event.target.value);
            $scope.focused = false;
        };

        $scope.$watch("inner.model", function(newVal) {
            $scope.model = newVal;
        });

        $scope.$watch("model", function (newValue, oldValue) {
            $scope.inner.model = newValue;
            if (!$scope.focused) {
                updateLabel(newValue);
            }
        });

        updateLabel($scope.model);
        $scope.focused = false;

        if ($scope.type == 'date') {
            uibDatepickerConfig.startingDay = 1;
            uibDatepickerConfig.minMode = "day";
            uibDatepickerConfig.datepickerMode = "day";
            uibDatepickerConfig.showWeeks = false;

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                $scope.datepicker.datePickerOpen = true;
            };
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.clearText = i18nService.message('wem.label.clear');
            $scope.datepicker = {
                datePickerOpen: false
            };
        }

        $scope.onDeleteMultiValue = function (property, value) {
            return _.reject(property, function(val){ return val === value; });
        };

        $scope.onAddMultiValue = function (property, newValue) {
            if (!property) {
                property = [];
            }
            if (!Array.isArray(property)) {
                property = Array.of(property);
            }
            if (newValue && newValue != null && newValue != '') {
                if (!_.contains(property, newValue)) {
                    property.push(angular.copy(newValue));
                }
            }

            return property;
        };
    }
})();