(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemNodePicker', wemNodePicker);

    function wemNodePicker() {
        return {
            restrict: "E",
            scope: {
                selectedPath: '=ngModel',
                selectedId: '=',
                nodeTypes: '@',
                selectableNodeTypes: '@',
                displayableNodesOnly: '@',
                basePath: '@',
                baseUrl: '@',
                label: '@'
            },
            link: linkFunc,
            templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/pickers/wem-node-picker-modal.directive.html" : "/components/pickers/wem-node-picker-modal.directive.legacy.html"),
            controller: WemNodePickerController
        };

        function linkFunc(scope, element, attrs) {
            if (!attrs.selectedId) {
                scope.selectedIdSpecified = false;
            } else {
                scope.selectedIdSpecified = true;
            }
        }
    }

    WemNodePickerController.$inject = ['$scope', 'modalService'];

    function WemNodePickerController($scope, modalService) {
        $scope.openPicker = function () {
            var modalData = {
                directiveScope : $scope,
                init : function ($scope, i18nService, modalData) {
                    $scope.root = {};
                    $scope.base = (modalData.directiveScope.baseUrl && ManagersContext[modalData.directiveScope.baseUrl]) ? ManagersContext[modalData.directiveScope.baseUrl] : ManagersContext.baseLive;

                    function selectInTree() {
                        if ($scope.selectedPath) {
                            $scope.root.children.forEach(function (child) {
                                if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                    $scope.toggle(child, selectItemInTree)
                                }
                            });
                        }
                    }

                    function selectItemInTree(item) {
                        if ($scope.selectedPath && item && item.hasChildren && item.children.length > 0) {
                            item.displayed = true;
                            item.children.forEach(function (child) {
                                if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                    $scope.toggle(child, selectItemInTree)
                                }
                            });
                        }
                    }

                    if (!modalData.directiveScope.basePath) {
                        modalData.directiveScope.basePath = "";
                    }

                    // init get root node
                    $.ajax({
                        type: "GET",
                        dataType: "json",
                        url: $scope.base + ManagersContext.currentSitePath + modalData.directiveScope.basePath + ".treeRootItem.json?nodeTypes=" + modalData.directiveScope.nodeTypes
                        + (modalData.directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + modalData.directiveScope.selectableNodeTypes : "")
                        + (modalData.directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                    }).done(function (data) {
                        $scope.$apply(function () {
                            if (data && data.length > 0) {
                                $scope.root = data[0];
                                $scope.root.displayed = true;
                                $scope.$watch('selectedPath', selectInTree);
                            }
                        });
                    });

                    $scope.getIcon = function (isRoot, item) {
                        if (isRoot) {
                            return "web";
                        }

                        if (item) {
                            if ($scope.isSelectable(item)) {
                                return "insert_drive_file";
                            } else {
                                return "folder_open"
                            }
                        }

                        return "insert_drive_file";
                    };

                    $scope.getArrow = function (item) {
                        if (item.hasChildren) {
                            if (item.displayed) {
                                return "keyboard_arrow_down";
                            } else {
                                return "keyboard_arrow_right";
                            }
                        }
                        return "";
                    };

                    $scope.getItemClass = function (item) {
                        if ($scope.isSelected(item)) {
                            return "selected";
                        }

                        if ($scope.isSelectable(item)) {
                            return "selectable";
                        }
                    };

                    $scope.isSelectable = function (item) {
                        return "selectable" == item.classes;
                    };

                    $scope.isSelected = function (item) {
                        return item.path == $scope.selectedPath;
                    };

                    $scope.select = function (item) {
                        if ("selectable" == item.classes) {
                            modalData.item = item;
                            $scope.ok();
                        }
                    };

                    $scope.toggle = function (item, cb) {
                        if (item.hasChildren) {
                            if (item.children && item.children.length > 0) {
                                item.displayed = !item.displayed;
                                if (cb) {
                                    cb(item)
                                }
                            } else {
                                $.ajax({
                                    type: "GET",
                                    dataType: "json",
                                    url: $scope.base + item.path + ".treeRootItem.json?nodeTypes=" + modalData.directiveScope.nodeTypes
                                    + (modalData.directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + modalData.directiveScope.selectableNodeTypes : "")
                                    + (modalData.directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                                }).done(function (data) {
                                    var children = data && data.length > 0 ? data[0].children : [];
                                    $scope.$apply(function () {
                                        item.children = children;
                                        item.displayed = true;
                                        if (cb) {
                                            cb(item)
                                        }
                                    });
                                });
                            }
                        }
                    };
                },
                templateUrl: ManagersContext.wemAppPath + (wemAppIsNewUI ? "/components/pickers/wem-node-picker.directive.html" : "/components/pickers/wem-node-picker.directive.legacy.html")
            };

            var okCallback = function (modalData) {
                $scope.selectedPath = modalData.item.path;
                if ($scope.selectedIdSpecified) {
                    $scope.selectedId = modalData.item.id;
                }
            };

            var modalInstance = modalService.newCustomModal(
                modalData,
                null,
                okCallback,
                null
            );

        }
    }
})();