(function() {
    'use strict';

    angular.module('wemApp.components').directive('wemProperty', wemPropertyDirective);

    function wemPropertyDirective() {
        return {
            restrict: 'E',
            scope: {
                ref: '=',
                types: '=',
                currentProfile: '='
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: WemPropertyDirectiveController
        };
    }

    WemPropertyDirectiveController.$inject = ['$scope'];

    function WemPropertyDirectiveController($scope) {

        function updateTemplate() {
            if ($scope.ref && $scope.ref.type != "empty") {
                $scope.valueType = $scope.getValueType($scope.ref);
                if ($scope.valueType != null) {
                    $scope.templateUrl = ManagersContext.wemAppPath + "/properties/templates/" + $scope.valueType.id + ".html";
                }
            }
        }

        $scope.getValueType = function (property) {
            for (var i = 0; i < $scope.types.length; i++) {
                var valueType = $scope.types[i];
                if (valueType.id == property.type) {
                    return valueType;
                }
            }
            return null;
        };

        if (!$scope.ref) {
            console.log("wem-property.directive.js: no property passed, creating empty one.");
            $scope.ref = {};
            $scope.ref.type = "empty";
        }

        console.log("wem-property.directive.js: Initializing property controller for property " + $scope.ref + " of type=" + $scope.ref.type);

        updateTemplate();

        $scope.$watch("ref", function (newValue, oldValue) {
            console.log("wem-property.directive.js : property changed, refreshing...");
            updateTemplate();
        });

        $scope.$watch("types.length", function (newValue, oldValue) {
            console.log("wem-property.directive.js : property type list changed, refreshing...");
            updateTemplate();
        });

    }
})();
