(function() {
    'use strict';

    angular.module('wemApp.components').controller('CountryListController', CountryListController);

    CountryListController.$inject = ['$scope', 'unomiGeonamesService'];

    function CountryListController($scope, unomiGeonamesService) {

        var populateScope = function (response) {
            var data = response.data;
            angular.forEach(data.reverse(), function (entry) {
                if (entry.featureCode.indexOf('PCL') == 0) {
                    $scope.country = entry;
                } else if (entry.featureCode == 'ADM1') {
                    $scope.admin1 = entry;
                } else if (entry.featureCode == 'ADM2') {
                    $scope.admin2 = entry;
                } else if (entry.featureCode.indexOf('PPL') == 0) {
                    $scope.city = entry;
                }
            });
        };

        var fillSelects = function (response) {
            var array = response.data;
            if (array.list[0]) {
                var entry = array.list[0];
                if (entry.featureCode == 'ADM1') {
                    $scope.adm1Array = array.list;
                    $scope.adm1Array.unshift({"name": ""});
                } else if (entry.featureCode == 'ADM2') {
                    $scope.adm2Array = array.list;
                    $scope.adm2Array.unshift({"name": ""});
                } else if (entry.featureCode.indexOf('PPL') == 0) {
                    $scope.citiesArray = array.list;
                    $scope.citiesArray.unshift({"name": ""});
                }
            }
        };

        $scope.lookupFromCountryList = false;

        $scope.$watch('ref.parameterValues', function(n,o) {
            if ($scope.ref.parameterValues.city) {
                unomiGeonamesService.getHierarchy($scope.ref.parameterValues.city).then(populateScope);
            } else if ($scope.ref.parameterValues.admin2) {
                unomiGeonamesService.getHierarchy($scope.ref.parameterValues.admin2).then(populateScope);
            } else if ($scope.ref.parameterValues.admin1) {
                unomiGeonamesService.getHierarchy($scope.ref.parameterValues.admin1).then(populateScope);
            } else if ($scope.ref.parameterValues.country) {
                $scope.lookupFromCountryList = true;
                if ($scope.countries) {
                    $scope.country = _.find($scope.countries, function (item) {
                        return item.countryCode == $scope.ref.parameterValues.country
                    });
                }
            }
        });

        unomiGeonamesService.getEntries()
            .then(function (response) {
                $scope.countries = response.data.list;
                if ($scope.lookupFromCountryList) {
                    $scope.country = _.find(response.data.list, function (item) {
                        return item.countryCode == $scope.ref.parameterValues.country
                    });
                }
            });

        $scope.$watch('country', function (newValue, oldValue) {
            if (!_.isEqual(oldValue, newValue)) {
                delete $scope.ref.parameterValues.admin1;
                delete $scope.ref.parameterValues.admin2;
                delete $scope.ref.parameterValues.city;
                $scope.adm1Array = [];
                $scope.adm2Array = [];
                $scope.citiesArray = [];

                if (newValue && newValue != "") {
                    $scope.ref.parameterValues.country = newValue.countryCode;
                    unomiGeonamesService.getEntries(newValue.countryCode).then(fillSelects);
                }
            }
        });
        $scope.$watch('admin1', function (newValue, oldValue) {
            if (!_.isEqual(oldValue, newValue)) {
                delete $scope.ref.parameterValues.admin1;
                delete $scope.ref.parameterValues.admin2;
                delete $scope.ref.parameterValues.city;
                $scope.adm2Array = [];
                $scope.citiesArray = [];


                if (newValue && newValue.itemId) {
                    $scope.ref.parameterValues.admin1 = newValue.itemId;
                    unomiGeonamesService.getEntries(newValue.countryCode, newValue.admin1Code).then(fillSelects);
                }
            }
        });

        $scope.$watch('admin2', function (newValue, oldValue) {
            if (!_.isEqual(oldValue, newValue)) {
                delete $scope.ref.parameterValues.admin2;
                delete $scope.ref.parameterValues.city;
                $scope.citiesArray = [];

                if (newValue && newValue.itemId) {
                    $scope.ref.parameterValues.admin2 = newValue.itemId;
                    unomiGeonamesService.getEntries(newValue.countryCode, newValue.admin1Code, newValue.admin2Code).then(fillSelects);
                }
            }
        });

        $scope.$watch('city', function (newValue, oldValue) {
            if (!_.isEqual(oldValue, newValue)) {
                delete $scope.ref.parameterValues.city;
                if (newValue && newValue.itemId) {
                    $scope.ref.parameterValues.city = newValue.itemId;
                }
            }
        });
    }
})();
