(function() {
    'use strict';

    angular.module('wemApp.components').controller('MapConditionController', MapConditionController);

    MapConditionController.$inject = ['$scope'];

    function MapConditionController($scope) {
        $scope.options = {scrollwheel: false};

        $scope.init = function () {
            var isCircle = $scope.ref.parameterValues.type && $scope.ref.parameterValues.type == "circle";
            var isRect = $scope.ref.parameterValues.type && $scope.ref.parameterValues.type == "rectangle";

            $scope.circle =
            {
                id: 1,
                center: {
                    latitude: $scope.ref.parameterValues.circleLatitude != undefined ? $scope.ref.parameterValues.circleLatitude : 48,
                    longitude: $scope.ref.parameterValues.circleLongitude != undefined ? $scope.ref.parameterValues.circleLongitude : 2
                },
                radius: $scope.ref.parameterValues.distance != undefined ? $scope.ref.parameterValues.distance : 500000,
                stroke: {
                    color: '#08B21F',
                    opacity: 1
                },
                fill: {
                    color: '#08B21F',
                    opacity: 0.5
                },
                control: {},
                geodesic: true,
                draggable: true,
                clickable: true,
                editable: true,
                visible: isCircle ? true : false,
                events: {
                    center_changed: function (circle) {
                        $scope.ref.parameterValues.circleLatitude = circle.getCenter().lat();
                        $scope.ref.parameterValues.circleLongitude = circle.getCenter().lng();
                        $scope.ref.parameterValues.distance = circle.getRadius();
                    },
                    radius_changed: function (circle) {
                        $scope.ref.parameterValues.distance = circle.getRadius();
                    },
                    dragend: function (circle) {
                        $scope.ref.parameterValues.circleLatitude = circle.getCenter().lat();
                        $scope.ref.parameterValues.circleLongitude = circle.getCenter().lng();
                        $scope.ref.parameterValues.distance = circle.getRadius();
                    }
                }
            };

            $scope.rectangle = {
                bounds: {
                    ne: {
                        latitude: $scope.ref.parameterValues.rectLatitudeNE != undefined ? $scope.ref.parameterValues.rectLatitudeNE : 48,
                        longitude: $scope.ref.parameterValues.rectLongitudeNE != undefined ? $scope.ref.parameterValues.rectLongitudeNE : 11
                    },
                    sw: {
                        latitude: $scope.ref.parameterValues.rectLatitudeSW != undefined ? $scope.ref.parameterValues.rectLatitudeSW : 45,
                        longitude:  $scope.ref.parameterValues.rectLongitudeSW != undefined ? $scope.ref.parameterValues.rectLongitudeSW : 0
                    }
                },
                visible: isRect ? true : false,
                clickable: true,
                editable: true,
                stroke: {
                    color: '#AE25E9',
                    weight: 1.5,
                    opacity: 1
                },
                draggable: true,
                fill: {
                    color: '#AE25E9',
                    opacity: 0.5
                },
                events: {
                    bounds_changed: function (rectangle) {
                        if(rectangle.getBounds()) {
                            $scope.ref.parameterValues.rectLatitudeSW = rectangle.getBounds().getSouthWest().lat();
                            $scope.ref.parameterValues.rectLongitudeSW = rectangle.getBounds().getSouthWest().lng();
                            $scope.ref.parameterValues.rectLatitudeNE = rectangle.getBounds().getNorthEast().lat();
                            $scope.ref.parameterValues.rectLongitudeNE = rectangle.getBounds().getNorthEast().lng();
                        }
                    },
                    dragend: function (rectangle) {
                        if(rectangle.getBounds()) {
                            $scope.ref.parameterValues.rectLatitudeSW = rectangle.getBounds().getSouthWest().lat();
                            $scope.ref.parameterValues.rectLongitudeSW = rectangle.getBounds().getSouthWest().lng();
                            $scope.ref.parameterValues.rectLatitudeNE = rectangle.getBounds().getNorthEast().lat();
                            $scope.ref.parameterValues.rectLongitudeNE = rectangle.getBounds().getNorthEast().lng();
                        }
                    }
                }
            };

            var mapCenter = {
                latitude: 48,
                longitude: 2
            };
            if (isCircle) {
                mapCenter = $scope.circle.center;
            }
            if (isRect) {
                mapCenter = $scope.rectangle.bounds.ne;
            }
            $scope.map = {
                center: mapCenter,
                zoom: 2
            };
        };

        $scope.drawCircle = function () {
            $scope.ref.parameterValues.type = "circle";
            $scope.rectangle.visible = false;
            $scope.circle.visible = true;
            $scope.map.center = angular.copy($scope.circle.center);
        };

        $scope.drawRectangle = function () {
            $scope.ref.parameterValues.type = "rectangle";
            $scope.circle.visible = false;
            $scope.rectangle.visible = true;
            $scope.map.center = angular.copy($scope.rectangle.bounds.ne);
        };

        $scope.$watch("ref.parameterValues", function(newVal) {
            if(newVal) {
                $scope.init();
            }
        });
    }
})();