(function() {
    'use strict';
    angular.module('wemApp.services').service('autocompleteService', autocompleteService);
    autocompleteService.$inject = ['_','unomiProfileService', 'unomiQueryService'];


    function autocompleteService(_,unomiProfileService,unomiQueryService) {

        return {
            completePropertyValue: completePropertyValue,
            getAutocompletedProperties: getAutocompletedProperties,
            isAutocomplete:isAutocomplete
        };

        function isAutocomplete (propertyName, propertiesList){
            var propertyIndex = _.findIndex(propertiesList, function(value) {
                return value.itemId == propertyName.replace("properties.", "");
            });
            return propertyIndex > -1;
        }

        /**
         * This function calculates the candidates values for a typed term
         * @param propertyName : The property for which the term is typed
         * @param term : the input value after last keypress
         * @returns {*} a table a values which results from a Unomi query on the properties values using a startsWith operator
         */
            function completePropertyValue(propertyName,term,propertyType, multivalued){
                return unomiQueryService.query(propertyType, propertyName, getPropertyValuesQuery(propertyName, term)).then(function (response) {
                    var filteredList = _.omit(response.data, ['_all', '_filtered']);
                    if(multivalued){
                        filteredList = _.omit(filteredList,function(value,key){
                            return !(key.toUpperCase().startsWith(term));
                        });
                    }
                    return _.map(filteredList, function(value,key){
                        return {value:key,count:value};
                    });
                });
            }

            function getAutocompletedProperties(){
                return unomiProfileService.getPropertyTypesByTag("autocompleted", true);
            }

            /**
             * This function generates the query to get all the property values that starts with the "value" value
             * @param propertyName
             * @param value
             * @returns {{condition: {type: string, parameterValues: {operator: string, subConditions: *[]}}}}
             */
            function getPropertyValuesQuery(propertyName,  value) {
                return {
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": propertyName,
                                        "comparisonOperator": "startsWith",
                                        "propertyValue": value
                                    }
                                }
                            ]
                        }
                    }
                };
            }

        String.prototype.startsWith = function (str)
        {
            return this.indexOf(str) == 0;
        }
    }

})();