(function() {
    'use strict';

    angular.module('wemApp.services').service('dataTableService', dataTableService);

    dataTableService.$inject = ['_', 'DTOptionsBuilder', 'DTColumnBuilder', 'i18nService'];

    function dataTableService(_, DTOptionsBuilder, DTColumnBuilder, i18nService) {
        var _promise;

        return {
            generateDataTableOptions: generateDataTableOptions,
            generateDataTableColumns: generateDataTableColumns,
            setPromise: setPromise
        };

        function generateDataTableOptions(options) {

            var dtOptions = DTOptionsBuilder.newOptions();

            dtOptions.withOption('lengthMenu', options.lengthMenu ? options.lengthMenu : [ 25, 50, 100 ])
                .withOption('pageLength', options.pageLength ? options.pageLength : 25);

            if (options.rowCallback) {
                dtOptions = dtOptions.withOption('fnRowCallback',
                    options.rowCallback);
            }
            if (options.createdRow) {
                dtOptions = dtOptions.withOption('createdRow',
                    options.createdRow);
            }
            dtOptions = dtOptions.withOption('processing', options.processing ? options.processing : true)
                .withOption('serverSide', options.serverSide ? options.serverSide : true)
                .withOption('sDom', 'lt')
                .withFnServerData(_serverData)
                .withOption('order', options.defaultOrder);
            if (options.paging != undefined) {
                dtOptions = dtOptions.withOption('paging', false);
            }
            if (options.ordering != undefined) {
                dtOptions = dtOptions.withOption('ordering', false);
            }
            return dtOptions;

        }

        function generateDataTableColumns(columns) {
            var dtColumns = [];
            _.forEach(columns, function(column){
                var columnItem = DTColumnBuilder.newColumn(column.name).withTitle(i18nService.message(column.bundleKey));
                if (column.notSortable) {
                    columnItem = columnItem.notSortable();
                }
                if (column.actionsHtml) {
                    columnItem = columnItem.renderWith(column.actionsHtml);
                }
                if (column.options && column.options.length) {
                    _.forEach(column.options, function(columnOption){
                        columnItem = columnItem.withOption(columnOption[0], columnOption[1]);
                    });
                }
                dtColumns.push(columnItem);
            });
            return dtColumns;
        }

        function setPromise(promise) {
            _promise = promise;
        }

        // Private functions under this line
        function _serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function(x) {
                return (x.name === 'draw');
            }).value;

            var start = _.find(aoData, function(x) {
                return (x.name === 'start');
            }).value;

            var length = _.find(aoData, function(x) {
                return (x.name === 'length');
            }).value;

            var orderData = _.find(aoData, function(x) {
                return (x.name === "order");
            }).value;

            var order = _.map(orderData, function(o) {
                return {prop: o.column, dir: o.dir};
            })[0];

            _promise.then(
                function(dataMap) {

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": dataMap.data.length,
                        "iTotalDisplayRecords": dataMap.data.length,
                        "aaData": dataMap.data
                    };
                    if (order) {
                        results.aaData = _.sortBy(dataMap.data, _.keys(dataMap.data[0])[order.prop]);
                        if (order.dir === 'desc') {
                            results.aaData = results.aaData.reverse();
                        }
                    }
                    if (dataMap.limit > 0) {
                        results.aaData.splice(dataMap.limit);
                    } else {
                        results.aaData = results.aaData.splice(start, length);
                    }

                    fnCallback(results);
                },
                function() {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                }
            );
        }
    }
})();