(function() {
    'use strict';

    angular.module('wemApp.services').service('dtPagerService', dtPagerService);

    dtPagerService.$inject = ['$timeout', 'i18nService'];

    function dtPagerService($timeout, i18nService) {
        return {
            refreshPager: refreshPager,
            initPagerProperties: initPagerProperties
        };

        function refreshPager($scope) {
            $timeout(function () {
                initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                // need to call $scope.$digest to force variable link of the pager when reordering.
                if (!$scope.$$phase) {
                    $scope.$digest();
                }
            });
        }

        function initPagerProperties(DataTable, $scope, i18nService) {
            var init = function () {
                if (!DataTable.data().length) {
                    $scope.pagerActive = false;
                    return;
                }
                $scope.pageInfo = DataTable.page.info();
                $scope.pageLength = DataTable.page.len();
                $scope.pagerPages = [];

                if ($scope.pageInfo.pages < 8) {
                    for (var i = 0; i < $scope.pageInfo.pages; i++) {
                        $scope.pagerPages.push({
                            id: i,
                            label: i + 1,
                            active: $scope.pageInfo.page == i,
                            enabled: true
                        });
                    }
                } else {
                    $scope.pagerPages.push({
                        id: 0,
                        label: 1,
                        active: $scope.pageInfo.page == 0,
                        enabled: true
                    });
                    if ($scope.pageInfo.page < 4) {
                        for (var i = 1; i < 5; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                    }
                    $scope.pagerPages.push({
                        id: 0,
                        label: "...",
                        active: false,
                        enabled: false
                    });
                    if ($scope.pageInfo.page >= 4 && $scope.pageInfo.page <= $scope.pageInfo.pages - 5) {
                        for (var i = $scope.pageInfo.page - 1; i <= $scope.pageInfo.page + 1; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                        $scope.pagerPages.push({
                            id: 0,
                            label: "...",
                            active: false,
                            enabled: false
                        });
                    }

                    if ($scope.pageInfo.page > $scope.pageInfo.pages - 5) {
                        for (var i = $scope.pageInfo.pages - 5; i < $scope.pageInfo.pages - 1; i++) {
                            $scope.pagerPages.push({
                                id: i,
                                label: i + 1,
                                active: $scope.pageInfo.page == i,
                                enabled: true
                            });
                        }
                    }

                    $scope.pagerPages.push({
                        id: $scope.pageInfo.pages - 1,
                        label: $scope.pageInfo.pages,
                        active: $scope.pageInfo.page == $scope.pageInfo.pages - 1,
                        enabled: true
                    });
                }
                $scope.pagerActive = $scope.pageInfo.pages > 1;

                if (i18nService && $scope.pagerActive) {
                    $scope.$watch("pageInfo.start", function (newValue, oldValue) {
                        $scope.pageInfoText = i18nService.format("serverSettings.manageWem.pageInfos", $scope.pageInfo.start + 1 + "|" + $scope.pageInfo.end + "|" +
                            $scope.pageInfo.recordsTotal);
                    });
                }
            };
            init();

            $scope.goToPage = function (page) {
                DataTable.page(page).draw(false);
                init();
            };
        }
    }
})();