(function() {
    'use strict';

    angular.module('wemApp.services').service('formMappingService', formMappingService);

    formMappingService.$inject = ['$http', '_'];

    function formMappingService($http, _) {
        var _formMappingContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/';
        var _openMappedProp = "(";
        var _setPropertyNamePrefix = "properties" + _openMappedProp;
        var _setPropertyValuePrefix = "eventProperty::properties" + _openMappedProp;
        var _endPrefix = ")";
        var _newItem;
        var _deletedItem;

        return {
            save: save,
            getAll: getAll,
            get: get,
            serverToLocalFormMapping: serverToLocalFormMapping,
            serverToLocalFormMappingActions: serverToLocalFormMappingActions,
            localToServerFormMapping: localToServerFormMapping,
            remove: remove,
            getFormFactory: getFormFactory
        };

        function save(formMapping) {
            _newItem = formMapping;
            return $http.post(_formMappingContext, formMapping);
        }

        function getAll(offset, limit, sortby) {
            var query = {
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.tags",
                                    "comparisonOperator": "equals",
                                    "propertyValue": "formMappingRule"
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ]
                    }
                },
                "offset": offset,
                "limit": limit,
                "sortby": sortby
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            query.condition,
                            {
                                parameterValues: {
                                    comparisonOperator: "notEquals",
                                    propertyName: "itemId",
                                    propertyValue: _deletedItem
                                },
                                type: "sessionPropertyCondition"
                            }
                        ]
                    }
                };
                _deletedItem = undefined;
            }

            return $http.post(_formMappingContext + "query", query);
        }

        function get(id) {
            return $http.get(_formMappingContext + id);
        }

        function serverToLocalFormMapping(formMapping) {
            var _formMapping = {
                name: formMapping.metadata.name,
                description: formMapping.metadata.description,
                id: formMapping.metadata.id,
                scope: formMapping.metadata.scope,
                additionalPaths: [],
                tempActions: formMapping.actions,
                actions: {},
                lists: []
            };
            _formMapping.allSite = false;

            _formMapping.formId = _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type == 'formEventCondition';
            }).parameterValues.formId;

            _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type != 'formEventCondition';
            }).parameterValues.subConditions.forEach(function (entry) {
                if (entry.parameterValues.scope) {
                    _formMapping.allSite = true;
                } else {
                    if (!_formMapping.primaryPath) {
                        _formMapping.primaryPath = entry.parameterValues.path;
                    } else if (entry.parameterValues.path != _formMapping.primaryPath) {
                        _formMapping.additionalPaths.push(entry.parameterValues.path);
                    }
                }
            });

            return _formMapping;
        }

        function serverToLocalFormMappingActions(formMapping, propertyNameChoiceListValuesIndexById) {
            formMapping.tempActions.forEach(function (action) {
                if (action.parameterValues.setPropertyName == "systemProperties.lists" && action.parameterValues.setPropertyValue) {
                    formMapping.lists.push(action.parameterValues.setPropertyValue);
                } else {
                    var propertyName = _extractNameFromPrefixedName(action.parameterValues.setPropertyName, _setPropertyNamePrefix);
                    var fieldName = action.parameterValues.setPropertyValue ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValue, _setPropertyValuePrefix) : undefined;
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueInteger ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueInteger, _setPropertyValuePrefix) : undefined;
                    }
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueMultiple ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueMultiple, _setPropertyValuePrefix) : undefined;
                    }
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueBoolean ? _extractNameFromPrefixedName(action.parameterValues.setPropertyValueBoolean, _setPropertyValuePrefix) : undefined;
                    }
                    var strategy = action.parameterValues.setPropertyStrategy ? action.parameterValues.setPropertyStrategy : undefined;
                    if (!propertyNameChoiceListValuesIndexById[propertyName]) {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById["user-specified-property"],
                            newProp: propertyName,
                            strategy: strategy
                        };
                    } else {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById[propertyName],
                            strategy: strategy
                        };
                    }
                }
            });
            delete formMapping.tempActions;
        }

        function localToServerFormMapping(formMapping) {
            var _formMapping = {
                "metadata": {
                    "id": formMapping.id,
                    "name": formMapping.name,
                    "description": formMapping.description,
                    "scope": formMapping.scope,
                    "tags": ['formMappingRule']
                },

                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": []
                    }
                },

                "actions": []
            };

            //conditions
            var pathConditions = [];
            pathConditions.push({
                "parameterValues": {
                    "path": formMapping.primaryPath
                },
                "type": "sourceEventPropertyCondition"
            });
            if (formMapping.allSite) {
                pathConditions.push({
                    "parameterValues": {
                        "scope": formMapping.scope
                    },
                    "type": "sourceEventPropertyCondition"
                });
            } else {
                formMapping.additionalPaths.forEach(function (path) {
                    pathConditions.push({
                        "parameterValues": {
                            "path": path
                        },
                        "type": "sourceEventPropertyCondition"
                    })
                });
            }

            _formMapping.condition.parameterValues.subConditions.push(
                {
                    "parameterValues": {
                        "formId": formMapping.formId
                    },
                    "type": "formEventCondition"
                },
                {
                    "parameterValues": {
                        "operator": "or",
                        "subConditions": pathConditions
                    },
                    "type": "booleanCondition"
                }
            );

            //actions
            angular.forEach(formMapping.actions, function(value) {
                value.order = ManagersUtils.formMappingActionStrategies.length - ManagersUtils.formMappingActionStrategies.indexOf(value.strategy);
            });

            angular.forEach(_.sortBy(formMapping.actions, 'order'), function (value) {
                if (value.propertyType) {
                    var strategy = value.strategy;
                    var target = ((value.propertyType.metadata.id == 'user-specified-property' && value.newProp) ? value.newProp : value.propertyType.metadata.id);
                    var action = {
                        "parameterValues": {
                            "setPropertyName": _prefixPropertyName(target, _setPropertyNamePrefix),
                            "setPropertyStrategy": strategy
                        },
                        "type": "setPropertyAction"
                    };

                    if (value.propertyType.type == "integer") {
                        action.parameterValues.setPropertyValueInteger = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                    }

                    if (value.propertyType.type == "boolean") {
                        action.parameterValues.setPropertyValueBoolean = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                    }

                    else {
                        if (value.propertyType.multivalued) {
                            action.parameterValues.setPropertyValueMultiple = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                        }else{
                            action.parameterValues.setPropertyValue = _prefixPropertyName(value.fieldName, _setPropertyValuePrefix);
                        }
                    }
                    _formMapping.actions.push(action);
                }
            });

            //lists
            angular.forEach(formMapping.lists, function (value, key) {
                _formMapping.actions.push({
                    "parameterValues": {
                        "setPropertyName": "systemProperties.lists",
                        "setPropertyValue": value,
                        "setPropertyStrategy": "addValue"
                    },
                    "type": "setPropertyAction"
                });
            });

            return _formMapping;
        }

        function remove(id) {
            _deletedItem = id;
            return $http.delete(_formMappingContext + id);
        }

        function getFormFactory(id) {
            return $http.get(ManagersContext.cmsContext + "/modules/formfactory/live/skeleton/" + id + "/" + ManagersContext.mainResourceLocale);
        }

        // Private function under this line
        function _extractNameFromPrefixedName(prefixedName, prefix) {
            var extracted;
            if (prefixedName.indexOf(_endPrefix) > 0) {
                extracted = prefixedName.substring(prefix.length, prefixedName.length - _endPrefix.length);
            } else {
                // we have an old style property, so remove the old style prefix which wasn't using indexed properties
                // prefix contains the opening of the mapped prop ((') so remove it and add + 1 for the . that was there before
                extracted = prefixedName.substr(prefix.length - _openMappedProp.length + 1);
            }
            return extracted;
        }

        function _prefixPropertyName(name, prefix) {
            return prefix + name + _endPrefix;
        }
    }
})();