(function() {
    'use strict';

    angular.module('wemApp.services').service('internalSearchService', internalSearchService);

    internalSearchService.$inject = ['$q', '_', 'i18nService', 'unomiQueryService'];

    function internalSearchService($q, _, i18nService, unomiQueryService) {
        var _selectedUrl;
        var _selectedSearch;
        // todo: extract scope condition to reuse it
        // todo: use ManagerUtils.booleanAnd
        var _searchEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "search"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        // todo: use ManagerUtils.booleanAnd
        var _viewEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "view"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        // todo: replace by ManagerUtils.AggregateDisplay at some point
        var Search = function (searchString, occurrences) {
            var SearchItem = function (value, occurrences, missing) {
                this.item = value;
                this.occurrences = occurrences;
                this.missing = missing;

                this.importance = function (totalOccurrences) {
                    return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
                }
            };

            this.searchString = searchString;
            this.occurrences = occurrences;

            this.origins = [];
            this.setOrigins = function (origins) {
                this.origins = this._setDetails(origins);
            };

            this.userIds = [];
            this.setUserIds = function (userIds) {
                this.userIds = this._setDetails(userIds);
            };

            this.importance = function (totalOccurrences) {
                return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
            };

            this._setDetails = function (details) {
                if (details) {
                    var detailsArray = [];
                    var totalOccurrences = 0;
                    for (var detail in details) {
                        if (ManagersUtils.acceptProperty(details, detail)) {
                            occurrences = details[detail];

                            var missing = detail == "_missing";
                            if (missing) {
                                // replace _missing by something more user friendly
                                detail = i18nService.message("serverSettings.internalSearches.missing");
                            }

                            totalOccurrences += occurrences;
                            detailsArray.push(new SearchItem(detail, occurrences, missing));
                        }
                    }
                    this.occurrences = totalOccurrences;
                    return detailsArray;
                }
            };

            this.languages = [];
            this.setLanguages = function (lang) {
                this.languages = this._setDetails(lang);
            }
        };

        return {
            setSelectedSearch: setSelectedSearch,
            setSelectedUrl: setSelectedUrl,
            getSearchOccurrencesByTerm: getSearchOccurrencesByTerm,
            getSearchOccurrencesByTermAndDate: getSearchOccurrencesByTermAndDate,
            getSearchDetails: getSearchDetails,
            getMostVisitedList: getMostVisitedList,
            getMostVisitedDetails: getMostVisitedDetails,
            getMostVisitedLanguages: getMostVisitedLanguages,
            getSearchLanguages: getSearchLanguages
        };

        function setSelectedSearch(searchTerm) {
            _selectedSearch = searchTerm;
        }

        function setSelectedUrl(urlPath) {
            _selectedUrl = urlPath;
        }

        function getSearchOccurrencesByTerm() {
            return unomiQueryService.query('event', 'properties.keyword', _searchEventsCondition);
        }

        function getSearchOccurrencesByTermAndDate(from, to, language) {
            var deferred = $q.defer();
            var searchEventsWithDateRangeCondition = {
                "condition": ManagersUtils.getDateRangeConditionBody(from, to)
            };

            if (language && language != null && language !== '_all') {
                searchEventsWithDateRangeCondition = {
                    "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                        {
                            "type": "eventPropertyCondition",
                            "parameterValues": {
                                "propertyName": "properties.language",
                                "comparisonOperator": "equals",
                                "propertyValue": language
                            }
                        }
                    )
                };
            }

            searchEventsWithDateRangeCondition.condition.parameterValues.subConditions.push({
                type: "eventTypeCondition",
                parameterValues: {
                    eventTypeId: "search"
                }
            });

            unomiQueryService.query('event', 'properties.keyword', searchEventsWithDateRangeCondition)
                .then(function (response) {
                    var data = response.data;
                    var searches = [];
                    for (var search in data) {
                        if (ManagersUtils.acceptProperty(data, search)) {
                            searches.push({
                                searchString: search,
                                nbOcc: data[search],
                                importance: ManagersUtils.getPercentage(data[search] / data._filtered)
                            });
                        }
                    }
                    var result = {data: searches};
                    deferred.resolve(result);
                },
                function () {
                    deferred.reject();
                }
            );
            return deferred.promise;
        }

        function getSearchDetails(searchTerm) {
            // if we didn't provide a search term, use the one potentially already selected
            searchTerm = searchTerm || _selectedSearch;

            var search = new Search(searchTerm, -1);

            var condition = {};
            angular.copy(_searchEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "properties.keyword",
                    "comparisonOperator": "equals",
                    "propertyValue": searchTerm
                }
            });

            unomiQueryService.query('event', 'properties.origin', condition).then(function (response) {
                search.setOrigins(response.data);
            });

            unomiQueryService.query('event', 'profileId', condition).then(function (response) {
                search.setUserIds(response.data);
            });

            unomiQueryService.query('event', 'properties.language', condition).then(function (response) {
                search.setLanguages(response.data);
            });

            return search;
        }

        function getMostVisitedList(limit, from, to, language) {
            var deferred = $q.defer();
            var viewEventsByDateCondition = {
                "condition":
                    ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    {
                        "type": "eventTypeCondition",
                        "parameterValues": {
                            "eventTypeId": "view"
                        }
                    })
            };

            if (language && language != null && language !== '_all') {
                viewEventsByDateCondition = {
                    "condition": ManagersUtils.booleanAndArray([
                        ManagersUtils.getDateRangeConditionBody(from, to),
                        {
                            "type": "eventTypeCondition",
                            "parameterValues": {
                                "eventTypeId": "view"
                            }
                        },
                        {
                            "type": "eventPropertyCondition",
                            "parameterValues": {
                                "propertyName": "target.properties.pageInfo.language",
                                "comparisonOperator": "equals",
                                "propertyValue": language
                            }
                        }
                    ])
                };
            }

            unomiQueryService.query("event", "target.properties.pageInfo.destinationURL", viewEventsByDateCondition)
                .then(function (response) {
                    var dataMap = _.map(response.data, function (occurrences, url) {
                        return {url: url, occurrences: occurrences}
                    });
                    dataMap.splice(0, 2);

                    var result = {data: dataMap, limit: 0};

                    if(limit > 0){
                        result.limit = limit;
                    }

                    //fnCallback(results);
                    deferred.resolve(result);

                },
                function () {
                    deferred.reject();
                });
            return deferred.promise;
        }

        function getMostVisitedDetails() {
            var urlPath = _selectedUrl;
            var search = new Search(urlPath, -1);

            var condition = {};
            angular.copy(_viewEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.destinationURL",
                    "comparisonOperator": "equals",
                    "propertyValue": urlPath
                }
            });


            unomiQueryService.query('event', 'profileId', condition).then(function (response) {
                search.setUserIds(response.data);
            });

            return search;
        }

        function getMostVisitedLanguages() {
            var deferred = $q.defer();
            unomiQueryService.query('event', 'target.properties.pageInfo.language', _viewEventsCondition)
                .then(function (response) {
                    var dataMap = _.reduce(response.data, function (memo, value, key) {
                        if (key !== '_filtered') {
                            var name = key;
                            if (name === '_all') {
                                name = "all";
                            }
                            memo.push({id: key, name: name});
                        }
                        return memo;
                    }, []);
                    deferred.resolve(dataMap);
                },function () {
                    deferred.reject();
                });
            return deferred.promise;
        }

        function getSearchLanguages() {
            var deferred = $q.defer();
            unomiQueryService.query('event', 'properties.language', _searchEventsCondition)
                .then(function (response) {
                    var dataMap = _.reduce(response.data, function (memo, value, key) {
                        if (key !== '_filtered') {
                            var name = key;
                            if (name === '_all') {
                                name = "all";
                            }
                            memo.push({id: key, name: name});
                        }
                        return memo;
                    }, []);
                    deferred.resolve(dataMap);
                },function () {
                    deferred.reject();
                });
            return deferred.promise;
        }
    }
})();