(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiExportConfigService', unomiExportConfigService);

    unomiExportConfigService.$inject = ['$http', '$q'];

    function unomiExportConfigService($http, $q) {
        var _exportConfigContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'exportConfiguration/';
        var _exportConfigOneshotContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'exportConfiguration/oneshot/';
        var _selectedTab;

        return {
            getAll: getAll,
            setExportConfig: setExportConfig,
            setOneshotExportConfig: setOneshotExportConfig,
            getExportConfig: getExportConfig,
            deleteExportConfig: deleteExportConfig
        };

        function getAll(limit, offset, sortby) {
            var actualLimit = limit || 100;
            var actualOffset = offset || 0;
            var actualSortBy = sortby || null;
            return $http.get(_exportConfigContext);
        }

        function setExportConfig(exportConfig) {
            return $http.post(_exportConfigContext, exportConfig);
        }

        function setOneshotExportConfig(exportConfig) {
            var deferred = $q.defer();
            setExportConfig(exportConfig).then(function(response) {
                $http({
                    method: 'POST',
                    url: _exportConfigOneshotContext,
                    data: response.data,
                    responseType: 'arraybuffer'
                }).then(function (response) {
                    var headers = response.headers();
                    var fileNameAttr = headers["content-disposition"].split(';')[1];
                    var filename = fileNameAttr.substring(fileNameAttr.indexOf('=') + 1, fileNameAttr.length);
                    var contentType = headers['content-type'];
                    var linkElement = document.createElement('a');
                    try {
                        var blob = new Blob([response.data], {type: contentType});
                        var url = window.URL.createObjectURL(blob);

                        linkElement.setAttribute('href', url);
                        linkElement.setAttribute("download", filename);

                        var clickEvent = new MouseEvent("click", {
                            "view": window,
                            "bubbles": true,
                            "cancelable": false
                        });
                        linkElement.dispatchEvent(clickEvent);
                        deferred.resolve({msg: "success"});
                    } catch (ex) {
                        console.log(ex);
                        deferred.resolve({msg: ex});
                    }
                }, function (data) {
                    console.log(data);
                    deferred.resolve({msg: data});
                });
            });
            return deferred.promise;
        }

        function getExportConfig(id) {
            return $http.get(_exportConfigContext + id);
        }

        function deleteExportConfig(id) {
            return $http.delete(_exportConfigContext + id);
        }
    }
})();