(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiGoalService', unomiGoalService);

    unomiGoalService.$inject = ['$http'];

    function unomiGoalService($http) {
        var ADD_TO_LIST_RULE = "addToListRULE_";
        var _goalsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'goals/';

        return {
            localToServerGoalRule: localToServerGoalRule,
            getAll: getAll,
            getAllForSite: getAllForSite,
            getAllForCampaign: getAllForCampaign,
            getGoalReport: getGoalReport,
            getGoalReportAggregated: getGoalReportAggregated,
            getAddToListRulePrefix: getAddToListRulePrefix,
            setGoal: setGoal,
            getGoal: getGoal,
            deleteGoal: deleteGoal
        };

        function localToServerGoalRule(goal, lists) {
            if (goal) {
                var _goalRule = {
                    "metadata": {
                        "id": ADD_TO_LIST_RULE + goal.metadata.id,
                        "name": goal.metadata.name,
                        "description": goal.metadata.description,
                        "scope": goal.metadata.scope
                    },
                    "condition": {
                        type: "booleanCondition",
                        parameterValues: {
                            subConditions: [
                                {
                                    type: "eventPropertyCondition",
                                    parameterValues: {
                                        propertyName: "target.itemId",
                                        comparisonOperator: "equals",
                                        propertyValue: goal.metadata.id+'TargetEvent'
                                    }
                                },
                                {
                                    type: "eventTypeCondition",
                                    parameterValues: {
                                        eventTypeId: "ruleFired"
                                    }
                                }
                            ],
                            operator: "and"
                        }
                    },

                    "actions": []
                };

                //lists
                angular.forEach(lists, function (value, key) {
                    _goalRule.actions.push({
                        "parameterValues": {
                            "setPropertyName": "systemProperties.lists",
                            "setPropertyValue": value,
                            "setPropertyStrategy": "addValue"
                        },
                        "type": "setPropertyAction"
                    });
                });

                return _goalRule;
            }

            return null;
        }

        function getAll() {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getAllForSite() {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "missing"
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getAllForCampaign(campaign) {
            return $http.post(_goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaign
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        }

        function getGoalReport(id) {
            var uri = _goalsContext + id + "/report";
            return $http.get(uri);
        }

        function getGoalReportAggregated(id, query) {
            var uri = _goalsContext + id + "/report";
            return $http.post(uri, query);
        }

        function getAddToListRulePrefix(){
            return ADD_TO_LIST_RULE;
        }

        function setGoal(goal) {
            return $http.post(_goalsContext, goal);
        }

        function getGoal(id) {
            return $http.get(_goalsContext + id);
        }

        function deleteGoal(id) {
            return $http.delete(_goalsContext + id);
        }
    }
})();