(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiProfileService', unomiProfileService);

    unomiProfileService.$inject = ['$http', 'unomiQueryService'];

    function unomiProfileService($http, unomiQueryService) {
        var _profilesContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'profiles/';
        var _personasContext = _profilesContext + 'personas/';
        var _personasWithSessionsContext = _profilesContext + 'personasWithSessions/';
        var _selectedTab;

        return {
            search: search,
            searchSessions: searchSessions,
            exportUsingForm: exportUsingForm,
            getProfile: getProfile,
            saveProfile: saveProfile,
            newProfile: newProfile,
            deleteProfile: deleteProfile,
            batchProfilesUpdate: batchProfilesUpdate,
            getSessions: getSessions,
            savePersonaWithSessions: savePersonaWithSessions,
            getPropertyTypes: getPropertyTypes,
            createPropertyType: createPropertyType,
            deletePropertyType: deletePropertyType,
            getPropertyTypesByTarget: getPropertyTypesByTarget,
            getPropertyTypesByTag: getPropertyTypesByTag,
            getCount: getCount,
            constructCondition: constructCondition,
            constructQuery: constructQuery,
            constructQueryOnCurrentSelectedTab: constructQueryOnCurrentSelectedTab,
            setSelectedTab: setSelectedTab,
            setSelectedTabType: setSelectedTabType,
            setSelectedTabDeletedProfileId: setSelectedTabDeletedProfileId,
            setSelectedTabConditionTypes: setSelectedTabConditionTypes,
            setSelectedTabEventConditionTypes: setSelectedTabEventConditionTypes,
            setSelectedTabSearchCondition: setSelectedTabSearchCondition,
            setSelectedTabSearchConditionType: setSelectedTabSearchConditionType,
            setSelectedTabSearchConditionParameterValues: setSelectedTabSearchConditionParameterValues,
            getSelectedTab: getSelectedTab,
            deleteSelectedTabBaseCondition: deleteSelectedTabBaseCondition,
            setSelectedTabBaseCondition: setSelectedTabBaseCondition
        };

        function search(query, profileType) {
            return $http.post(_getContextFor(profileType) + "search", query);
        }

        function searchSessions(query, profileType) {
            return $http.post(_getContextFor(profileType) + 'search/sessions', query);
        }

        function exportUsingForm(query, profileType, method) {
            ManagersUtils.downloadForm(_getContextFor(profileType) + "export", query, method);
        }

        function getProfile(id, profileType) {
            return $http.get(_getContextFor(profileType) + id);
        }

        function saveProfile(profile, profileType) {
            // remove empty props
            var cleanedProp = {};
            angular.forEach(profile.properties, function (value, key) {
                if (value) {
                    cleanedProp[key] = value;
                } else {
                    if(typeof value != 'boolean') {
                        cleanedProp[key] = null;
                    } else {
                        cleanedProp[key] = value;
                    }
                }
            });
            profile.properties = cleanedProp;
            return $http.post(_getContextFor(profileType), profile);
        }

        function newProfile(id, profileType) {
            return $http.put(_getContextFor(profileType) + id);
        }

        function deleteProfile(id, profileType) {
            return $http.delete(_getContextFor(profileType) + id);
        }

        function batchProfilesUpdate(batchUpdate) {
            return $http.post(_profilesContext + 'batchProfilesUpdate', batchUpdate)
        }

        function getSessions(id, profileType, offset, numberOfSessions, sort) {
            return $http.get(_getContextFor(profileType) + id + '/sessions?offset=' + offset + '&size=' + numberOfSessions + '&sort=' + sort);
        }

        function savePersonaWithSessions(personaObject) {
            return $http.post(_personasWithSessionsContext, personaObject);
        }

        function getPropertyTypes() {
            return $http.get(_profilesContext + 'properties');
        }

        function createPropertyType(propertyType) {
            return $http.post(_profilesContext + 'properties', propertyType);
        }

        function deletePropertyType(propertyId) {
            return $http.delete(_profilesContext + 'properties/' + propertyId);
        }

        function getPropertyTypesByTarget(target) {
            return $http.get(_profilesContext + 'properties/' + target)
        }

        function getPropertyTypesByTag(tag, recursive) {
            if (recursive) {
                return $http.get(_profilesContext + 'properties/tags/' + tag + '?recursive=true')
            } else {
                return $http.get(_profilesContext + 'properties/tags/' + tag)
            }
        }

        function getCount() {
            return unomiQueryService.getMatchingItemsCountForCondition('profile', constructCondition(null,null,null, true,'profile'), false);
        }

        function constructCondition(baseCondition, searchCondition, deletedProfileId, excludeMergedAndAnonymous, profileType) {
            var subConditions = [];
            if (baseCondition) {
                subConditions.push(baseCondition);
            }
            if (excludeMergedAndAnonymous && profileType != "personas") {
                // we do not want the merged profiles:
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "missing",
                        propertyName: "mergedWith"
                    },
                    type: "profilePropertyCondition"
                });
                subConditions.push({
                    parameterValues: {
                        propertyName: "systemProperties.isAnonymousProfile",
                        comparisonOperator: "missing"
                    },
                    type: "profilePropertyCondition"
                });
            }
            if (searchCondition && searchCondition.type) {
                if (searchCondition.type == "booleanCondition"
                    && searchCondition.parameterValues.operator
                    && searchCondition.parameterValues.operator == "and"
                    && searchCondition.parameterValues.subConditions) {
                    subConditions = subConditions.concat(searchCondition.parameterValues.subConditions);
                } else {
                    subConditions.push(searchCondition);
                }
            }
            if (deletedProfileId) {
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: deletedProfileId
                    },
                    type: "profilePropertyCondition"
                });
            }

            if (subConditions.length == 0) {
                return null;
            } else if (subConditions.length == 1) {
                return subConditions[0];
            } else {
                return {
                    parameterValues: {
                        operator: "and",
                        subConditions: subConditions
                    },
                    type: "booleanCondition"
                };
            }
        }

        function constructQuery(baseCondition, searchCondition, deletedProfileId, text, offset, limit, orderBy, profileType) {
            return {
                "condition": constructCondition(baseCondition, searchCondition, deletedProfileId, true, profileType),
                "limit": limit ? limit : 50,
                "offset": offset ? offset : 0,
                "sortby": orderBy != undefined ? orderBy : null,
                "text": text != undefined ? text : null
            };
        }

        function constructQueryOnCurrentSelectedTab(text, offset, limit, orderBy) {
            if (_selectedTab) {
                return {
                    "condition": constructCondition(_selectedTab.baseCondition, _selectedTab.searchCondition, _selectedTab.deletedProfileId, true, _selectedTab.type),
                    "limit": limit ? limit : -1,
                    "offset": offset ? offset : 0,
                    "sortby": orderBy != undefined ? orderBy : null,
                    "text": text != undefined ? text : null
                };
            }
        }

        // TODO: selectedTab is used outside of the service so I put some getter and setter here but I believe we should rewrite most of this
        function getSelectedTab() {
            return _selectedTab;
        }

        function setSelectedTab(selectedTab) {
            _selectedTab = selectedTab;
        }

        function setSelectedTabSearchCondition(searchCondition) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.searchCondition = searchCondition;
        }

        function setSelectedTabSearchConditionType(searchConditionType) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            if (!_selectedTab.searchCondition) {
                _selectedTab.searchCondition = {};
            }
            _selectedTab.searchCondition.type = searchConditionType;
        }

        function setSelectedTabSearchConditionParameterValues(searchConditionParameterValues) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            if (!_selectedTab.searchCondition) {
                _selectedTab.searchCondition = {};
            }
            _selectedTab.searchCondition.parameterValues = searchConditionParameterValues;
        }

        function setSelectedTabType(selectedTabType) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.type = selectedTabType;
        }

        function setSelectedTabDeletedProfileId(deletedProfileId) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.deletedProfileId = deletedProfileId;
        }

        function setSelectedTabConditionTypes(conditionTypes) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.conditionTypes = conditionTypes;
        }

        function setSelectedTabEventConditionTypes(eventConditionTypes) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.eventConditionTypes = eventConditionTypes;
        }

        function deleteSelectedTabBaseCondition() {
            delete _selectedTab.baseCondition;
        }

        function setSelectedTabBaseCondition(baseCondition) {
            if (!_selectedTab) {
                _selectedTab = {};
            }
            _selectedTab.baseCondition = baseCondition;
        }
        
        // Private functions under this line
        function _getContextFor(profileType) {
            return profileType == 'personas' ? _personasContext : _profilesContext;
        }
    }
})();