(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiScoringService', unomiScoringService);

    unomiScoringService.$inject = ['$http', 'jcrService'];

    function unomiScoringService($http, jcrService) {
        var _scoringContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'scoring/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            getImpacted: getImpacted,
            query: query,
            setScoring: setScoring,
            getScoring: getScoring,
            deleteScoring: deleteScoring,
            buildUpdateBatchRemoveScoring: buildUpdateBatchRemoveScoring
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_scoringContext);
        }

        function getImpacted(segmentId) {
            return $http.get(_scoringContext + segmentId + "/impacted");
        }

        function query(query) {
            return $http.post(_scoringContext + "query", query);
        }

        function setScoring(scoring, shared) {
            scoring.elements.forEach(function (element) {
                _addDownloadMixin(element.condition);
            });
            if (shared) {
                scoring.metadata.scope = 'systemscope';
            }
            _newItem = scoring;
            return $http.post(_scoringContext, scoring);
        }

        function getScoring(id) {
            return $http.get(_scoringContext + id);
        }

        function deleteScoring(id, validate) {
            var uri = _scoringContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }

        function buildUpdateBatchRemoveScoring(scoreId) {
            var baseCondition = {
                parameterValues: {
                    comparisonOperator: "exists",
                    propertyName: "scores."+scoreId
                },
                type: "profilePropertyCondition"
            };
            return {
                condition: baseCondition,
                propertyName: "systemProperties.notificationAck.scoring."+scoreId,
                propertyValue: null,
                strategy: "remove"
            };
        }

        // Private function
        function _addDownloadMixin(condition) {
            var pastEventConditions = _getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = _getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).then(function (response) {
                        // mixin added
                    });
                });
            });
        }

        function _getSubConditionsOfType(rootCondition, list, conditionType) {
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    _getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        }
    }
})();