(function() {
    'use strict';

    angular.module('wemApp.services').service('unomiSegmentService', unomiSegmentService);

    unomiSegmentService.$inject = ['$http', 'jcrService'];

    function unomiSegmentService($http, jcrService) {
        var _segmentsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'segments/';
        var _newItem;
        var _deletedItem;

        return {
            constructQuery: constructQuery,
            getAll: getAll,
            getImpacted: getImpacted,
            query: query,
            addDownloadMixin: addDownloadMixin,
            setSegment: setSegment,
            getSegment: getSegment,
            getSegmentCount: getSegmentCount,
            deleteSegment: deleteSegment
        };

        function constructQuery(length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (_newItem) {
                query.forceRefresh = true;
                _newItem = undefined;
            }

            if (_deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: _deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                _deletedItem = undefined;
            }

            return query;
        }

        function getAll() {
            return $http.get(_segmentsContext);
        }

        function getImpacted(segmentId) {
            return $http.get(_segmentsContext + segmentId + "/impacted");
        }

        function query(query) {
            return $http.post(_segmentsContext + "query", query);
        }

        function addDownloadMixin(condition) {
            var pastEventConditions = _getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = _getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).then(function (response) {
                        // mixin added
                    });
                })
            });
        }

        function setSegment(segment, shared) {
            addDownloadMixin(segment.condition);
            if (shared) {
                segment.metadata.scope = 'systemscope';
            }
            _newItem = segment;
            return $http.post(_segmentsContext, segment);
        }

        function getSegment(id) {
            return $http.get(_segmentsContext + id);
        }

        function getSegmentCount(id) {
            return $http.get(_segmentsContext + id + "/count");
        }

        function deleteSegment(id, validate) {
            var uri = _segmentsContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }

        // Private function under this line
        function _getSubConditionsOfType(rootCondition, list, conditionType) {
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    _getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        }
    }
})();