(function() {
    'use strict';

    angular.module('wemCampaignsApp').controller('CampaignGoalsListController', CampaignGoalsListController);

    CampaignGoalsListController.$inject = [
        '$scope', '$route', '$location', '$uibModal', 'unomiGoalService', 'unomiQueryService', 'unomiDefinitionService',
        'i18nService', 'unomiCampaignService', 'campaignDetailed', '_', '$controller', 'DTOptionsBuilder',
        'DTColumnBuilder', '$compile', 'modalService', 'dtPagerService'];

    function CampaignGoalsListController($scope, $route, $location, $uibModal, unomiGoalService, unomiQueryService, unomiDefinitionService,
                                         i18nService, unomiCampaignService, campaignDetailed, _, $controller, DTOptionsBuilder,
                                         DTColumnBuilder, $compile, modalService, dtPagerService) {

        angular.extend(this, $controller('GoalsListController', {
            $scope: $scope,
            $route: $route,
            $location: $location,
            $uibModal: $uibModal,
            unomiGoalService: unomiGoalService,
            unomiQueryService: unomiQueryService,
            unomiDefinitionService: unomiDefinitionService,
            i18nService: i18nService,
            currentCampaign: campaignDetailed.campaign,
            _: _
        }));

        // campaign type and localized names of types
        $scope.currentCampaignType = {value: unomiCampaignService.getCampaignType($scope.currentCampaign), param: unomiCampaignService.getTypeSpecificCampaignParameter($scope.currentCampaign)};
        $scope.localizedCampaignType = unomiCampaignService.getLocalizedCampaignType($scope.currentCampaignType.value);
        $scope.localizedCampaignParam = unomiCampaignService.getLocalizedCampaignParam($scope.currentCampaignType.value);

        if (!_.isUndefined(campaignDetailed.campaign)) {
            // timeline data
            var drawCampaignEvents = function () {
                if ($scope.campaignChart) {
                    var eventDates = {};
                    angular.forEach($scope.campaignEvents, function (event) {
                        var eventDateLabel = moment(event.eventDate).format($scope.timelineData.interval.scaleFormat);
                        if (eventDates[eventDateLabel] == undefined) {
                            eventDates[eventDateLabel] = eventDateLabel;
                        }
                    });
                    //now let's paint point with events in red.
                    var ctx = $scope.campaignChart.chart.ctx;
                    ctx.strokeStyle = "#FF0000";

                    if ($scope.campaignChart.datasets) {
                        var min;
                        var max;
                        angular.forEach($scope.campaignChart.datasets, function (dataset) {
                            angular.forEach(dataset.points, function (point) {
                                if (!min || point.y < min) min = point.y;
                                if (!max || point.y > max) max = point.y;
                            });
                        });
                        angular.forEach($scope.campaignChart.datasets[0].points, function (point) {
                            if (eventDates[point.label] != undefined) {
                                ctx.strokeRect(point.x - 10, min - 10, 20, max - min + 20);
                            }
                        });
                    }
                }
            };

            $scope.timelineData.chartOptions = {
                tooltips: {
                    mode: 'label',
                    callbacks: {
                        label: function (tooltipItem, data) {
                            var eventNames = [];
                            angular.forEach($scope.campaignEvents, function (event) {
                                if (tooltipItem.xLabel == (moment(event.eventDate).format($scope.timelineData.interval.scaleFormat))) {
                                    eventNames.push(event.metadata.name);
                                }
                            });
                            var label = tooltipItem.xLabel;
                            if (eventNames.length > 0) {
                                label = "Campaign Events: " + eventNames.join(',');
                            }
                            return label;
                        }
                    }
                },
                animation: {
                    onComplete: function () {
                        $scope.campaignChart = this;
                        drawCampaignEvents();
                    },
                    onProgress: function () {
                        $scope.campaignChart = this;
                        drawCampaignEvents();
                    }
                }
            };
            $scope.timelineData.series = [i18nService.message('serverSettings.manageCampaigns.engaged'), i18nService.message('serverSettings.manageCampaigns.converted')];
            $scope.timelineData.loadData = function (success) {
                var aggregateSessionQuery = {
                    "aggregate": {
                        "type": "date",
                        "parameters": {
                            "interval": this.interval.key,
                            "format": "yyyy-MM-dd"
                        }
                    },
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "scope",
                                        "comparisonOperator": "equals",
                                        "propertyValue": ManagersContext.dmfScope
                                    }
                                }
                            ]
                        }
                    }
                };
                aggregateSessionQuery.condition.parameterValues.subConditions.push(
                    {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "systemProperties.campaigns." + this.currentCampaign.metadata.id + "Engaged",
                            "comparisonOperator": "between",
                            "propertyValuesDate": [
                                this.currentCampaign.startDate,
                                this.currentCampaign.endDate
                            ]
                        }
                    });
                // get all sessions
                unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                    var aggregateSessionsData = response.data;
                    var datasets = [];
                    datasets[0] = aggregateSessionsData;

                    // get new sessions:
                    aggregateSessionQuery.condition.parameterValues.subConditions.push({
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "systemProperties.goals." + $scope.timelineData.currentCampaign.primaryGoal + "TargetReached",
                            "comparisonOperator": "between",
                            "propertyValuesDate": [
                                $scope.timelineData.currentCampaign.startDate,
                                $scope.timelineData.currentCampaign.endDate
                            ]
                        }
                    });
                    unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                        datasets[1] = response.data;
                        unomiCampaignService.getCampaignEvents($scope.timelineData.currentCampaign.metadata.id, -1, 0, null, {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "eventDate",
                                "comparisonOperator": "between",
                                "propertyValuesDate": [
                                    $scope.timelineData.currentCampaign.startDate,
                                    $scope.timelineData.currentCampaign.endDate
                                ]
                            }
                        }).then(function (response) {
                            $scope.campaignEvents = response.data.list;
                            success(datasets);
                        });
                    });
                });
            };

            $scope.currentCampaign.KPI = {
                engaged: campaignDetailed.engagedProfiles,
                sessions: campaignDetailed.campaignSessionViews,
                reached: campaignDetailed.campaignSessionSuccess,
                cost_per_visitor: $scope.currentCampaign.cost / campaignDetailed.campaignSessionViews,
                conversion_rate: campaignDetailed.conversionRate * 100,
                conversion_cost: $scope.currentCampaign.cost / campaignDetailed.campaignSessionSuccess,
                page_viewed: 0,
                page_viewed_per_user: 0
            };

            unomiQueryService.nbOfVisits($scope.currentCampaign.metadata.id, $scope.currentCampaign.startDate, $scope.currentCampaign.endDate, function (response) {
                var metricsData = response.data;
                $scope.currentCampaign.KPI.page_viewed = metricsData._sum;
                $scope.currentCampaign.KPI.page_viewed_per_user = metricsData._avg;
            });

            $scope.loading = true;

            $scope.dtOptions = DTOptionsBuilder.newOptions()
                .withOption('processing', true)
                .withOption('serverSide', true)
                .withOption('sDom', 't')
                .withOption('createdRow', createdRow)
                .withOption('order', [[2, 'desc']])
                .withFnServerData(serverData);

            $scope.dtColumns = [
                DTColumnBuilder.newColumn("metadata.name").withTitle(i18nService.message("serverSettings.manageCampaigns.campaignName")),
                DTColumnBuilder.newColumn("metadata.description").withTitle(i18nService.message("serverSettings.manageWem.description")).withOption('defaultContent', ''),
                DTColumnBuilder.newColumn("eventDate").withTitle(i18nService.message("serverSettings.manageCampaigns.eventDate")),
                DTColumnBuilder.newColumn("cost").withTitle(i18nService.message("serverSettings.manageCampaigns.event.cost")),
                DTColumnBuilder.newColumn(null).withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                    .renderWith(function (data, type, full, meta) {
                        return "<button type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteCampaignEvent('" + meta.row + "')\"><span class=\"material-icons\">delete</span></button>";
                    })
            ];

            $scope.deleteCampaignEvent = function (rowIndex) {
                var eventId = $scope.dtInstance.DataTable.row(rowIndex).data().metadata.id;
                var eventName = $scope.dtInstance.DataTable.row(rowIndex).data().metadata.name;
                modalService.newAlertModal({
                    title: i18nService.message("serverSettings.manageWem.delete"),
                    bodyMessage: i18nService.format("serverSettings.manageCampaigns.event.delete.confirm", eventName)
                }, function () {
                    unomiCampaignService.deleteCampaignEvent(eventId).then(function (response) {
                        $scope.campaignEvents = _.filter($scope.campaignEvents, function (item) {
                            return item.metadata.id != eventId;
                        });
                        $scope.dtInstance.DataTable.draw();
                        $scope.campaignChart.draw();
                    });
                });
            };

            $scope.createCampaignEvent = function () {
                var modalInstance = $uibModal.open({
                    templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-create-event-modal.html',
                    controller: 'CampaignCreateEventModalController',
                    size: 'lg',
                    resolve: {
                        "currentCampaign": function () {
                            return $scope.currentCampaign;
                        }
                    }
                });

                modalInstance.result.then(function (newEvent) {
                    var id = ManagersUtils.generateId();
                    var campaignEventDate = moment(newEvent.eventDate).tz(newEvent.timezone.label);
                    campaignEventDate.hours(newEvent.startTime.getHours());
                    campaignEventDate.minutes(newEvent.startTime.getMinutes());
                    campaignEventDate.seconds(0);
                    var event = {
                        metadata: {
                            id: id,
                            name: newEvent.name,
                            description: newEvent.description,
                            scope: ManagersContext.dmfScope
                        },
                        eventDate: campaignEventDate.toISOString(),
                        cost: newEvent.cost,
                        currency: newEvent.currency.value,
                        campaignId: $scope.currentCampaign.metadata.id,
                        timezone: newEvent.timezone.label
                    };
                    unomiCampaignService.setCampaignEvent(event).then(function () {
                        $scope.campaignEvents.push(event);
                        $scope.dtInstance.DataTable.draw();
                    });
                });
            };

            $scope.displayDate = function (date) {
                return moment(date).tz($scope.currentCampaign.timezone).format('LLLL z');
            };

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
            };
        }

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData)  + ":" + o.dir;
            }).join(",");

            unomiCampaignService.getCampaignEvents($scope.currentCampaign.metadata.id, length, start, order).then(function (response) {
                var data = response.data;
                var convertedData = [];
                angular.forEach(data.list, function (value, key) {
                    var eventMoment = moment(value.eventDate).tz(value.timezone);
                    var localEventMoment = eventMoment.clone().tz(jstz.determine().name());

                    convertedData.push({
                        "metadata": {
                            name: value.metadata.name,
                            "id": value.metadata.id,
                            "description": value.metadata.description
                        },
                        "eventDate": "<span data-tooltip='" + localEventMoment.format('LLLL z') + "'>" + eventMoment.format('LLLL z') + "&nbsp;(" + localEventMoment.fromNow() + ")" + "</span>",
                        "cost": "<span class='text-success'>(" + value.currency + ")" + value.cost + "</span>"
                    })
                });

                var results = {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": convertedData
                };
                fnCallback(results);
                dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            },
            function () {
                $scope.searchError = true;
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
                fnCallback(results);
            });
        }

        function createdRow(row, data, dataIndex) {
            $compile(angular.element(row).contents())($scope);
        }
    }
})();
