(function() {
    'use strict';

    angular.module('wemCampaignsApp').controller('CampaignsListController', CampaignsListController);

    CampaignsListController.$inject = [
            '$scope', 'unomiCampaignService', '$location', 'unomiGoalService', 'unomiRulesService',
            '_', 'DTOptionsBuilder', 'DTColumnBuilder', '$q', '$compile',
            'i18nService', 'modalService', 'dtPagerService'];

    function CampaignsListController($scope, unomiCampaignService, $location, unomiGoalService, unomiRulesService,
                                     _, DTOptionsBuilder, DTColumnBuilder, $q, $compile,
                                     i18nService, modalService, dtPagerService) {
        $scope.loading = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var colName = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                return ManagersUtils.campaignPropertiesForColName[colName] ? (ManagersUtils.campaignPropertiesForColName[colName] + ":" + o.dir) : null;
            }).join(",");

            unomiCampaignService.queryDetails(unomiCampaignService.constructQuery(length, start, order))
                .then(function (response) {
                    var data = response.data;
                    var convertedData = [];
                    angular.forEach(data.list, function (value, key) {
                        var startCampaignMoment = value.campaign.timezone != undefined ? moment(value.campaign.startDate).tz(value.campaign.timezone) : moment(value.campaign.startDate);
                        var endCampaignMoment = value.campaign.timezone != undefined ? moment(value.campaign.endDate).tz(value.campaign.timezone) : moment(value.campaign.endDate);

                        convertedData.push({
                            "id": value.campaign.metadata.id,
                            "name": value.campaign.metadata.name,
                            type: unomiCampaignService.getLocalizedCampaignType(unomiCampaignService.getCampaignType(value.campaign)),
                            "description": value.campaign.metadata.description,
                            "startDate": "<span>" + startCampaignMoment.format('LL') + "</span>",
                            "endDate": "<span>" + endCampaignMoment.format('LL') + "</span>",
                            "conversion": (value.conversionRate * 100).toFixed(2) + "%",
                            "engagedProfiles": value.engagedProfiles,
                            "numberOfGoals": value.numberOfGoals,
                            "primaryGoal" : value.campaign.primaryGoal
                        })
                    });
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": data.totalSize,
                        "iTotalDisplayRecords": data.totalSize,
                        "aaData": convertedData
                    };
                    fnCallback(results);
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    delete $scope.campaignRemoved;
                },
                function () {
                    $scope.searchError = true;
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
        }

        $scope.goalsByTag = {};

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('lengthMenu', [ 25, 50, 100 ])
            .withOption('pageLength', 25)
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 't')
            .withOption('order', [[3, 'asc']])
            .withOption('createdRow', createdRow)
            .withFnServerData(serverData);

        $scope.dtColumns = [
            {
                "data": 'name',
                "title": i18nService.message("serverSettings.manageCampaigns.campaignName"),
                "width": "10%"
            },
            {
                data: 'type',
                title: i18nService.message("serverSettings.manageCampaigns.type.short"),
                width: '10%'
            },
            {
                "data": 'description',
                "title": i18nService.message("serverSettings.manageWem.description"),
                "width": "10%"
            },
            {
                "data": "startDate",
                "title": i18nService.message("serverSettings.manageCampaigns.startDate"),
                "width": "10%"
            },
            {
                "data": "endDate",
                "title": i18nService.message("serverSettings.manageCampaigns.endDate"),
                "width": "10%"
            },
            DTColumnBuilder.newColumn("engagedProfiles").withTitle(i18nService.message("serverSettings.manageCampaigns.engaged")).notSortable()
                .withOption("width", "5%"),
            DTColumnBuilder.newColumn("conversion").withTitle(i18nService.message("serverSettings.manageCampaigns.conversion.rate")).notSortable()
                .withOption("width", "5%"),
            DTColumnBuilder.newColumn("numberOfGoals").withTitle(i18nService.message("serverSettings.manageCampaigns.goals")).notSortable()
                .withOption("width", "5%"),
            DTColumnBuilder.newColumn(null).withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                .renderWith(actionsHtml).withOption("width", "180px").withOption("contentPadding", "0")
        ];

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: " + row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        function rowCallback(row, aData, iDisplayIndex, iDisplayIndexFull) {
            // Unbind first in order to avoid any duplicate handler (see https://github.com/l-lin/angular-datatables/issues/87)
            $('td:not(:last-child)', row).unbind('click');
            $('td:not(:last-child)', row).bind('click', function () {
                $scope.$apply(function () {
                    $location.path('/campaigndashboard/' + aData.metadata.id);
                });
            });
            return row;
        }

        function actionsHtml(data, type, full, meta) {
            return "<button title=\"" + i18nService.message("serverSettings.manageGoals.campaignDashboard") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDashboard('" + data.id + "')\"><i class=\"material-icons\">multiline_chart</i></button> " +
                "<button title=\"" + i18nService.message("serverSettings.manageCampaigns.manageEngagedProfiles") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToEngagedProfiles('" + data.id + "')\" ><i class=\"material-icons\">people</i></button>" +
                "<button title=\"" + i18nService.message("serverSettings.manageCampaigns.manageConvertedProfiles") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToConvertedProfiles('" + data.id + "', '" + data.primaryGoal + "')\" ><i class=\"material-icons\">person</i></button>" +
                "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteCampaign('" + meta.row + "')\"><i class=\"material-icons\">delete</i></button>";
        }

        $scope.goToDashboard = function (id) {
            $location.path("/campaigndashboard/" + id);
        };

        $scope.goToEngagedProfiles = function (id) {
            $location.path("/campaign/" + id + "/profiles");
        };

        $scope.goToConvertedProfiles = function (campaignId, goalId) {
            $location.path("/campaign/" + campaignId + "/converted/" + goalId);
        };

        $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.campaingTags, i18nService);

        $scope.deleteCampaign = function (rowIndex) {
            var id = $scope.dtInstance.DataTable.row(rowIndex).data().id;
            var name = $scope.dtInstance.DataTable.row(rowIndex).data().name;
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.format("serverSettings.manageCampaigns.delete.confirm", name)
            }, function () {
                unomiGoalService.getAllForCampaign(id).then(function (response) {
                    var goalsMetadata = response.data;
                    unomiCampaignService.deleteCampaign(id).then(function () {
                        if(goalsMetadata && goalsMetadata.length) {
                            var promises = [];
                            _.forEach(goalsMetadata, function (goal) {
                                promises.push(unomiRulesService.deleteRule(unomiGoalService.getAddToListRulePrefix() + goal.id));
                            });
                            $q.all(promises).then(function (response) {
                                console.log("Add to list rules deleted successfully");
                            });
                        }
                    });
                    // reload the table
                    $scope.dtInstance.DataTable.draw();
                });
            });
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
        };
    }
})();
