(function() {
    'use strict';

    angular.module('wemCampaignsApp').config(wemCampaignsAppConfig);

    wemCampaignsAppConfig.$inject = ['$routeProvider'];

    function wemCampaignsAppConfig($routeProvider) {
        $routeProvider
            .when('/campaigns', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaigns-list.html',
                controller: 'CampaignsListController'

            })
            .when('/newCampaign', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-edit.html',
                controller: 'CampaignEditController',
                resolve: {
                    "currentCampaign": function(unomiCampaignService) {
                        var zonename = jstz.determine().name();

                        //The new campaign
                        if(unomiCampaignService.campaignToBeSaved) {
                            var currentCampaign = angular.copy(unomiCampaignService.campaignToBeSaved);
                            delete unomiCampaignService.campaignToBeSaved;
                            return currentCampaign;
                        } else {
                            return {
                                metadata: {
                                    id: ManagersUtils.generateId(),
                                    name: "",
                                    description: "",
                                    scope: ManagersContext.dmfScope,
                                    tags: ["referrer"]
                                },
                                startDate: moment().tz(zonename).toDate().toISOString(),
                                endDate: moment().tz(zonename).add(1, 'M').toDate().toISOString(),
                                cost: 0,
                                currency: "USD",
                                timezone: zonename
                            };
                        }
                    },
                    "isNew": function() {
                        return true;
                    }
                }
            })
            .when('/campaign/:campaignId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-edit.html',
                controller: 'CampaignEditController',
                resolve: {
                    // resolve rule here to avoid async issue using deferred API
                    "currentCampaign": function( $q, $route, unomiCampaignService) {
                        var campaign = $q.defer();
                        unomiCampaignService.getCampaign($route.current.params.campaignId).then(function(response){
                            campaign.resolve(response.data);
                        });
                        return campaign.promise;
                    },
                    "isNew": function() {
                        return false;
                    }
                }
            })
            .when('/campaign/:campaignId/newGoal', {
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-create.html',
                controller: 'CampaignGoalCreateController',
                resolve: {
                    "currentCampaign": function( $q, $route, unomiCampaignService) {
                        if (unomiCampaignService.campaignToBeSaved && unomiCampaignService.campaignToBeSaved.metadata.id == $route.current.params.campaignId) {
                            return unomiCampaignService.campaignToBeSaved;
                        }
                        var campaign = $q.defer();
                        console.log("Editing campaign:");
                        unomiCampaignService.getCampaign($route.current.params.campaignId).then(function(response){
                            campaign.resolve(response.data);
                        });
                        return campaign.promise;
                    },
                    "goal": function () {
                        return undefined
                    }
                }
            })
            .when('/campaigndashboard/:campaignId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-goals-list.html',
                controller: 'CampaignGoalsListController',
                resolve: {
                    // resolve rule here to avoid async issue using deferred API
                    "campaignDetailed": function( $q, $route, unomiCampaignService) {
                        var campaign = $q.defer();
                        unomiCampaignService.getCampaignDetail($route.current.params.campaignId).then(function(response){
                            var campaignData = response.data;
                            if (unomiCampaignService.updatedCampaign) {
                                campaignData.campaign = unomiCampaignService.updatedCampaign;
                                delete unomiCampaignService.updatedCampaign;
                            }
                            campaign.resolve(campaignData);
                        });
                        return campaign.promise;
                    }
                }
            })
            .when('/campaigndashboard/:campaignId/goal/:goalId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-create.html',
                controller: 'CampaignGoalCreateController',
                resolve: {
                    "currentCampaign": function( $q, $route, unomiCampaignService) {
                        if (unomiCampaignService.campaignToBeSaved && unomiCampaignService.campaignToBeSaved.metadata.id == $route.current.params.campaignId) {
                            return unomiCampaignService.campaignToBeSaved;
                        }
                        var campaign = $q.defer();
                        console.log("Editing campaign:");
                        unomiCampaignService.getCampaign($route.current.params.campaignId).then(function(response){
                            campaign.resolve(response.data);
                        });
                        return campaign.promise;
                    },
                    // resolve goal here to avoid async issue using deferred API
                    "goal": function( $q, $route, unomiGoalService) {
                        var goal = $q.defer();
                        unomiGoalService.getGoal($route.current.params.goalId).then(function(response){
                            goal.resolve(response.data);
                        });
                        return goal.promise;
                    }
                }
            })
            .when('/campaigndashboard/:campaignId/goal-report/:goalId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-report.html',
                controller: 'GoalReportController',
                resolve: {
                    // resolve goal here to avoid async issue using deferred API
                    "goal": function ($q, $route, unomiGoalService) {
                        var goal = $q.defer();
                        unomiGoalService.getGoal($route.current.params.goalId).then(function (response) {
                            goal.resolve(response.data);
                        });
                        return goal.promise;
                    }
                }
            })
            .when('/campaigndashboard/:campaignId/goal-report/:goalId/primary', {
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-report.html',
                controller: 'GoalReportController',
                resolve: {
                    // resolve goal here to avoid async issue using deferred API
                    "goal": function ($q, $route, unomiGoalService) {
                        var goal = $q.defer();
                        unomiGoalService.getGoal($route.current.params.goalId).then(function (response) {
                            var goalData = response.data;
                            goalData.isPrimary = true;
                            goal.resolve(goalData);
                        });
                        return goal.promise;
                    }
                }
            })
            .when('/campaign/:campaignId/profiles', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-profiles-list.html',
                controller: function ($scope, unomiProfileService) { }
            })
            .when('/campaign/:campaignId/profile/:profileId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/manage-profile.html',
                controller: 'ManageProfileController',
                resolve: {
                    // resolve profile here to avoid async issue using deferred API
                    "profile": function ($q, $route, unomiProfileService) {
                        var profile = $q.defer();
                        unomiProfileService.getProfile($route.current.params.profileId).then(function (response) {
                            profile.resolve(response.data);
                        });
                        return profile.promise;
                    }
                }
            })
            .when('/campaign/:campaignId/profile/:profileId/:sessionId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/profile-session.html',
                controller: 'ProfileSessionController'
            })
            .when('/campaign/:campaignId/converted/:goalId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-campaigns/campaign-profiles-list.html',
                controller: function ($scope, $route) {
                    $scope.goalId = $route.current.params.goalId;
                }
            })
            .when('/campaign/:campaignId/converted/:goalId/profile/:profileId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/manage-profile.html',
                controller: 'ManageProfileController',
                resolve: {
                    // resolve profile here to avoid async issue using deferred API
                    "profile": function ($q, $route, unomiProfileService) {
                        var profile = $q.defer();
                        unomiProfileService.getProfile($route.current.params.profileId).then(function (response) {
                            profile.resolve(response.data);
                        });
                        return profile.promise;
                    }
                }
            })
            .when('/campaign/:campaignId/converted/:goalId/profile/:profileId/:sessionId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/profile-session.html',
                controller: 'ProfileSessionController'
            })
            .otherwise({
                redirectTo: '/campaigns'
            });
    }
})();
