(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalLandingPageGoalController', GoalLandingPageGoalController);

    GoalLandingPageGoalController.$inject = ['$scope', 'i18nService'];

    function GoalLandingPageGoalController($scope, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.numberOfPagesView = {};
        $scope.targetEvents = [
            { id: 'pageView', name: i18nService.message('serverSettings.manageGoals.selectLandingType.pageView') },
            { id: 'formSubmit', name: i18nService.message('serverSettings.manageGoals.selectLandingType.formSubmit') },
            { id: 'numberOfPagesView', name: i18nService.message('serverSettings.manageGoals.selectLandingType.numberOfPagesView') }
        ];

        var conditions = ManagersUtils.getProperty($scope.goal, "startEvent.parameterValues.subConditions");
        if (conditions && conditions.length > 0) {
            conditions.forEach(function (condition) {
                if (condition.type == "pageViewEventCondition") {
                    $scope._goal.startPath = ManagersUtils.getProperty(condition, "parameterValues.pagePath")
                }
            })
        }

        if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "pageViewEventCondition") {
            $scope._goal.landingType = "pageView";
            $scope._goal.targetPath = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.pagePath");
        } else if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "booleanCondition") {
            var subConditions = ManagersUtils.getProperty($scope.goal, "targetEvent.parameterValues.subConditions");
            if (subConditions) {
                for (var i = 0; i < subConditions.length; i++) {
                    var subCondition = subConditions[i];
                    if (subCondition.type == "formEventCondition") {
                        $scope._goal.targetForm = ManagersUtils.getProperty(subCondition, "parameterValues.formId");
                        $scope._goal.landingType = "formSubmit";
                    }
                    if (subCondition.type == "hasSeenNPagesCondition") {
                        $scope._goal.landingType = "numberOfPagesView";
                        $scope._goal.numberOfPagesView.value = subCondition.parameterValues.value + 1;
                    }
                }
            }
        }

        var doSave = function(callback) {
            // construct landing page event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "pagePath": $scope._goal.startPath
                            },
                            "type": "pageViewEventCondition"
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "duration",
                                "comparisonOperator": "equals",
                                "propertyValueInteger": 0
                            }
                        }
                    ]
                }
            };
            switch ($scope._goal.landingType) {
                case "pageView":
                    $scope.goal.targetEvent = {
                        "parameterValues": {
                            "pagePath": $scope._goal.targetPath
                        },
                        "type": "pageViewEventCondition"
                    };
                    break;
                case "formSubmit":
                    $scope.goal.targetEvent = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "parameterValues": {
                                        "formId": $scope._goal.targetForm
                                    },
                                    "type": "formEventCondition"
                                },
                                {
                                    "parameterValues": {
                                        "path": $scope._goal.startPath
                                    },
                                    "type": "sourceEventPropertyCondition"
                                }
                            ]
                        }
                    };
                    break;
                case "numberOfPagesView":
                    $scope.goal.targetEvent = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "pageViewEventCondition",
                                    "parameterValues": {}
                                },
                                {
                                    "type": "hasSeenNPagesCondition",
                                    "parameterValues": {
                                        "value": $scope._goal.numberOfPagesView.value - 1,
                                        "scope": ManagersContext.currentSiteKey,
                                        "comparisonOperator": 'greaterThanOrEqualTo'
                                    }
                                }
                            ]
                        }
                    };
                    break;
            }

            if (callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback) {
                if (callback) {
                    callback();
                }
            };

            $scope.callback.save = function (callback) {
                doSave(callback);
            }
        }
    }
})();
