(function() {
    'use strict';

    angular.module('wemGoalsApp').directive('goalReportTable', goalReportTableDirective);

    function goalReportTableDirective() {
        return {
            restrict: 'E',
            scope: {
                goal: '=',
                properties: '=',
                data: '=ngModel',
                type: '@',
                queryRowHook: '@',
                queryHook: '@',
                tableColumnsHook: '@'
            },
            templateUrl: ManagersContext.wemAppPath + '/wem-goals/goal-report-table.directive.html',
            controller: GoalReportTableController
        };
    }

    GoalReportTableController.$inject = ['$scope', 'unomiGoalService', 'i18nService', 'unomiSegmentService', 'DTOptionsBuilder', 'DTColumnBuilder', '$uibModal', 'unomiGeonamesService'];

    function GoalReportTableController($scope, unomiGoalService, i18nService, unomiSegmentService, DTOptionsBuilder, DTColumnBuilder, $uibModal, unomiGeonamesService) {
        $scope.facets = {};
        $scope.split = [];
        $scope.ranges = {};

        $scope.actionsHtml = function (data, type, full, meta) {
            // data.selectedProperty.
            return '<button title="' + i18nService.message('serverSettings.manageSegments.manageMembers') + '" type="button" class="btn btn-icon command-profiles" ng-click="openManagement(' + data.key + ')" ><i class="material-icons">person</i></button>';
        };

        $scope.openManagement = function () {
            var dialogModal = {
                backdrop: true,
                keyboard: true,
                templateUrl: ManagersContext.wemAppPath + '/wem-goals/sessions-list-modal.html',
                controller: 'SessionsListModalController',
                resolve: {
                    scopeParent: function () {
                        return $scope; // We pass a ref of the parent scope to the modal
                    },
                    currentProperty: function () {
                        return $scope.data.selectedProperty.itemId;
                    }
                }
            };
            $uibModal.open(dialogModal);
        };

        $scope.setDtInstance = function (val) {
            $scope.dtInstance = val;
        };

        $scope.isNotFacetedProp = function (value, index) {
            return !$scope.facets[value.id];
        };

        $scope.showSegment = function (value, index) {
            return value.itemType != 'segments' || (value.itemType == 'segments' && $scope.isQualitativeReport());
        };

        $scope.showRangeCheckbox = function () {
            return $scope.data && $scope.data.selectedProperty && $scope._isPropertyRanged($scope.data.selectedProperty);
        };

        $scope.showDateInterval = function () {
            return $scope.data && $scope.data.selectedProperty && $scope.data.selectedProperty.type == 'date';
        };

        $scope.getFacetDisplay = function (facet) {
            return facet.property.metadata.name + ' : ' + (facet.value.displayableKey ? ((facet.value.displayableKey.length > 50)?facet.value.displayableKey.substring(0, 50) + '...':facet.value.displayableKey) : facet.value.key);
        };

        $scope.addFacet = function (selectedProperty, rowValue) {
            var facet = {
                property: angular.copy(selectedProperty),
                value: angular.copy(rowValue),
                conditions: []
            };

            // resolve associated conditions to selectedProperty and rowValue
            var isSession = facet.property.itemType == 'sessions';

            var propertyName = facet.property.metadata.id;
            if (facet.property.itemType == 'sessions') {
                propertyName = 'properties.' + propertyName;
            } else if (facet.property.itemType == 'profiles') {
                propertyName = 'profile.properties.' + propertyName;
            } else if (propertyName == 'segments') {
                propertyName = 'profile.segments'
            }
            if ($scope._isPropertyRanged(facet.property) && $scope.data.useRanges) {
                var range = $scope.ranges[facet.property.metadata.id] ? $scope.ranges[facet.property.metadata.id][facet.value.key] : undefined;
                if (range) {
                    if (range.from) {
                        facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'greaterThan', facet.property.type, false, range.from));
                    }
                    if (range.to) {
                        facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'lessThan', facet.property.type, false, range.to));
                    }
                }
            } else if (facet.property.type == 'date') {
                // handle date range
                for (var i = 0; i < $scope.split.length; i++) {
                    var split = $scope.split[i];
                    if (split.key == facet.value.key) {
                        if (i == $scope.split.length - 1) {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'greaterThan', facet.property.type, false,
                                $scope._getSelectedIntervalAsDate(split.key)));
                        } else {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'between', facet.property.type, true,
                                [$scope._getSelectedIntervalAsDate(split.key), $scope._getSelectedIntervalAsDate($scope.split[i + 1].key)]));
                        }
                        break;
                    }
                }
            } else {
                if (facet.value.missing) {
                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'missing', facet.property.type, false, null));
                } else {
                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, 'equals', facet.property.type, false, facet.value.key));
                }
            }


            $scope.facets[selectedProperty.metadata.id] = facet;
            var filteredProperties = $scope._getFilteredProperties();
            $scope.data.selectedProperty = filteredProperties && filteredProperties.length > 0 ? filteredProperties[0] : undefined;
        };

        $scope.removeFacet = function (facet) {
            if (facet && $scope.facets[facet.property.metadata.id]) {
                delete $scope.facets[facet.property.metadata.id];
                $scope.data.selectedProperty = $scope._getProperty(facet.property.metadata.id);
            }
        };

        $scope.isQualitativeReport = function () {
            return $scope.type == 'qualitative';
        };

        $scope.isQuantitativeReport = function () {
            return $scope.type == 'quantitative';
        };

        $scope.getTableLabel = function () {
            return i18nService.message('serverSettings.manageGoals.report.title.' + $scope.type);
        };

        $scope._getProperty = function (propertyId) {
            for (var propIndex in $scope.properties) {
                if (propertyId == $scope.properties[propIndex].metadata.id) {
                    return $scope.properties[propIndex];
                }
            }
            return null;
        };

        $scope._getFilteredProperties = function () {
            return $scope.properties.filter(function (property) {
                return !$scope.facets[property.metadata.id];
            });
        };

        $scope._isPropertyRanged = function (property) {
            return (property.numericRanges && property.numericRanges.length > 0) ||
                (property.dateRanges && property.dateRanges.length > 0) ||
                (property.ipRanges && property.ipRanges.length > 0);
        };

        $scope._getPropertyCondition = function (isSession, propertyName, operator, type, multiple, propertyValue) {
            var condition = {
                'type': isSession ? 'sessionPropertyCondition' : 'profilePropertyCondition',
                'parameterValues': {
                    'propertyName': propertyName,
                    'comparisonOperator': operator
                }
            };

            switch (type) {
                case 'integer':
                    if (multiple) {
                        condition.parameterValues.propertyValuesInteger = propertyValue;
                    } else {
                        condition.parameterValues.propertyValueInteger = propertyValue;
                    }
                    break;
                case 'date':
                    if (multiple) {
                        condition.parameterValues.propertyValuesDate = propertyValue;
                    } else {
                        condition.parameterValues.propertyValueDate = propertyValue;
                    }
                    break;
                default :
                    condition.parameterValues.propertyValue = propertyValue;
            }

            return condition;
        };

        $scope._getI18nForRange = function (type, to, from) {
            var t,f;
            if (to != 'now') {
                t = to;
            }
            if (from != 'now') {
                f = from;
            }
            if (type == 'date') {
                if (t && _.isString(t)) {
                    t = ManagersUtils.parseDateMath(t);
                }
                if (f && _.isString(f)) {
                    f = ManagersUtils.parseDateMath(f);
                }
            }
            if (t && f) {
                return i18nService.format('ranges.' + type, f.toString() + '|' + t.toString());
            }
            if (t) {
                return i18nService.format('ranges.' + type + '.leftLimit', t.toString());
            }
            if (f) {
                return i18nService.format('ranges.' + type + '.rightLimit', f.toString());
            }
        };

        $scope._getSelectedIntervalAsDate = function (value) {
            var splitted = value.split('-');
            if ($scope.data.selectedInterval.key == '1M') {
                return new Date(splitted[0], parseInt(splitted[1] - 1), 1, null, null, null, null)
            } else {
                return new Date(splitted[0], parseInt(splitted[1] - 1), splitted[2], null, null, null, null)
            }
        };

        $scope._addConditionToQuery = function (query, condition) {
            if (query.condition) {
                query.condition.parameterValues.subConditions.push(condition);
            } else {
                query.condition = {
                    'type': 'booleanCondition',
                    'parameterValues': {
                        'operator': 'and',
                        'subConditions': []
                    }
                };
                $scope._addConditionToQuery(query, condition);
            }
        };

        function doQuery(callback) {
            if ($scope.data) {
                var aggregate;
                switch ($scope.data.selectedProperty.itemType) {
                    case 'profiles':
                        aggregate = 'profile.properties.' + $scope.data.selectedProperty.metadata.id;
                        break;
                    case 'segments':
                        aggregate = 'profile.segments';
                        break;
                    default :
                        aggregate = 'properties.' + $scope.data.selectedProperty.metadata.id;
                }
                //
                var query = {
                    aggregate: {
                        property: aggregate
                    }
                };

                // handle ranged properties
                if ($scope._isPropertyRanged($scope.data.selectedProperty) && $scope.data.useRanges) {
                    var ranges;
                    if ($scope.data.selectedProperty.ipRanges.length > 0) {
                        query.aggregate.type = 'ipRange';
                        query.aggregate.ipRanges = angular.copy($scope.data.selectedProperty.ipRanges);
                        ranges = query.aggregate.ipRanges;
                    } else if ($scope.data.selectedProperty.dateRanges.length > 0) {
                        query.aggregate.type = 'dateRange';
                        query.aggregate.dateRanges = angular.copy($scope.data.selectedProperty.dateRanges);
                        ranges = query.aggregate.dateRanges;
                    } else if ($scope.data.selectedProperty.numericRanges.length > 0) {
                        query.aggregate.type = 'numericRange';
                        query.aggregate.numericRanges = angular.copy($scope.data.selectedProperty.numericRanges);
                        ranges = query.aggregate.numericRanges;
                    }
                    $scope.ranges[$scope.data.selectedProperty.metadata.id] = {};
                    angular.forEach(ranges, function (v, k) {
                        $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                        v.key = $scope._getI18nForRange($scope.data.selectedProperty.type, v.to, v.from);
                        $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                    });
                } else if ($scope.data.selectedProperty.type == 'date') {
                    query.aggregate.type = 'date';
                    query.aggregate.parameters = {
                        'interval': $scope.data.selectedInterval.key,
                        'format': 'yyyy-MM-dd'
                    }
                }

                // handle fromDate and toDate
                if ($scope.data.fromDate && $scope.data.toDate) {
                    var queryFromDate = moment($scope.data.fromDate).toISOString();
                    var queryToDate = moment($scope.data.toDate).endOf($scope.data.timelineInterval.unit).toISOString();
                    $scope._addConditionToQuery(query, $scope._getPropertyCondition(true, 'timeStamp', 'between', 'date', true, [queryFromDate, queryToDate]));
                }

                // handle facets
                if ($scope.facets) {
                    for (var key in $scope.facets) {
                        if ($scope.facets.hasOwnProperty(key)) {
                            var facet = $scope.facets[key];
                            if (facet.conditions && facet.conditions.length > 0) {
                                for (var i = 0; i < facet.conditions.length; i++) {
                                    $scope._addConditionToQuery(query, facet.conditions[i]);
                                }
                            }
                        }
                    }
                }


                if ($scope.goal) {
                    unomiGoalService.getGoalReportAggregated($scope.goal.metadata.id, query).then(function (response) {
                        var data = response.data;
                        if ($scope.queryHook) {
                            $scope.$parent[$scope.queryHook](data, $scope.data);
                        }

                        angular.forEach(data.split, function (v, k) {
                            if (v.key == '_missing') {
                                v.missing = true;
                                v.key = i18nService.message('serverSettings.manageGoals.report.table.missing');
                            } else if ($scope.data.selectedProperty.itemType == 'segments') {
                                if ($scope.segmentsIndexedById.hasOwnProperty(v.key)) {
                                    v.displayableKey = $scope.segmentsIndexedById[v.key].name;
                                } else {
                                    v.displayableKey = i18nService.message('serverSettings.manageGoals.report.table.unknownSegment')
                                }
                            } else if (query.aggregate.type == 'date') {
                                v.displayableKey = moment(v.key).format($scope.data.selectedInterval.aggregateFormat);
                            }
                            if ($scope.queryRowHook) {
                                $scope.$parent[$scope.queryRowHook](v, $scope.data);
                            }
                        });
                        if (callback) {
                            callback(data.split);
                        }
                    });
                } else {
                    var aggregateSessionQuery = {
                        'aggregate': {
                            'type': 'string',
                            'property': aggregate
                        }
                    };

                    var tabSplit = [];
                    var queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext;
                    unomiQueryService.query('session', aggregate, aggregateSessionQuery).then(function (response) {
                        var aggregateSessionsData = response.data;
                        if (callback) {
                            var tabLine = {};
                            if (aggregateSessionsData._all) {
                                angular.forEach(aggregateSessionsData, function (v, k) {
                                    if (k != '_all') {
                                        tabLine = {};
                                        tabLine.key = k;
                                        tabLine.targetCount = v;
                                        tabLine.percentage = v / aggregateSessionsData._all;
                                        tabSplit.push(tabLine);
                                    }
                                });
                            }
                            callback(tabSplit);
                        }
                    });


                }

            }
        }

        function loadTableColumns() {
            var table = [];
            if ($scope.isQualitativeReport()) {
                table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                    return full.displayableKey ? full.displayableKey : data;
                }));
                table.push(DTColumnBuilder.newColumn('startCount').withTitle(i18nService.message('serverSettings.manageGoals.views')));
                table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.success')));
                table.push(DTColumnBuilder.newColumn('conversionRate').withTitle(i18nService.message('serverSettings.manageGoals.conversion')).renderWith(function (data, type, full) {
                    return ManagersUtils.getPercentage(data) + ' %';
                }));
            } else {
                table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                    return full.displayableKey ? full.displayableKey : data;
                }));
                if ($scope.goal) {
                    table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.success')));
                    table.push(DTColumnBuilder.newColumn('percentage').withTitle(i18nService.message('serverSettings.manageGoals.percentage')).renderWith(function (data, type, full) {
                        return ManagersUtils.getPercentage(data) + ' %';
                    }));
                } else {
                    table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message('serverSettings.manageGoals.count')));
                    table.push(
                        DTColumnBuilder
                            .newColumn(null)
                            .withTitle(i18nService.message('serverSettings.manageWem.listAction'))
                            .notSortable()
                            .renderWith($scope.actionsHtml)
                            .withOption('width', '25%')
                            .withOption('contentPadding', '0'));

                }
            }
            if ($scope.tableColumnsHook) {
                $scope.$parent[$scope.tableColumnsHook](table, DTColumnBuilder, $scope.data);
            }
            $scope.dtColumns = angular.copy(table);
        }

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('aaSorting', [])
            .withOption('sDom', 'lt')
            .withDisplayLength(10)
            .withOption('lengthMenu', [10, 25, 50, 100])
            .withOption('rowCallback', function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $('td', nRow).unbind();
                $('td', nRow).bind('click', function () {
                    $scope.$apply(function () {
                        $scope.addFacet($scope.data.selectedProperty, aData);
                    });
                });
                $('td .btn.btn-icon.command-profiles', nRow).unbind();
                $('td .btn.btn-icon.command-profiles', nRow).bind('click', function (event) {
                    event.stopPropagation();
                    $scope.openManagement();

                });
                return nRow;
            });

        // watch data to refresh table
        $scope.$watch('data', function(newVal, oldVal) {
            if ($scope.data) {
                doQuery(function(split) {
                    //Handle geonames properties in goal report
                    if ($scope.data.selectedProperty.itemId && $scope.data.selectedProperty.itemId.indexOf('sessionAdminSubDiv') != -1) {
                        //Browsing on each table line
                        _.forEach(split, function(object, index) {
                            //If not localhost
                            if (object.key != 'GE' && object.key != 2500 && object.key!= 'Not provided' ) {
                                var geoId = object.key;
                                //Request the geonames entity for the property value
                                unomiGeonamesService.getCapital(geoId).then(function(response) {
                                    var data = response.data;
                                    var filtered = _.filter(data, function(o) {
                                        return o.itemId == geoId;
                                    });
                                    if (filtered.length > 0 ) {
                                        //If result in geonames display the geonames entity name
                                        split[index].displayableKey = filtered[0].name
                                    }
                                    if (index == (split.length - 1)) {
                                        //Refresh datatable after geonames call for the last line
                                        $scope.split = split;
                                        $scope.dtOptions.data = split;
                                        loadTableColumns();
                                    }
                                });
                            }
                        });
                    }

                    $scope.split = split;
                    $scope.dtOptions.data = split;
                    loadTableColumns();
                });
            }
        }, true);


        if ($scope.isQualitativeReport()) {
            unomiSegmentService.getAll().then(function (response) {
                $scope.segmentsIndexedById = _.indexBy(response.data, 'id');
            });
        }

    }
})();
