(function() {
    'use strict';

    angular.module('wemGoalsApp').controller('GoalsListController', GoalsListController);

    GoalsListController.$inject = ['$scope', '$route', 'unomiGoalService', 'unomiQueryService', 'i18nService', 'currentCampaign', 'filteringService', 'unomiProfileService', '_', 'loadingSpinnerService'];

    function GoalsListController($scope, $route, unomiGoalService, unomiQueryService, i18nService, currentCampaign, filteringService, unomiProfileService, _, loadingSpinnerService) {
        $scope.aggregateSessions = null;
        $scope.aggregateNewSessions = {};
        $scope.lineChartData = {
            labels: [],
            datasets: [],
            series: []
        };
        $scope.goalsByTag = {};
        $scope.currentCampaign = currentCampaign;
        $scope.primaryGoal = {};

        // timeline data
        $scope.timelineData = {
            toDate: _.isUndefined(currentCampaign) ? moment().endOf('month').toDate() : moment(currentCampaign.endDate).toDate(),
            fromDate: _.isUndefined(currentCampaign) ? moment().subtract(1, "years").startOf('month').toDate() : moment(currentCampaign.startDate).toDate(),
            interval: ManagersUtils.supportedTimelineIntervalesObj[_.isUndefined(currentCampaign) ? "1M" : "1d"],
            title: _.isUndefined(currentCampaign) ? "serverSettings.manageGoals.sessionsTimeline" : currentCampaign.metadata.name,
            currentCampaign: currentCampaign,
            series: [i18nService.message('serverSettings.manageGoals.allSessions'), i18nService.message('serverSettings.manageGoals.newSessions')],
            loadData: function (success) {
                var aggregateSessionQuery = {
                    "aggregate": {
                        "type": "date",
                        "parameters": {
                            "interval": this.interval.key,
                            "format": "yyyy-MM-dd"
                        }
                    },
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "scope",
                                        "comparisonOperator": "equals",
                                        "propertyValue": ManagersContext.dmfScope
                                    }
                                }
                            ]
                        }
                    }
                };
                // get all sessions
                unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                    var aggregateSessionsData = response.data;
                    var datasets = [];
                    datasets[0] = aggregateSessionsData;

                    // get new sessions:
                    aggregateSessionQuery.condition.parameterValues.subConditions.push({
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "profile.properties.nbOfVisits",
                            "comparisonOperator": "equals",
                            "propertyValueInteger": 1
                        }
                    });
                    unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                        datasets[1] = response.data;
                        success(datasets);
                    });
                });
            }
        };

        var processGoals = function (goals, currentCampaign) {
            goals = filteringService.filterItems(goals, unomiGoalService);

            if (goals && goals.length > 0) {
                goals.forEach(function (goal) {
                    if (currentCampaign && goal.id == currentCampaign.primaryGoal) {
                        $scope.primaryGoal = goal;
                        goal.isPrimary = true;
                    } else {
                        if (goal.tags && goal.tags.length > 0 && !goal.hidden) {
                            var tag = $scope.goalsByTag[goal.tags[0]];
                            if (tag) {
                                if (tag.goals) {
                                    tag.goals.push(goal);
                                } else {
                                    tag.goals = [goal];
                                }
                            }
                        }
                    }
                });
            }
        };

        $scope.goalsByTag = {};
        // load goals
        // process tags and create a dictionary by tag id
        if (_.isUndefined(currentCampaign)) {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService);
            unomiGoalService.getAllForSite().then(function (response) {
                processGoals(response.data);
                loadingSpinnerService.hide();
            });
        } else {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.campaingTags, i18nService);
            unomiGoalService.getAllForCampaign(currentCampaign.metadata.id).then(function (response) {
                processGoals(response.data, currentCampaign);
                $scope.subGoalsAvailable = _.reduce($scope.goalsByTag, function (memo, tag) {
                    return memo + tag.goals.length;
                }, 0) > 0;
                loadingSpinnerService.hide();
            });
        }

        $scope.refresh = function () {
            $route.reload();
        };

        $scope.subGoalsAvailable = false;

        // ABO(+) :
        $scope.quantitativeProperties = [];
        unomiProfileService.getPropertyTypes().then(function (response) {
            var data = response.data;
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {

                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": '',
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": ''
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                        });
                    }
                });
            }
        });
    }
})();
