(function() {
    'use strict';

    angular.module('wemListsApp').config(wemListsAppConfig);

    wemListsAppConfig.$inject = ['$routeProvider'];

    function wemListsAppConfig($routeProvider) {
        $routeProvider
            .when('/lists', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segments-list.html',
                controller: 'SegmentsListController',
                reloadOnSearch: false
            })
            .when('/list/:listId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-edit.html',
                controller: 'SegmentEditController',
                resolve: {
                    // resolve rule here to avoid async issue using deferred API
                    "currentSegment": function( $q, $route, unomiListService) {
                        var list = $q.defer();
                        unomiListService.getList($route.current.params.listId).then(function (response) {
                            list.resolve(response.data);
                        });
                        return list.promise;
                    },
                    "newSegment": function () {
                        return false;
                    }
                }
            })
            .when('/newList', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-edit.html',
                controller: 'SegmentEditController',
                resolve: {
                    "currentSegment": function () {
                        return {
                            metadata: {
                                id: ManagersUtils.generateId(),
                                scope: ManagersContext.dmfScope,
                                enabled: true
                            }
                        };
                    },
                    "newSegment": function () {
                        return true;
                    }
                }
            })
            .when('/list/:listId/profiles', {
                templateUrl: ManagersContext.wemAppPath + '/wem-segments/segment-profiles-list.html',
                controller: function($scope) {
                    $scope.profileType='list';
                }
            })
            .when('/list/:listId/profile/:profileId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/manage-profile.html',
                controller: 'ManageProfileController',
                resolve: {
                    // resolve profile here to avoid async issue using deferred API
                    "profile": function ($q, $route, unomiProfileService) {
                        var profile = $q.defer();
                        unomiProfileService.getProfile($route.current.params.profileId).then(function (response) {
                            profile.resolve(response.data);
                        });
                        return profile.promise;
                    }
                }
            })
            .when('/list/:listId/profile/:profileId/:sessionId', {
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/profile-session.html',
                controller: 'ProfileSessionController'
            })
            .otherwise({
                redirectTo: '/lists'
            });
    }
})();
