(function() {
    'use strict';

    angular.module('wemManageOptimizationTestApp').controller('ManageOptimizationTestTrafficController', ManageOptimizationTestTrafficController);

    ManageOptimizationTestTrafficController.$inject = ['$scope', 'i18nService'];

    function ManageOptimizationTestTrafficController($scope, i18nService) {
        $scope.variants = variants;
        $scope.variantsTraffic = variantsTraffic;
        $scope.hasVariants = $scope.variants.length > 0;
        $scope.totalTraffic = 0;

        var init = function() {
            if ($scope.hasVariants) {
                if (angular.equals({}, $scope.variantsTraffic)) {
                    $scope.totalTraffic = 100;
                    var length = $scope.variants.length;
                    var variantValue = Math.floor(100/length);
                    var rem = 100 % length;
                    angular.forEach($scope.variants, function(variant, index) {
                        variant.traffic = variantValue;
                        if (index == (length - 1)) {
                            variant.traffic += rem;
                        }
                    });
                } else {
                    angular.forEach($scope.variants, function(variant) {
                        if (variant.nodeIdentifier in $scope.variantsTraffic) {
                            $scope.totalTraffic += $scope.variantsTraffic[variant.nodeIdentifier];
                            variant.traffic = $scope.variantsTraffic[variant.nodeIdentifier];
                        } else {
                            $scope.totalTraffic += 0;
                            variant.traffic = 0;
                        }
                    });
                }
            }
        };

        init();

        $scope.updateTotalTraffic = function() {
            var totalTraffic = 0;
            angular.forEach($scope.variants, function(variant) {
                totalTraffic += variant.traffic;
            });
            $scope.totalTraffic = totalTraffic;
        };

        $scope.validate = function(gwtValidation) {
            if ($scope.totalTraffic != 100) {
                gwtValidation.addValidationError(i18nService.message('wem_optimizationTest.customTrafficManagementEngine.error.totalTraffic'));
            }
            return true;
        };

        $scope.save = function(gwtNode) {
            $scope.variantsTraffic = {};
            angular.forEach($scope.variants, function (variant) {
                $scope.variantsTraffic[variant.nodeIdentifier] = variant.traffic;
            });
            gwtNode.setProperty("wem:variantsTraffic", JSON.stringify($scope.variantsTraffic), 1);
        };

        $scope.trimVariantName = function (variantName) {
            if (variantName.length > 25) {
                return variantName.substring(0, 25) + '...';
            } else {
                return variantName;
            }
        }
    }
})();
