(function() {
    'use strict';

    angular.module('wemProfilesApp').controller('ManageProfileController', ManageProfileController);

    ManageProfileController.$inject = ['$scope', '$routeParams', '$resource', 'i18nService', 'unomiProfileService', 'profile', '$http', 'DTOptionsBuilder', 'DTColumnBuilder', 'unomiListService', '$uibModal', 'unomiSegmentService', 'unomiGoalService', '$location', 'unomiQueryService', '$compile', 'modalService', 'notificationService', 'unomiScoringService', 'jcrService', 'loadingSpinnerService', 'dtPagerService'];

    function ManageProfileController($scope, $routeParams, $resource, i18nService, unomiProfileService, profile, $http, DTOptionsBuilder, DTColumnBuilder, unomiListService, $uibModal, unomiSegmentService, unomiGoalService, $location, unomiQueryService, $compile, modalService, notificationService, unomiScoringService, jcrService, loadingSpinnerService, dtPagerService) {
        if (profile.itemType == 'persona') {
            $scope.profileType = 'personas';
        } else {
            $scope.profileType = 'profile';
        }
        if (!unomiProfileService.getSelectedTab()) {
            unomiProfileService.setSelectedTab({
                type: 'visitors'
            });
        }
        $scope.countries = ManagersContext.countries;
        $scope.loaded = true;
        $scope.propertyTypesLoaded = false;
        $scope.propertiesLoaded = false;
        $scope.profileId = $routeParams.profileId;
        $scope.currentProfile = profile;
        $scope.siteKey = ManagersContext.dmfScope;
        $scope.sites = {};
        // compute age from birth date
        if (profile.properties && profile.properties.birthDate) {
            var birthDate = new Date(profile.properties.birthDate);
            var cur = new Date();
            var diff = cur - birthDate;
            $scope.age = i18nService.format("profilesProperty.age.value", Math.floor(diff / 31536000000).toString());
        }

        $scope.currentProfileHasSegments = function () {
            return $scope.currentProfile.segments && !angular.equals([], $scope.currentProfile.segments);
        };

        $scope.currentProfileHasLists = function () {
            return $scope.currentProfile.systemProperties.lists && !angular.equals([], $scope.currentProfile.systemProperties.lists);
        };

        $scope.currentProfileHasScores = function () {
            return $scope.currentProfile.scores && !angular.equals({}, $scope.currentProfile.scores);
        };

        function updateProfileView() {
            if ($scope.currentProfile.properties) {
                $scope.profileName = ManagersUtils.getProfileDisplayName(i18nService, $scope.currentProfile.properties);

                if (profile.itemType == 'persona') {
                    $scope.profileRB = 'managePersonas';
                } else if ($scope.currentProfile.segments.indexOf("leads") != -1) {
                    $scope.profileRB = "manageLeads";
                } else if ($scope.currentProfile.segments.indexOf("contacts") != -1) {
                    $scope.profileRB = "manageContacts";
                } else {
                    $scope.profileRB = "manageVisitors";
                }
                $scope.profileTypeLabel = i18nService.message("serverSettings." + $scope.profileRB + ".profileType");
            }
        }

        updateProfileView();

        if ($scope.currentProfile.properties) {
            if ($scope.currentProfile.properties.firstVisit) {
                $scope.firstVisitTimeAgo = moment($scope.currentProfile.properties.firstVisit).fromNow();
            }
            if ($scope.currentProfile.properties.lastVisit) {
                $scope.lastVisitTimeAgo = moment($scope.currentProfile.properties.lastVisit).fromNow();
            }
        }

        var propertyTypeRootTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/tags/profileProperties?filterHidden=true', {}, {
            'get': {
                method: 'GET',
                isArray: false
            }
        });
        var propertyTypesByTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/tags/:tagId', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });
        var existingSessionPropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/existingProperties?tagId=sessionProperties&itemType=session', {}, {
            'get': {
                method: 'GET',
                isArray: true
            }
        });

        $scope.propertiesByTag = {};
        propertyTypeRootTagResource.get(function (rootTag) {
            console.log("manage-profile.controller.js: Loaded " + rootTag.subTags.length + " property type tags from Apache Unomi.");
            for (var i = 0; i < rootTag.subTags.length; i++) {
                var propertyTag = rootTag.subTags[i];
                $scope.propertiesByTag[propertyTag.id] = propertyTypesByTagResource.get({tagId: propertyTag.id}, function (propertyTypes) {
                    console.log("manage-profile.controller.js: Loaded " + propertyTypes.length + " property types from Apache Unomi.");
                });
            }
        });

        $scope.middleIndex = function (properties) {
            if (properties) {
                return Math.ceil(properties.length / 2);
            }
            return 0;
        };

        $scope.loading = true;

        $scope.saveProfile = function () {
            if ($scope.currentProfile.itemId == "newPersona") {
                $scope.currentProfile.itemId = ManagersUtils.generateId();
                unomiProfileService.saveProfile($scope.currentProfile, $scope.profileType).then(function (response) {
                    notificationService.successToast(i18nService.message("serverSettings.manageProfiles.save.success"));
                    $location.path('/persona/' + $scope.currentProfile.itemId);
                });
            } else {
                unomiProfileService.saveProfile($scope.currentProfile, $scope.profileType).then(function (response) {
                    $scope.currentProfile = response.data;
                    updateProfileView();
                    notificationService.successToast(i18nService.message("serverSettings.manageProfiles.save.success"));
                });
            }
        };

        $scope.getPropertyInputType = function (property) {
            if (property.type == "integer") {
                return "number";
            } else if (property.type == "email") {
                return "email";
            } else if (property.type == "date") {
                return "date";
            } else if (property.type == "boolean") {
                return "boolean";
            }
            return "text";
        };

        $scope.resetProfile = function () {
            unomiProfileService.getProfile($scope.profileId, $scope.profileType).then(function (response) {
                var profile = response.data;
                console.log("manage-profile.controller.js: Loaded profile " + profile.itemId + " succesfully from Apache Unomi.");
                $scope.currentProfile = profile;
                $scope.propertiesLoaded = true;
                $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;
            });
        };

        $scope.deleteProfile = function (profileId) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings." + $scope.profileRB + ".delete"),
                bodyMessage: i18nService.message("serverSettings." + $scope.profileRB + ".delete.confirm")
            }, function () {
                unomiProfileService.deleteProfile($scope.profileId, $scope.profileType).then(function (response) {
                    $location.url("/" + (profile.itemType == 'persona' ? "personas" : "profiles") + "?deletedProfileId=" + $scope.profileId);
                });
            });
        };

        /**
         * Copy a profile and its 10 last sessions into a persona
         */
        $scope.copyAsPersona = function() {
            loadingSpinnerService.show();
            var personaObject = {
                persona: {}
            };
            unomiProfileService.getProfile($scope.profileId).then(function (response) {
                var data = response.data;
                //Creating the persona object from the profile
                personaObject.persona["itemId"] = "MF_persona_" + new Date().getTime();
                personaObject.persona["itemType"] = "persona";
                personaObject.persona.properties = data.properties;
                personaObject.persona.properties["description"] = "persona copied from " + $scope.profileId;
                personaObject.persona.segments = data.segments;
                //Getting the profile 10 last sessions
                unomiProfileService.getSessions($scope.profileId, $scope.profileType, 0, 10, "timestamp:desc").then(function (response) {
                    var data = response.data;
                    personaObject["sessions"] = data.list;
                    for (var i = 0; i < data.list.length; i++) {
                        var session = data.list[i];
                        //updating profile sessions with new values corresponding to the new persona
                        session.itemId = "MF_persona_session_" + new Date().getTime() + "_" + session.itemId;
                        session.profileId = personaObject.persona.itemId;
                        //Removing unnecessary data from session
                        delete session.itemType;
                        delete session.profile;
                        delete session.scope;
                        delete session.size;
                        delete session.systemProperties;
                    }
                    //Saving the persona with sessions
                    unomiProfileService.savePersonaWithSessions(personaObject).then(function (response) {
                        loadingSpinnerService.hide();
                        notificationService.successToast(i18nService.message('serverSettings.manageProfiles.savedAsPersona'));
                    });
                });
            });
        };

        $scope.timelineColours = [
            {
                backgroundColor: "rgba(220,220,220,0.2)",
                borderColor: "rgba(220,220,220,1)",
                pointBackgroundColor: "rgba(220,220,220,1)",
                pointBorderColor: "#fff",
                pointHoverBackgroundColor: "#fff",
                pointHoverBorderColor: "rgba(220,220,220,1)"
            },
            {
                backgroundColor: "rgba(132, 84, 198, 0.2)",
                borderColor: "#8454c6",
                pointBackgroundColor: "#8454c6",
                pointBorderColor: "#fff",
                pointHoverBackgroundColor: "#fff",
                pointHoverBorderColor: "#8454c6"
            }
        ];
        $scope.timelineData = {
            toDate: new Date(),
            fromDate: moment().subtract(1, "weeks").toDate(),
            interval: ManagersUtils.supportedTimelineIntervalesObj["1d"],
            series: [i18nService.message('serverSettings.manageProfiles.pageViews'), i18nService.message('serverSettings.manageProfiles.sessions')],
            loadData: function (success) {
                if ($scope.profileId) {
                    var aggregateSessionQuery = {
                        "aggregate": {
                            "type": "date",
                            "parameters": {
                                "interval": this.interval.key,
                                "format": "yyyy-MM-dd"
                            }
                        },
                        "condition": {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "profileId",
                                "propertyValue": $scope.profileId,
                                "comparisonOperator": "equals"
                            }
                        }
                    };
                    var aggregatePageViewsQuery = {
                        "aggregate": {
                            "type": "date",
                            "parameters": {
                                "interval": this.interval.key,
                                "format": "yyyy-MM-dd"
                            }
                        },
                        "condition": {
                            "type": "booleanCondition",
                            "parameterValues": {
                                "operator": "and",
                                "subConditions": [
                                    {
                                        "type": "eventTypeCondition",
                                        "parameterValues": {
                                            "eventTypeId": "view"
                                        }
                                    },
                                    {
                                        "type": "sessionPropertyCondition",
                                        "parameterValues": {
                                            "propertyName": "profileId",
                                            "propertyValue": $scope.profileId,
                                            "comparisonOperator": "equals"
                                        }
                                    }
                                ]
                            }
                        }
                    };
                    // get all sessions
                    unomiQueryService.query("event", "timeStamp", aggregatePageViewsQuery).then(function (response) {
                        var datasets = [];
                        datasets[0] = response.data;
                        unomiQueryService.query("session", "timeStamp", aggregateSessionQuery).then(function (response) {
                            datasets[1] = response.data;
                            success(datasets);
                        });
                    });
                }
            }
        };

        if ($scope.currentProfile.properties) {
            var interests = _.map($scope.currentProfile.properties.interests, function (num, key) {
                return {name: key, value: num};
            });
            interests = _.sortBy(interests, 'value');
            interests = _.last(interests, ManagersUtils.maxInterestsInChart);
            interests = _.sortBy(interests, 'name');

            $scope.interestsData = [_.pluck(interests, 'value')];
            $scope.interestsLabels = _.pluck(interests, 'name');
            $scope.interestsOptions = {
                responsive: true
            };
        }
        $scope.interestsColors = [
            {
                backgroundColor: "#F1DE30",
                borderColor: "rgba(51,153,255,0)",
                pointBackgroundColor: "rgba(51,153,255,0)",
                pointHoverBackgroundColor: "#fff",
                pointHoverBorderColor: "rgba(51,153,255,0)"
            }
        ];
        if ($scope.profileId) {
            var aggregateSessionQuery = {
                "aggregate": {
                    "type": "string"
                },
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "profileId",
                        "propertyValue": $scope.profileId,
                        "comparisonOperator": "equals"
                    }
                }
            };
            unomiQueryService.query("session", "properties.deviceCategory", aggregateSessionQuery).then(function (response) {
                var aggregateDeviceData = response.data;
                var desktop = aggregateDeviceData["Personal computer"] ? aggregateDeviceData["Personal computer"] : 0;
                var mobile = aggregateDeviceData["Smartphone"] ? aggregateDeviceData["Smartphone"] : 0;
                var tablet = aggregateDeviceData["Tablet"] ? aggregateDeviceData["Tablet"] : 0;
                var total = aggregateDeviceData._filtered;
                $scope.desktopUsage = (desktop * 100 / total).toFixed(1) + "%";
                $scope.mobileUsage = (mobile * 100 / total).toFixed(1) + "%";
                $scope.tabletUsage = (tablet * 100 / total).toFixed(1) + "%";
                $scope.othersUsage = ((total - desktop - mobile - tablet) * 100 / total).toFixed(1) + "%";
            });

            unomiQueryService.query("session", "properties.userAgentNameAndVersion", aggregateSessionQuery).then(function (response) {
                var aggregateAgentData = response.data;
                var userAgents = [];
                for (var key in aggregateAgentData) {
                    if (key.indexOf('_') != 0) {
                        userAgents.push({name: key, value: aggregateAgentData[key]});
                    }
                }
                var maxUserAgent = _.max(userAgents, function (userAgent) {
                    return userAgent.value;
                });

                if (maxUserAgent.name) {
                    var userAgentNameAndVersion = maxUserAgent.name.split("@@");
                    $scope.highestTraficBrowser = userAgentNameAndVersion[0] + " " + userAgentNameAndVersion[1];
                    $scope.highestTraficBrowserImage = $scope.getCurrentModuleResource('/images/browsers/' + userAgentNameAndVersion[0].toLowerCase().replace(" ", "-") + '.png');
                }
                $scope.highestTraficBrowserUsage = (maxUserAgent.value * 100 / aggregateAgentData._filtered).toFixed(1) + "%";
            });

            unomiQueryService.query("session", "properties.countryAndCity", aggregateSessionQuery).then(function (response) {
                var aggregateLocationData = response.data;
                var locations = [];
                for (var key in aggregateLocationData) {
                    if (key.indexOf('_') != 0) {
                        locations.push({name: key, value: aggregateLocationData[key]});
                    }
                }
                var maxLocation = _.max(locations, function (location) {
                    return location.value;
                });

                if (maxLocation.name) {
                    var countryAndCity = maxLocation.name.split("@@");
                    $scope.userLocationCountry = countryAndCity[0] == 'null' ? i18nService.message("serverSettings.manageProfiles.location.unknown") : countryAndCity[0];
                    $scope.userLocationCity = countryAndCity[1] == 'null' ? i18nService.message("serverSettings.manageProfiles.location.unknown") : countryAndCity[1];
                    $scope.userLocationState = countryAndCity[2] != 'null' ? countryAndCity[2] : '';
                    $scope.userLocationSubState = countryAndCity[3] != 'null' ? countryAndCity[3] : '';
                }
            });
        }
        $scope.segmentsIndexedById = {};
        unomiSegmentService.getAll().then(function (response) {
            var segments = response.data;
            var segmentsFiltered = _.reject(segments, function (segment) {
               return segment.scope != 'systemscope' && segment.scope != ManagersContext.dmfScope;
            });
            $scope.segmentsIndexedById = _.indexBy(segments, 'id');
            $scope.segmentsFiltered = segmentsFiltered;
        });
        $scope.getSegmentById = function (segmentId) {
            return $scope.segmentsIndexedById[segmentId];
        };
        $scope.getScopeLabelById = function (id, type) {
            var collection = $scope.segmentsIndexedById;
            if(type === 'list'){
                collection = $scope.profileListsIndexedById;
            }
            if(!_.isEmpty(collection)) {
                var currentScope = collection[id].scope;
                if (currentScope != ManagersContext.dmfScope) {
                    var stringToReturn = ' (';
                    if (currentScope === 'systemscope') {
                        stringToReturn += i18nService.message("serverSettings.manageSegments.shared");
                    } else {
                        stringToReturn += currentScope;
                    }
                    stringToReturn += ')';
                    return stringToReturn
                } else {
                    return '';
                }
            }
        };
        $scope.goalsIndexedById = {};
        unomiGoalService.getAll().then(function (response) {
            $scope.goalsIndexedById = _.indexBy(response.data, 'id');
        });
        $scope.getGoalById = function (goalId) {
            return $scope.goalsIndexedById[goalId];
        };
        $scope.getGoalClass = function(scope){
            return ManagersContext.dmfScope == scope?"in-scope-goal":"out-scope-goal";
        };
        $scope.isProfileScoreNotified = function (scoringId) {
            return $scope.currentProfile.systemProperties.notificationAck &&
                $scope.currentProfile.systemProperties.notificationAck.scoring &&
                $scope.currentProfile.systemProperties.notificationAck.scoring[scoringId] &&
                $scope.currentProfile.systemProperties.notificationAck.scoring[scoringId] > 0;
        };

        $scope.scoringsIndexedById = {};
        unomiScoringService.getAll().then(function (response) {
            var scorings = response.data;
            var scoringsFiltered = _.reject(scorings, function (scoring) {
                return scoring.scope != 'systemscope' && scoring.scope != ManagersContext.dmfScope;
            });
            $scope.scoringsIndexedById = _.indexBy(scorings, 'id');
            $scope.scoringsFiltered = scoringsFiltered;
        });
        $scope.getScoringById = function (scoringId) {
            return $scope.scoringsIndexedById[scoringId];
        };

        $scope.openListsModal = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/add-to-lists-modal.html',
                controller: 'AddToListsModalController',
                size: 'sm',
                resolve: {
                    "lists": function () {
                        return $scope.userLists;
                    },
                    "selectedProfile": function () {
                        return angular.copy($scope.currentProfile);
                    },
                    "profileType" : function () {
                        return $scope.profileType;
                    }
                }
            });
            modalInstance.result.then(function (profile) {
                $scope.currentProfile.systemProperties.lists = profile.systemProperties.lists;
            });
        };

        $scope.openScoreModifiersModal = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/add-score-modifiers-modal.html',
                controller: 'AddScoreModifiersModalController',
                resolve: {
                    "scorings": function () {
                        return $scope.scoringsFiltered;
                    },
                    "selectedProfile": function () {
                        return angular.copy($scope.currentProfile);
                    },
                    "profileType" : function (){
                        return $scope.profileType;
                    }
                }
            });
            modalInstance.result.then(function (profile) {
                $scope.resetProfile();
            });
        };

        $scope.openSegmentsModal = function () {
            var modalInstance = $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/add-to-segments-modal.html',
                controller: 'AddToSegmentsModalController',
                resolve: {
                    "segments": function () {
                        return $scope.segmentsFiltered;
                    },
                    "selectedProfile": function () {
                        return angular.copy($scope.currentProfile);
                    },
                    "profileType" : function (){
                        return $scope.profileType;
                    }
                }
            });
            modalInstance.result.then(function (profile) {
                $scope.currentProfile.segments = profile.segments;
            });
        };

        $scope.openAddPropertyModal = function (tagId) {
            $uibModal.open({
                templateUrl: ManagersContext.wemAppPath + '/wem-profiles/add-property-modal.html',
                controller: 'AddPropertyModalController',
                resolve: {
                    propertiesByTag: function () {
                        return $scope.propertiesByTag;
                    },
                    "tagId": function () {
                        return tagId;
                    }
                }
            });
        };

        $scope.confirmDeleteProperty = function (propertyId) {
            modalService.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.format("serverSettings.manageProfiles.property.delete.confirm", propertyId)
            }, function () {
                unomiProfileService.deletePropertyType(propertyId).then(function () {
                    $scope.currentProfile.properties[propertyId] = null;
                    for (var tag in $scope.propertiesByTag) {
                        if ($scope.propertiesByTag.hasOwnProperty(tag)) {
                            $scope.propertiesByTag[tag] = _.filter($scope.propertiesByTag[tag], function (prop) {
                                return prop.itemId != propertyId;
                            });
                        }
                    }
                    notificationService.successToast(i18nService.format("serverSettings.manageProfiles.property.delete.success", propertyId));
                });
            });
        };

        $scope.cardsEdition = [];
        $scope.toggleEdition = function (tag) {
            $scope.cardsEdition[tag] = $scope.cardsEdition[tag] ? !$scope.cardsEdition[tag] : true;
        };

        $scope.isEditionEnabled = function (tag) {
            return $scope.cardsEdition[tag];
        };

        // lists
        $scope.profileListsIndexedById = {};
        $scope.userLists = [];
        $scope.profileLists = [];
        $scope.getListById = function (listId) {
            return $scope.profileListsIndexedById[listId];
        };
        var queryListWithoutScope = {
            "condition": {
                "parameterValues": {
                    "propertyName": "metadata.scope",
                    "comparisonOperator": "exists"
                },
                "type": "sessionPropertyCondition"
            },
            "limit": 50,
            "offset": 0,
            "sortby": null,
            "forceRefresh": false
        };

        unomiListService.query(queryListWithoutScope).then(function (response) {
            var data = response.data;
            $scope.userLists = _.reject(data.list, function (entry) {
                return entry.scope != 'systemscope' && entry.scope != ManagersContext.dmfScope;
            });
            $scope.profileListsIndexedById = _.indexBy(data.list, 'id');
        });

        $scope.dtOptions = DTOptionsBuilder.newOptions()
            .withOption('processing', true)
            .withOption('serverSide', true)
            .withOption('sDom', 't')
            .withOption('createdRow', createdRow)
            .withOption('pageLength', 6)
            .withFnServerData(serverData)
            .withPaginationType('full_numbers')
            .withOption("order", [[0, 'desc']]);

        function createdRow(row, data, dataIndex) {
            console.log("Row is created: " + row);
            // Recompiling so we can bind Angular directive to the DT
            $compile(angular.element(row).contents())($scope);
        }

        $scope.existingSessionPropertyTypes = existingSessionPropertyTypesResource.get({}, function (sessionPropertyTypes) {
            console.log("manage-profile.controller.js: " + sessionPropertyTypes.length + " propertyTypes loaded.");

            $scope.dtColumns = [
                DTColumnBuilder.newColumn('timeStamp').withTitle(i18nService.message('session.timeStamp')).renderWith(function (data, type, full, meta) {
                    return "<a href=\"\" ng-click=\"goToSessionEdit('" + full.id + "')\" class=\"userID\">" + moment(data).format('LL') + "</a>";
                }),
                DTColumnBuilder.newColumn('duration').withTitle(i18nService.message('session.duration')).renderWith(function (data, type, full, meta) {
                    return moment.duration(data).humanize();
                }),
                DTColumnBuilder.newColumn('size').withTitle(i18nService.message('session.size'))
            ];

            if (_.some(sessionPropertyTypes, function (propertyType) {
                    return propertyType.itemId == "countryName";
                })) {
                $scope.dtColumns.push(DTColumnBuilder.newColumn('countryName').withTitle(i18nService.message('sessionsProperty.countryName')));
            }

            $scope.loading = false;
        });

        $scope.isObject = function (property) {
            return angular.isObject(property);
        };

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                if (prop == "countryName") {
                    prop = "properties." + prop;
                }
                return prop + ":" + o.dir
            }).join(",");

            var config = {
                url: ManagersContext.proxyServletUrl + "/cxs/profiles/" + $scope.profileId + "/sessions",
                method: 'get',
                params: {
                    offset: start,
                    size: length,
                    sort: order
                }
            };

            $http(config)
                .then(function (response) {
                    var data = response.data;
                    console.log("manage-profile.controller.js: " + data.list.length + " sessions loaded out of a total " + data.totalSize + " sessions.");
                    // copy the session information to a format that can be used by the table component we use
                    var sessionTable = [];
                    for (var i = 0; i < data.list.length; i++) {
                        var sessionTableRow = {};
                        var session = data.list[i];
                        sessionTableRow.id = session.itemId;
                        sessionTableRow.timeStamp = new Date(session.timeStamp);
                        sessionTableRow.duration = session.duration;
                        sessionTableRow.size = session.size;
                        sessionTableRow.countryName = session.properties.countryName ? session.properties.countryName : '';
                        sessionTable.push(sessionTableRow);
                    }

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": data.totalSize,
                        "iTotalDisplayRecords": data.totalSize,
                        "aaData": sessionTable
                    };
                    fnCallback(results);
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
        }

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
        };

        $scope.getTabTitle = function (key) {
            return i18nService.message(key);
        };

        $scope.getPreviousPage = function () {
            // otherwise if we have search id, route to the search
            if ($routeParams.searchId) {
                // if we have a profile and session id in the route, route back to the profile view
                return '#!/search/' + $routeParams.searchId;
            } else if ($routeParams.segmentId) {
                return '#!/segment/' + $routeParams.segmentId + '/profiles';
            } else if ($routeParams.campaignId) {
                var previousPage = "#!/campaign/" + $routeParams.campaignId;
                if ($routeParams.goalId) {
                    previousPage += "/converted/" + $routeParams.goalId;
                } else {
                    previousPage += "/profiles";
                }
                return previousPage;
            } else if ($routeParams.scoringId) {
                return '#!/scoring/' + $routeParams.scoringId + '/profiles';
            } else if ($routeParams.listId) {
                return '#!/list/' + $routeParams.listId + '/profiles';
            } else if ($scope.profileType == 'personas') {
                return '#!/personas';
            } else if ($routeParams.queryID) {
                return '#!/incoming-search/' + $routeParams.queryID;
            } else if ($routeParams.referrerId) {
                return '#!/referrer/' + $routeParams.referrerId;
            }

            return '#!/profiles';
        };

        $scope.goToSessionEdit = function (sessionId) {
            var route;
            if ($routeParams.searchId) {
                // if we have a profile and session id in the route, route back to the profile view
                route = '/search/' + $routeParams.searchId;
            } else if ($routeParams.segmentId) {
                route = '/segment/' + $routeParams.segmentId;
            } else if ($routeParams.campaignId != undefined) {
                route = '/campaign/' + $routeParams.campaignId;
                if ($routeParams.goalId) {
                    route += "/converted/" + $routeParams.goalId;
                }
            } else if ($routeParams.scoringId) {
                route = '/scoring/' + $routeParams.scoringId;
            } else if ($routeParams.listId) {
                route = '/list/' + $routeParams.listId;
            } else if ($routeParams.queryID) {
                route = '/incoming-search/' + $routeParams.queryID;
            } else if ($routeParams.referrerId) {
                route = '/referrer/' + $routeParams.referrerId;
            } else {
                route = '';
            }

            $location.path(route + '/profile/' + $routeParams.profileId + '/' + sessionId);
        };
        $scope.goals = [];
        if ($scope.currentProfile.systemProperties) {
            angular.forEach($scope.currentProfile.systemProperties.goals, function (v, k) {
                if (k.substring(k.length - "TargetReached".length) == "TargetReached") {
                    var goalId = k.substring(0, k.length - "TargetReached".length);
                    unomiGoalService.getGoal(goalId).then(function(response){
                        var currentGoal = response.data.metadata;
                        if (currentGoal) {
                            if($scope.sites[currentGoal.scope] === undefined){
                                jcrService.doGetOnPath("default",ManagersContext.mainResourceLocale,"/sites/"+currentGoal.scope+"/properties/j__title").then(function(response) {
                                    var jcrResponse = response.data;
                                    console.log("manage-profile.controller.js: Get site "+currentGoal.scope+" in jcr");
                                    $scope.sites[currentGoal.scope] = jcrResponse.value;
                                    currentGoal['sitename'] = jcrResponse.value;
                                    $scope.goals.push({
                                        id: goalId,
                                        date: moment(v),
                                        name: currentGoal.name,
                                        scope: currentGoal.scope,
                                        applyon: "( "+i18nService.message("serverSettings.manageProfiles.site")+": "+currentGoal.sitename+" )"
                                    })
                                },
                                function(){
                                    $scope.goals.push({
                                        id: goalId,
                                        date: moment(v),
                                        name: currentGoal.name,
                                        scope: currentGoal.scope,
                                        applyon: "( "+i18nService.message("serverSettings.manageProfiles.scope")+": "+currentGoal.scope+" )"
                                    });
                                });
                            } else {
                                $scope.goals.push({
                                    id: goalId,
                                    date: moment(v),
                                    name: currentGoal.name,
                                    scope: currentGoal.scope,
                                    applyon: "( "+i18nService.message("serverSettings.manageProfiles.site")+": "+$scope.sites[currentGoal.scope]+" )"
                                });
                            }
                        }
                    });
                }
            });
        }
        $scope.goals = _.sortBy($scope.goals, function (goal) {
            return moment(goal.date);
        }).reverse();
    }
})();