(function() {
    'use strict';

    angular.module('wemReferrersApp').controller('ReferrerDetailsController', ReferrerDetailsController);

    ReferrerDetailsController.$inject = ['$scope', '$routeParams', 'referrerService', 'i18nService', '$location', '$compile', 'dataTableService', 'dtPagerService'];

    function ReferrerDetailsController($scope, $routeParams, referrerService, i18nService, $location, $compile, dataTableService, dtPagerService) {
        $scope.referrerHost = $routeParams.referrerId;

        var columnsPlaceHolder = function (data, type, full, meta) {
            if (!data || data == '') {
                return i18nService.message('serverSettings.internalSearches.missing');
            }
            if (meta.col == 1) {
                return moment(data).locale(ManagersContext.jahiaUILocale).format('LLL');
            }
            if (meta.col == 4) {
                return i18nService.message('profilesProperty.gender.' + data);
            }
            return data;
        };

        var serverData = function (sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var offset = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
            }).join(",");

            referrerService.getReferrerDetails($scope.referrerHost, offset, length, order).then(function(response) {
                var data = response.data;
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": data.list
                };
                fnCallback(results);
                dtPagerService.refreshPager($scope);
            },
            function(response) {
                console.error(response);
                var results = {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
                fnCallback(results);
                dtPagerService.refreshPager($scope);
            });
        };

        $scope.dtOptions = dataTableService.generateDataTableOptions({
            pageLength: 25,
            processing: true,
            serverSide: true,
            sDom: 't',
            defaultOrder: [[1, 'desc']],
            rowCallback: function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $compile(nRow)($scope);
            }
        });

        $scope.dtOptions.withFnServerData(serverData);

        var columns = [
            {name: 'properties.referrerQuery', bundleKey: 'serverSettings.referrers.searchedTerms', actionsHtml: columnsPlaceHolder},
            {name: 'timeStamp', bundleKey: 'serverSettings.referrers.date', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.firstName', bundleKey: 'serverSettings.referrers.firstName', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.lastName', bundleKey: 'serverSettings.referrers.lastName', actionsHtml: columnsPlaceHolder},
            {name: 'profile.properties.gender', bundleKey: 'serverSettings.referrers.gender', actionsHtml: columnsPlaceHolder},
            {
                name: null, bundleKey: 'serverSettings.referrers.actions',
                actionsHtml: function(data, type, full, meta) {
                    return '<button type="button" class="btn btn-icon command-dashboard" ng-click="goToEditProfile(\'' + data.profileId + '\')" title="' + i18nService.message('serverSettings.incomingSearches.editProfile') + '"><i class="material-icons">person</i></button>';
                },
                notSortable: true,
                options: [
                    ["width", "90px"],
                    ["contentPadding", "0"]
                ]
            }
        ];
        $scope.dtColumns = dataTableService.generateDataTableColumns(columns);

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToEditProfile = function (profileId) {
            $location.path('/referrer/' + $scope.referrerHost + '/profile/' + profileId);
        };
    }
})();