(function() {
    'use strict';

    angular.module('wemReferrersApp').controller('ReferrersListController', ReferrersListController);

    ReferrersListController.$inject = ['$scope', 'referrerService', 'i18nService', 'uibDatepickerConfig', '$compile', 'dataTableService', '$location', 'dtPagerService'];

    function ReferrersListController($scope, referrerService, i18nService, uibDatepickerConfig, $compile, dataTableService, $location, dtPagerService) {
        $scope.referrerFrom = Date.now() + -30 * 24 * 3600 * 1000;
        $scope.referrerTo = Date.now();

        var init = function (from, to) {
            var promise = referrerService.getReferrers(from, to);
            dataTableService.setPromise(promise);
            $scope.dtOptions = dataTableService.generateDataTableOptions({
                defaultOrder: [[2, 'desc']],
                rowCallback: function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    $compile(nRow)($scope);
                }
            });

            var columns = [
                {name: 'host', bundleKey: 'serverSettings.referrers'},
                {name: 'occurrences', bundleKey: 'serverSettings.referrers.occurrences'},
                {name: 'importance', bundleKey: 'serverSettings.referrers.importance'},
                {
                    name: null, bundleKey: 'serverSettings.manageWem.listAction',
                    actionsHtml: function (data, type, full, meta) {
                        var html = "<button title=\"" + i18nService.message("serverSettings.referrers.referrerQueries") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"displayReferrerQueries(" + meta.row + ")\"><i class=\"material-icons\">search</i></button> ";
                        html +=  "<button title=\"" + i18nService.message("serverSettings.referrers.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><i class=\"material-icons\">list</i></button> ";
                        return html;
                    },
                    notSortable: true,
                    options: [
                        ["width", "180px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];
            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
            promise.then(function () {
                dtPagerService.refreshPager($scope);
            });
        };

        init($scope.referrerFrom, $scope.referrerTo);

        // date config
        uibDatepickerConfig.startingDay = 1;
        uibDatepickerConfig.minMode = "day";
        uibDatepickerConfig.datepickerMode = "day";
        uibDatepickerConfig.showWeeks = false;
        $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
        $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
        $scope.clearText = i18nService.message('wem.label.clear');
        $scope.datePickers = {};

        $scope.openDatePopup = function ($event, selector) {
            $event.preventDefault();
            $event.stopPropagation();

            angular.forEach($scope.datePickers, function (value, key) {
                $scope.datePickers[key] = false;
            });
            $scope.datePickers[selector] = true;
        };

        $scope.setDtInstance = function (instance) {
            $scope.dtInstance = instance;
            dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
        };

        $scope.goToDetails = function (index) {
            var referrer = $scope.dtInstance.DataTable.row(index).data().host;
            referrerService.setSelectedReferrer(referrer);
            $location.path('/referrer/' + referrer);
        };

        $scope.displayReferrerQueries = function (index) {
            var referrer = $scope.dtInstance.DataTable.row(index).data().host;
            referrerService.setSelectedReferrer(referrer);
            $location.path('/referrer/queries/' + referrer);
        };

        $scope.updateReferrersList = function () {
            init(new Date($scope.referrerFrom).getTime(), new Date($scope.referrerTo).getTime());
            $scope.dtInstance.DataTable.draw();
        };

        $scope.getMessage = function (key) {
            return i18nService.message(key);
        };

    }
})();