(function() {
    'use strict';

    angular.module('wemRulesApp').controller('RuleAdvancedEditController', RuleAdvancedEditController);

    RuleAdvancedEditController.$inject = ['$location', '$resource', '$q', '$timeout', '$routeParams',
                                            'unomiRulesService', 'unomiDefinitionService',
                                            'i18nService', 'modalService', 'rulesService', 'loadingSpinnerService'];

    function RuleAdvancedEditController($location, $resource, $q, $timeout, $routeParams,
                                        unomiRulesService, unomiDefinitionService,
                                        i18nService, modalService, rulesService, loadingSpinnerService) {

        var vm = this;

        vm.saveRule = saveRule;
        vm.deleteRule = deleteRule;

        _init();

        // public functions

        function saveRule($event) {
            if (!vm.advancedRuleForm.$valid) {
                modalService.newInfoModal({
                    title: i18nService.message("serverSettings.manageRules.formErrorTitle"),
                    bodyMessage: i18nService.message("serverSettings.manageRules.formErrorBody")
                }, function (modalData) {
                    console.warn("Form not submitted");
                });
                $event.preventDefault();
                return;
            }
            if (vm.sharedRule) {
                vm.currentRule.metadata.scope = 'systemscope';
            }
            loadingSpinnerService.show();
            unomiRulesService.setRule(vm.currentRule).then(function() {
                $timeout(function () {$location.path("/rules")}, 2000);
            });
        }

        function deleteRule(id) {
            modalService.newAlertModal({title: i18nService.message("serverSettings.manageRules.deleteRule"), bodyMessage: i18nService.message("serverSettings.manageRules.deleteRuleConfirmBody")}, function (modalData) {
                console.info("Delete confirmed, deleting rule", vm.currentRule);
                loadingSpinnerService.show();
                unomiRulesService.deleteRule(id).then(function(response) {
                    $timeout(function () {$location.path("/rules")}, 2000);
                });
            }, function (modalData) {
                console.info("Delete cancelled");
            });
        }

        // private functions

        function _init() {
            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions", {}, { 'get': {method:'GET', isArray : true  }});
            var actionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/actions", {}, { 'get': {method:'GET', isArray : true }});
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId : "profileTags" });

            vm.conditionTypes = [];
            vm.actionTypes = [];
            vm.tags = {};
            vm.isNewRule = $routeParams.ruleId == "newAdvancedRule";

            vm.currentRule = rulesService.loadOrCreateRule($routeParams.ruleId).then(function (data) {
                vm.currentRule = data;
                vm.sharedRule = vm.currentRule.metadata.scope != ManagersContext.dmfScope

                var profileTagsPromise = $q.defer();
                var conditionTypesPromise = $q.defer();
                var actionTypesPromise = $q.defer();

                vm.profileTags = tagResource.get({tagId : "profileTags"}, function(value, responseHeaders) {
                    console.info("rule-advanced-edit.controller.js : tags loaded from Apache Unomi.");

                    angular.forEach(value.subTags, function(v) { vm.tags[v.id] = v } );
                    profileTagsPromise.resolve(vm.tags);
                });

                vm.conditionTypes = conditionTypesResource.get(function (conditionTypes) {
                    console.info("rule-advanced-edit.controller.js: Loaded " + conditionTypes.length + " condition types");
                    conditionTypesPromise.resolve(conditionTypes);
                });

                vm.actionTypes = actionTypesResource.get(function (actionTypes) {
                    console.info("rule-advanced-edit.controller.js: Loaded " + actionTypes.length + " action types");
                    actionTypesPromise.resolve(actionTypes);
                });

                $q.all([profileTagsPromise.promise, conditionTypesPromise.promise, actionTypesPromise.promise]).then(function (data) {
                    console.info("Tags, conditions and action types were loaded sucessfully.");
                    console.debug("tags:", vm.profileTags.subTags.length, " conditionTypes:", vm.conditionTypes.length, " actionTypes:", vm.actionTypes.length);
                    unomiDefinitionService.resolveProfileTags(vm.conditionTypes, vm.tags);
                    unomiDefinitionService.resolveProfileTags(vm.actionTypes, vm.tags);
                });
            });

        }

    }
})();
