(function() {
    'use strict';

    angular.module('wemScoringApp').run(wemScoringAppRun);

    wemScoringAppRun.$inject = ['$rootScope', '$timeout', 'i18nService', 'modalService', 'unomiProfileService', 'unomiRulesService', 'unomiScoringService', 'loadingSpinnerService'];

    function wemScoringAppRun($rootScope, $timeout, i18nService, modalService, unomiProfileService, unomiRulesService, unomiScoringService, loadingSpinnerService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.tryDeleteItem = function (scoring, callback) {
            // if the scoring we're trying to delete is shared, display a modal to ask confirmation
            if (scoring.scope != ManagersContext.dmfScope) {
                modalService.newAlertModal({
                    title: i18nService.format("serverSettings.manageScoring.delete.sharedWarning.title", scoring.name),
                    bodyMessage: i18nService.message("serverSettings.manageScoring.delete.sharedWarning.message")
                }, function () {
                    deleteScoring(scoring, callback);
                });
            } else {
                modalService.newAlertModal({
                    title: i18nService.format("serverSettings.manageScoring.delete.title", scoring.name),
                    bodyMessage: i18nService.format("serverSettings.manageScoring.delete.message", scoring.name)
                }, function () {
                    deleteScoring(scoring, callback);
                });
            }
        };

        var deleteScoring = function(scoring, callback) {
            loadingSpinnerService.show();
            var update = unomiScoringService.buildUpdateBatchRemoveScoring(scoring.id);
            unomiProfileService.batchProfilesUpdate(update).then(function () {
                unomiScoringService.deleteScoring(scoring.id, true).then(function (response) {
                    var data = response.data;
                    loadingSpinnerService.hide();
                    unomiRulesService.deleteRule("sendEmailRulePrefix_" + scoring.id);
                    // if data isn't empty, it contains the list of the scoring that use the scoring we're trying to delete as a condition
                    if (data.segments.length > 0 || data.scorings.length > 0) {
                        // display the list of dependent scoring and ask what to do
                        modalService.newAlertModal({
                            title: i18nService.format("serverSettings.manageScoring.delete.dependentsModal.title", scoring.name),
                            bodyTemplateUrl: ManagersContext.wemAppPath + "/wem-scoring/scoring-delete-dependents-modal-body.html",
                            scoringName: scoring.name,
                            dependents: data
                        }, function () {
                            loadingSpinnerService.show();
                            unomiScoringService.deleteScoring(scoring.id, false).then(function (response) {
                                $timeout(function () {
                                    loadingSpinnerService.hide();
                                    if (callback) {
                                        callback();
                                    }
                                }, 1000);
                            });
                        });
                    } else {
                        $timeout(function () {
                            loadingSpinnerService.hide();
                            if (callback) {
                                callback();
                            }
                        }, 1000);
                    }
                });
            },
            function () {
                console.error("not updated. Failure!");
                $timeout(function () {
                    loadingSpinnerService.hide();
                    if (callback) {
                        callback();
                    }
                }, 1000);
            });
        };
    }
})();
