(function() {
    'use strict';

    angular.module('wemSegmentsApp').controller('SegmentsListController', SegmentsListController);

    SegmentsListController.$inject = ['$scope', 'unomiSegmentService', 'unomiQueryService', 'unomiListService', '$location', 'DTColumnBuilder', 'i18nService', '$compile', 'dataTableService', 'dtPagerService'];

    function SegmentsListController($scope, unomiSegmentService, unomiQueryService, unomiListService, $location, DTColumnBuilder, i18nService, $compile, dataTableService, dtPagerService) {
        $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
        $scope.items = {};
        $scope.stopPolling = true;

        function serverData(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return x.name == "draw"
            }).value;

            var start = _.find(aoData, function (x) {
                return x.name == "start"
            }).value;

            var length = _.find(aoData, function (x) {
                return x.name == "length"
            }).value;

            var order = _.map(_.find(aoData, function (x) {
                return x.name == "order"
            }).value, function (o) {
                return "metadata." + ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
            }).join(",");

            if($scope.isStaticList){
                unomiListService.query(unomiListService.constructQuery(length, start, order)).then(function (response) {
                    fnCallback(getResult(draw, response.data, unomiListService));
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    fnCallback(getErrorResult());
                });
            } else {
                unomiSegmentService.query(unomiSegmentService.constructQuery(length, start, order)).then(function (response) {
                    fnCallback(getResult(draw, response.data, unomiSegmentService));
                    dtPagerService.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                },
                function () {
                    fnCallback(getErrorResult());
                });
            }
        }

        var getResult = function(draw, data, service) {
            angular.forEach(data.list, processItem);

            $scope.stopPolling = false;

            return {
                "sEcho": draw,
                "iTotalRecords": data.totalSize,
                "iTotalDisplayRecords": data.totalSize,
                "aaData": data.list
            };
        };

        var getErrorResult = function (draw) {
            $scope.searchError = true;
            return {
                "sEcho": draw,
                "iTotalRecords": 0,
                "iTotalDisplayRecords": 0,
                "aaData": []
            };
        };

        $scope.dtOptions = dataTableService.generateDataTableOptions({
            pageLength: 25,
            lengthMenu: [ 25, 50, 100 ],
            processing: true,
            serverSide: true,
            sDom: 'lt',
            aaSorting: [],
            createdRow: function (row, data, dataIndex) {
                console.log("Row is created: "+row);
                // Recompiling so we can bind Angular directive to the DT
                $compile(angular.element(row).contents())($scope);
            }
        });

        $scope.dtOptions.withFnServerData(serverData);

        $scope.dtColumns = [
            DTColumnBuilder
                .newColumn("name")
                .withTitle(i18nService.message($scope.isStaticList ? "serverSettings.manageLists.list.name" : "serverSettings.manageSegments.segmentName"))
                .withOption("width","35%")
                .renderWith(renderNameCol),

            DTColumnBuilder.newColumn("description")
                .withTitle(i18nService.message("serverSettings.manageWem.description"))
                .withOption("width","50%"),

            DTColumnBuilder.newColumn(null)
                .withTitle(i18nService.message("serverSettings.manageWem.listAction"))
                .notSortable()
                .renderWith(actionsHtml)
                .withOption("width","15%")
                .withOption("contentPadding","0")
        ];

        $scope.setDtInstance = function(instance) {
            $scope.dtInstance = instance;
        };

        function actionsHtml(data, type, full, meta) {

            var actions = "<button title=\"" + i18nService.message($scope.isStaticList ? "serverSettings.manageSegments.editList" : "serverSettings.manageSegments.editSegment") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><i class=\"material-icons\">mode_edit</i></button> " +
                "<button title=\"" + i18nService.message("serverSettings.manageSegments.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><i class=\"material-icons\">person</i></button>";
            if( !_.contains(['contacts', 'leads'], data.id)){
                actions +=  "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><i class=\"material-icons\">delete</i></button>";
            }
            return actions;
        }

        $scope.goToEdit = function(id) {
            $scope.stopPolling = true;
            $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id );
        };

        $scope.goToProfiles = function(id) {
            $scope.stopPolling = true;
            $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id + "/profiles" );
        };

        $scope.goToNewSegment = function() {
            $scope.stopPolling = true;
            $location.path( "/newSegment");
        };

        $scope.goToNewList = function() {
            $scope.stopPolling = true;
            $location.path( "/newList");
        };

        function renderNameCol(data, type, full, meta) {
            var result = data + ' <span class="badge deep-purple-bg">{{items["' + full.id + '"].count}}</span>';
            if(full.isShared) {
                result += ' <span class="badge  mf-pink-bg">' + i18nService.message("serverSettings.manageSegments.shared") + '</span>'
            }
            if(!full.enabled) {
                result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
            }
            return result;
        }

        function getCount (segmentId) {
            var segmentCondition = {
                type:"booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "profilePropertyCondition",

                            parameterValues: {
                                propertyName: $scope.isStaticList ? "systemProperties.lists" : "segments",
                                comparisonOperator: "equals",
                                propertyValue: segmentId
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "mergedWith",
                                comparisonOperator: "missing"
                            }
                        },
                        {
                            type: "profilePropertyCondition",
                            parameterValues: {
                                propertyName: "systemProperties.isAnonymousProfile",
                                comparisonOperator: "missing"
                            }
                        }],
                    operator: "and"
                }
            };

            var observable = Rx.Observable.create(function(o) {
                unomiQueryService.getMatchingItemsCountForCondition('profile', segmentCondition)
                .then(function(response) {
                    o.onNext(response.data);
                    o.onCompleted();
                })
            });
            return observable.merge(
                Rx.Observable
                    .interval(5000)
                    .flatMapLatest(observable)
                    .takeWhile(function (x) { return !$scope.stopPolling; })
            );
        }


        var processItem = function (itemMetadata) {
            $scope.items[itemMetadata.id] = itemMetadata;
            itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
            $scope.items[itemMetadata.id].observable = getCount(itemMetadata.id).distinctUntilChanged().subscribe(function(count){
                $scope.items[itemMetadata.id].count = count;
            });
        };

        $scope.deleteItem = function (id) {
            $scope.tryDeleteItem($scope.items[id], $scope.isStaticList, function() {
                $scope.dtInstance.DataTable.draw();
            }, function() {
                unomiSegmentService.deletedItem = id;
                $scope.dtInstance.DataTable.draw();
            });
        };
    }
})();
